"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Common
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UiComponents = void 0;
const immer_1 = require("immer");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
/**
 * Manages the I18N service for the ui-components package.
 * @public
 */
class UiComponents {
    /**
     * Registers the I18N service namespace for UiComponents. Also initializes UiCore.
     * @param i18n The internationalization service created by the application. Defaults to IModelApp.i18n.
     */
    static async initialize(i18n) {
        if (UiComponents._initialized) {
            bentleyjs_core_1.Logger.logInfo(UiComponents.loggerCategory(UiComponents), `UiComponents.initialize already called`);
            return;
        }
        immer_1.enablePatches();
        UiComponents._i18n = i18n || imodeljs_frontend_1.IModelApp.i18n;
        await UiComponents._i18n.registerNamespace(UiComponents.i18nNamespace).readFinished;
        await ui_core_1.UiCore.initialize(UiComponents._i18n);
        UiComponents._initialized = true;
    }
    /** Unregisters the UiComponents I18N namespace */
    static terminate() {
        if (UiComponents._i18n)
            UiComponents._i18n.unregisterNamespace(UiComponents.i18nNamespace);
        UiComponents._i18n = undefined;
        ui_core_1.UiCore.terminate();
        UiComponents._initialized = false;
    }
    /** Determines if UiComponents has been initialized */
    static get initialized() { return UiComponents._initialized; }
    /** The internationalization service created by the application. */
    static get i18n() {
        if (!UiComponents._i18n)
            throw new ui_abstract_1.UiError(UiComponents.loggerCategory(this), "i18n: UiComponents.initialize has not been called. Unable to return I18N object.");
        return UiComponents._i18n;
    }
    /** The internationalization service namespace. */
    static get i18nNamespace() {
        return "UiComponents";
    }
    /** @internal */
    static get packageName() {
        return "ui-components";
    }
    /** Calls i18n.translateWithNamespace with the "UiComponents" namespace. Do NOT include the namespace in the key.
     * @internal
     */
    static translate(key) {
        if (!UiComponents.initialized) {
            bentleyjs_core_1.Logger.logError(UiComponents.loggerCategory(this), `translate: UiComponents.initialize has not been called. Returning blank string.`);
            return "";
        }
        return UiComponents.i18n.translateWithNamespace(UiComponents.i18nNamespace, key);
    }
    /** @internal */
    static loggerCategory(obj) {
        const className = ui_abstract_1.getClassName(obj);
        const category = UiComponents.packageName + (className ? `.${className}` : "");
        return category;
    }
}
exports.UiComponents = UiComponents;
UiComponents._initialized = false;
//# sourceMappingURL=UiComponents.js.map