"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BreadcrumbTreeUtils = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Breadcrumb
 */
const ui_abstract_1 = require("@bentley/ui-abstract");
const getPropertyRecordAsString_1 = require("../common/getPropertyRecordAsString");
const TableDataProvider_1 = require("../table/TableDataProvider");
const TreeDataProvider_1 = require("../tree/TreeDataProvider");
const UiComponents_1 = require("../UiComponents");
/* eslint-disable deprecation/deprecation */
/**
 * Utility class for tree searching and manipulation in the Breadcrumb component.
 * @beta
 */
class BreadcrumbTreeUtils {
    static createIcon(node) {
        return {
            key: "icon", record: new ui_abstract_1.PropertyRecord({
                value: TreeDataProvider_1.hasChildren(node) ? "icon-folder" : node.icon,
                valueFormat: ui_abstract_1.PropertyValueFormat.Primitive,
                displayValue: TreeDataProvider_1.hasChildren(node) ? "icon-folder" : node.icon,
            }, {
                name: "icon",
                displayLabel: UiComponents_1.UiComponents.translate("breadcrumb.icon"),
                typename: "icon",
            }),
        };
    }
    static createLabel(node) {
        return {
            key: "label",
            record: node.label,
        };
    }
    static createDescription(node) {
        return {
            key: "description",
            record: new ui_abstract_1.PropertyRecord({
                value: node.description || "",
                valueFormat: ui_abstract_1.PropertyValueFormat.Primitive,
                displayValue: node.description || "",
            }, {
                name: "description",
                displayLabel: UiComponents_1.UiComponents.translate("breadcrumb.description"),
                typename: "text",
            }),
        };
    }
    /**
     * Transforms a list of children from a tree node into a [[TableDataProvider]], given a list of column descriptions.
     * @param nodes Node list to use as a basis for the [[TableDataProvider]].
     * @param columns An array of column descriptions to specify which columns to provide to the resulting [[TableDataProvider]].
     * @returns A [[TableDataProvider]] object that can be used to populate a Table component.
     */
    static aliasNodeListToTableDataProvider(nodes, columns, treeDataProvider) {
        return {
            onColumnsChanged: new TableDataProvider_1.TableDataChangeEvent(),
            onRowsChanged: new TableDataProvider_1.TableDataChangeEvent(),
            getColumns: async () => columns,
            getRowsCount: async () => nodes.length,
            getRow: async (rowIndex, _unfiltered) => {
                if (rowIndex < 0 || rowIndex > nodes.length)
                    return { _node: {}, key: "", cells: [] };
                const node = nodes[rowIndex];
                const cells = [
                    BreadcrumbTreeUtils.createIcon(node),
                    BreadcrumbTreeUtils.createLabel(node),
                    BreadcrumbTreeUtils.createDescription(node),
                ];
                for (const key in node.extendedData) {
                    // only add string values to table cell list
                    if (node.extendedData.hasOwnProperty(key) && node.extendedData[key] instanceof ui_abstract_1.PropertyRecord) {
                        cells.push({
                            key,
                            record: node.extendedData[key],
                            style: node.style,
                        });
                    }
                    else if (node.extendedData.hasOwnProperty(key) && (typeof node.extendedData[key] === "string" || typeof node.extendedData[key] === "boolean" || typeof node.extendedData[key] === "number")) {
                        cells.push({
                            key,
                            record: new ui_abstract_1.PropertyRecord({
                                value: node.extendedData[key].toString(),
                                valueFormat: ui_abstract_1.PropertyValueFormat.Primitive,
                                displayValue: node.extendedData[key].toString(),
                            }, {
                                name: key,
                                displayLabel: key,
                                typename: "text",
                            }),
                            style: node.style,
                        });
                    }
                }
                node.extendedData = node.extendedData || {};
                node.extendedData.id = node.id;
                node.extendedData.label = getPropertyRecordAsString_1.getPropertyRecordAsString(node.label);
                node.extendedData.description = node.description;
                if (node.hasChildren !== undefined)
                    node.extendedData.hasChildren = node.hasChildren;
                if (node.children !== undefined)
                    node.extendedData.children = node.children;
                node.extendedData.dataProvider = treeDataProvider;
                node.extendedData.icon = node.icon;
                const row = {
                    _node: node,
                    key: node.id,
                    extendedData: node.extendedData,
                    isDisabled: false,
                    colorOverrides: node.style ? node.style.colorOverrides : undefined,
                    cells,
                };
                return row;
            },
            // TODO: implement sorting function
            sort: /* istanbul ignore next */ async () => { },
        };
    }
}
exports.BreadcrumbTreeUtils = BreadcrumbTreeUtils;
//# sourceMappingURL=BreadcrumbTreeUtils.js.map