"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Common
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.scheduleSubscription = exports.SubscriptionScheduler = void 0;
const BehaviorSubject_1 = require("rxjs/internal/BehaviorSubject");
const defer_1 = require("rxjs/internal/observable/defer");
const finalize_1 = require("rxjs/internal/operators/finalize");
const mergeMap_1 = require("rxjs/internal/operators/mergeMap");
const observeOn_1 = require("rxjs/internal/operators/observeOn");
const onErrorResumeNext_1 = require("rxjs/internal/operators/onErrorResumeNext");
const publish_1 = require("rxjs/internal/operators/publish");
const refCount_1 = require("rxjs/internal/operators/refCount");
const subscribeOn_1 = require("rxjs/internal/operators/subscribeOn");
const switchAll_1 = require("rxjs/internal/operators/switchAll");
const asap_1 = require("rxjs/internal/scheduler/asap");
const queue_1 = require("rxjs/internal/scheduler/queue");
const Subject_1 = require("rxjs/internal/Subject");
const MAX_CONCURRENT_SUBSCRIPTIONS = 1;
/**
 * Controls observable subscription order and timing.
 *
 * Scheduled observables are placed at the end of a queue. The scheduler always
 * subscribes to at most one observable from the queue and waits until observables
 * either complete or get unscheduled before removing them from the queue.
 *
 * To schedule an observable, subscribe to the result of `scheduleSubscription()` call.
 * @internal
 */
class SubscriptionScheduler {
    constructor() {
        // Maps source observables to scheduled observables.
        this._scheduledObservables = new Map();
        // Subject that does the actual subscription scheduling
        this._scheduler = new Subject_1.Subject();
        this._scheduler
            .pipe(
        // Process all observables one by one in sequence
        mergeMap_1.mergeMap((sourceObservable) => sourceObservable.pipe(
        // Connect the observable
        refCount_1.refCount(), 
        // Guard against stack overflow when a lot of observables are scheduled.
        // Without this operation `mergeMap` will process each observable
        // that is present in the pipeline recursively.
        observeOn_1.observeOn(queue_1.queueScheduler), 
        // Make sure that there is enough time to subscribe to a scheduled observable multiple times if needed
        subscribeOn_1.subscribeOn(asap_1.asapScheduler), 
        // On error, proceed to the next observable. Outside subscribers will still receive an error notification.
        onErrorResumeNext_1.onErrorResumeNext()), MAX_CONCURRENT_SUBSCRIPTIONS))
            // Start the scheduler and wait for observables to be scheduled
            .subscribe();
    }
    /**
     * Schedules `source` for subscription in the current scheduler.
     *
     * The actual scheduling is performed when the returned observable is subscribed to.
     * To cancel, simply remove all subscribers from the returned observable.
     *
     * @param source Input observable for which to schedule a subscription.
     * @returns Multicast observable which mirrors `source`.
     */
    scheduleSubscription(source) {
        // On subscription, determine which observable to return
        return defer_1.defer(() => {
            // If this observable has already been scheduled, return the scheduled observable
            let scheduledObservable = this._scheduledObservables.get(source);
            if (scheduledObservable) {
                return scheduledObservable;
            }
            // Create a subject which emits observable sources
            const sourceSubject = new BehaviorSubject_1.BehaviorSubject(source.pipe(
            // When source completes, allow scheduler to move on to the next observable
            finalize_1.finalize(() => {
                sourceSubject.complete();
            })));
            // Create an observable which will be passed to the scheduler
            const connectableObservable = publish_1.publish()(sourceSubject.pipe(
            // Unsubscribe from previous observable before subscribing to the next one
            switchAll_1.switchAll()));
            // Create an observable which will be cached and returned
            scheduledObservable = connectableObservable.pipe(finalize_1.finalize(() => {
                // Invalidate cached scheduled observable
                this._scheduledObservables.delete(source);
                // Allow scheduler to move on to the next observable
                sourceSubject.complete();
            }), 
            // Multicast values
            publish_1.publish(), 
            // Dispose the shared subscription when there are no subscribers left
            refCount_1.refCount());
            // Cache the scheduled observable
            this._scheduledObservables.set(source, scheduledObservable);
            // Schedule the subscription
            this._scheduler.next(connectableObservable);
            return scheduledObservable;
        });
    }
}
exports.SubscriptionScheduler = SubscriptionScheduler;
/**
 * Helper function for use as a `pipe()` argument with `rxjs` observables.
 * @internal
 */
function scheduleSubscription(scheduler) {
    return (source) => scheduler.scheduleSubscription(source);
}
exports.scheduleSubscription = scheduleSubscription;
//# sourceMappingURL=SubscriptionScheduler.js.map