"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDebouncedAsyncValue = void 0;
const react_1 = require("react");
const publish_1 = require("rxjs/internal/operators/publish");
const refCount_1 = require("rxjs/internal/operators/refCount");
const defer_1 = require("rxjs/internal/observable/defer");
const SubscriptionScheduler_1 = require("./SubscriptionScheduler");
/**
 * Custom hook for working with promise values.
 * Promises are debounced using trailing and leading edge.
 * The first promise will always be executed.
 * Any promises following the first while it is executing will be scheduled (and unscheduled), leaving only the last promise to be resolved.
 * Once the first promise finishes resolving, the last passed promise starts resolving.
 * @alpha
 */
function useDebouncedAsyncValue(valueToBeResolved) {
    const scheduler = react_1.useMemo(() => new SubscriptionScheduler_1.SubscriptionScheduler(), []); // eslint-disable-line react-hooks/exhaustive-deps
    const [value, setValue] = react_1.useState();
    const [inProgress, setInProgress] = react_1.useState(false);
    react_1.useEffect(() => {
        if (!valueToBeResolved) {
            setValue(undefined);
            setInProgress(false);
            return;
        }
        setInProgress(true);
        // schedule and subscribe to the observable emitting value from valueToBeResolved promise
        const subscription = defer_1.defer(valueToBeResolved)
            .pipe(publish_1.publish(), refCount_1.refCount(), SubscriptionScheduler_1.scheduleSubscription(scheduler))
            .subscribe({
            next: (data) => {
                setValue(data);
                setInProgress(false);
            },
        });
        return () => { subscription.unsubscribe(); };
    }, [valueToBeResolved, scheduler]);
    return { value, inProgress };
}
exports.useDebouncedAsyncValue = useDebouncedAsyncValue;
//# sourceMappingURL=UseDebouncedAsyncValue.js.map