"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectionHandler = exports.DragAction = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Common
 */
const geometry_core_1 = require("@bentley/geometry-core");
const SelectionModes_1 = require("./SelectionModes");
class BatchSelectionOperation {
    constructor(componentSelectionHandler, shouldReplace) {
        this.selections = new Array();
        this.deselections = new Array();
        this.shouldReplace = shouldReplace;
        this._componentSelectionHandler = componentSelectionHandler;
    }
    select(node) {
        this.addNodes(node, this.selections, this.shouldReplace ? undefined : this.deselections);
    }
    deselect(node) {
        this.addNodes(node, this.deselections, this.shouldReplace ? undefined : this.selections);
    }
    addNodes(node, addTo, removeFrom) {
        if (Array.isArray(node)) {
            for (const n of node) {
                if (removeFrom) {
                    const index = removeFrom.findIndex((x) => this._componentSelectionHandler.areEqual(x, n));
                    if (index > -1) {
                        removeFrom.splice(index, 1);
                        continue;
                    }
                }
                addTo.push(n);
            }
        }
        else {
            this.addNodes([node], addTo, removeFrom);
        }
    }
}
/** @internal */
class DragAction {
    constructor(componentSelectionHandler, itemSelectionHandlers, firstItem) {
        this._itemSelectionHandlers = itemSelectionHandlers;
        this._componentSelectionHandler = componentSelectionHandler;
        const itemPos = this.findItem(this._itemSelectionHandlers, firstItem);
        this._previousRow = itemPos.y;
        this._previousColumn = itemPos.x;
        this._firstItemRow = itemPos.y;
        this._firstItemColumn = itemPos.x;
        this._firstItemSelected = false;
    }
    updateDragAction(latestItem) {
        const currentPos = this.findItem(this._itemSelectionHandlers, latestItem);
        if (currentPos.y === this._previousRow && currentPos.x === this._previousColumn || currentPos.x < 0 || currentPos.y < 0)
            return { selections: [], deselections: [] };
        const currentRange = geometry_core_1.Range2d.createXYXY(this._firstItemColumn, this._firstItemRow, currentPos.x, currentPos.y);
        const previousRange = geometry_core_1.Range2d.createXYXY(this._firstItemColumn, this._firstItemRow, this._previousColumn, this._previousRow);
        const wholeRange = geometry_core_1.Range2d.createXYXY(currentRange.high.x > previousRange.high.x ? currentRange.high.x : previousRange.high.x, currentRange.high.y > previousRange.high.y ? currentRange.high.y : previousRange.high.y, currentRange.low.x < previousRange.low.x ? currentRange.low.x : previousRange.low.x, currentRange.low.y < previousRange.low.y ? currentRange.low.y : previousRange.low.y);
        const selections = [];
        const deselections = [];
        // have to select first item separately since it is will always be in both previous and current ranges.
        if (!this._firstItemSelected) {
            const handler = this._itemSelectionHandlers[this._firstItemRow][this._firstItemColumn];
            if (handler.isSelected())
                deselections.push(handler.item());
            else
                selections.push(handler.item());
            this._firstItemSelected = true;
        }
        for (let r = wholeRange.low.y; r <= wholeRange.high.y; r++) {
            for (let c = wholeRange.low.x; c <= wholeRange.high.x; c++) {
                const insidePrevious = previousRange.containsXY(c, r);
                const insideCurrent = currentRange.containsXY(c, r);
                // If item is in only one of the ranges that means it's selection needs to be toggled.
                if ((insidePrevious || insideCurrent) && insideCurrent !== insidePrevious) {
                    const itemHandler = this._itemSelectionHandlers[r][c];
                    if (itemHandler.isSelected())
                        deselections.push(itemHandler.item());
                    else
                        selections.push(itemHandler.item());
                }
            }
        }
        this._previousRow = currentPos.y;
        this._previousColumn = currentPos.x;
        return { selections, deselections };
    }
    findItem(itemSelectionHandlers, item) {
        if (this._previousRow !== undefined && this._previousRow !== -1 && this._previousColumn !== undefined && this._previousColumn !== -1 &&
            this._componentSelectionHandler.areEqual(itemSelectionHandlers[this._previousRow][this._previousColumn].item(), item)) {
            return { y: this._previousRow, x: this._previousColumn };
        }
        for (let row = 0; row < itemSelectionHandlers.length; row++) {
            for (let column = 0; column < itemSelectionHandlers[row].length; column++) {
                if (this._componentSelectionHandler.areEqual(itemSelectionHandlers[row][column].item(), item))
                    return { x: column, y: row };
            }
        }
        return { x: -1, y: -1 };
    }
}
exports.DragAction = DragAction;
/** @internal */
class SelectionHandler {
    constructor(selectionMode, onItemsSelectedCallback, onItemsDeselectedCallback) {
        this.selectionMode = selectionMode;
        this.onItemsSelectedCallback = onItemsSelectedCallback;
        this.onItemsDeselectedCallback = onItemsDeselectedCallback;
    }
    /** Get the onSelectionChange processed item */
    get processedItem() { return this._processedItem; }
    /** Creates a function that should be called when selection changes. */
    createSelectionFunction(componentHandler, itemHandler) {
        const onSelectionChange = (shiftDown, ctrlDown) => {
            this._processedItem = itemHandler.item();
            if (this.selectionMode === SelectionModes_1.SelectionMode.None) {
                return;
            }
            let operationCreated = false;
            let shiftSelected = false;
            if (!this._currentOperation) {
                // will replace selection if it is limited to one or keys are enabled but ctrl is not down
                const shouldReplace = (SelectionModes_1.hasSelectionModeFlag(this.selectionMode, SelectionModes_1.SelectionModeFlags.KeysEnabled) && !ctrlDown)
                    || SelectionModes_1.hasSelectionModeFlag(this.selectionMode, SelectionModes_1.SelectionModeFlags.SelectionLimitOne);
                this._currentOperation = new BatchSelectionOperation(componentHandler, shouldReplace);
                operationCreated = true;
            }
            if (SelectionModes_1.hasSelectionModeFlag(this.selectionMode, SelectionModes_1.SelectionModeFlags.KeysEnabled)) {
                if (!ctrlDown)
                    componentHandler.deselectAll();
                if (shiftDown && this._lastItem !== undefined) {
                    const selected = componentHandler.selectBetween(this._lastItem, itemHandler.item());
                    this._currentOperation.select(selected);
                    shiftSelected = true;
                }
            }
            if (!shiftSelected) {
                itemHandler.preselect();
                if (SelectionModes_1.hasSelectionModeFlag(this.selectionMode, SelectionModes_1.SelectionModeFlags.SelectionLimitOne)
                    && !(SelectionModes_1.hasSelectionModeFlag(this.selectionMode, SelectionModes_1.SelectionModeFlags.ToggleEnabled) && itemHandler.isSelected())) {
                    componentHandler.deselectAll();
                }
                if (itemHandler.isSelected()) {
                    itemHandler.deselect();
                    this._currentOperation.deselect(itemHandler.item());
                }
                else {
                    itemHandler.select();
                    this._currentOperation.select(itemHandler.item());
                }
                this._lastItem = itemHandler.item();
            }
            if (operationCreated)
                this.completeOperation();
        };
        return onSelectionChange.bind(this);
    }
    completeOperation() {
        if (!this._currentOperation)
            return;
        if (0 !== this._currentOperation.selections.length) {
            if (this.onItemsSelectedCallback) {
                this.onItemsSelectedCallback(this._currentOperation.selections, this._currentOperation.shouldReplace);
                if (this._currentOperation.shouldReplace) {
                    this._currentOperation = undefined;
                    return;
                }
            }
        }
        if (0 !== this._currentOperation.deselections.length) {
            if (this.onItemsDeselectedCallback)
                this.onItemsDeselectedCallback(this._currentOperation.deselections);
        }
        this._currentOperation = undefined;
    }
    /**
     * Creates drag action.
     * @param componentSelectionHandler Component selection handler.
     * @param items Ordered item selection handlers separated into arrays by rows.
     * @param firstItem Item on which drag action was started.
     */
    createDragAction(componentSelectionHandler, items, firstItem) {
        if (!SelectionModes_1.hasSelectionModeFlag(this.selectionMode, SelectionModes_1.SelectionModeFlags.DragEnabled))
            return;
        this._dragAction = new DragAction(componentSelectionHandler, items, firstItem);
        this._componentSelectionHandler = componentSelectionHandler;
    }
    /**
     * Updates existing drag action.
     * @param latestItem Latest item in drag action.
     */
    updateDragAction(latestItem) {
        if (!SelectionModes_1.hasSelectionModeFlag(this.selectionMode, SelectionModes_1.SelectionModeFlags.DragEnabled))
            return;
        if (!this._dragAction || !this._componentSelectionHandler)
            return;
        this._lastItem = latestItem;
        const selectionChanges = this._dragAction.updateDragAction(latestItem);
        if (selectionChanges.deselections.length !== 0 || selectionChanges.selections.length !== 0) {
            if (!this._currentOperation)
                this._currentOperation = new BatchSelectionOperation(this._componentSelectionHandler, false);
            this._currentOperation.select(selectionChanges.selections);
            this._currentOperation.deselect(selectionChanges.deselections);
            this._componentSelectionHandler.updateSelection(selectionChanges.selections, selectionChanges.deselections);
        }
    }
    /**
     * Completes drag action.
     */
    completeDragAction() {
        this._dragAction = undefined;
        this._componentSelectionHandler = undefined;
        this.completeOperation();
    }
}
exports.SelectionHandler = SelectionHandler;
//# sourceMappingURL=SelectionHandler.js.map