"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DateTimeTypeConverter = exports.ShortDateTypeConverter = exports.DateTimeTypeConverterBase = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module TypeConverters
 */
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
const DateField_1 = require("../datepicker/DateField");
const DateUtils_1 = require("../common/DateUtils");
const UiComponents_1 = require("../UiComponents");
const TypeConverter_1 = require("./TypeConverter");
const TypeConverterManager_1 = require("./TypeConverterManager");
// cSpell:ignore datepicker valuetypes
/**
 * DateTime Type Converter.
 * @public
 */
class DateTimeTypeConverterBase extends TypeConverter_1.TypeConverter {
    convertToString(value) {
        if (value === undefined)
            return "";
        if (typeof value === "string")
            value = new Date(value);
        // istanbul ignore else
        if (value instanceof Date) {
            switch (this.getTimeFormat()) {
                case ui_core_1.TimeFormat.Short: return value.toLocaleDateString();
                case ui_core_1.TimeFormat.Long: return value.toLocaleString();
            }
            return value.toISOString();
        }
        // istanbul ignore next
        return value.toString();
    }
    static isValidTimeDisplay(type) {
        return Object.keys(ui_abstract_1.TimeDisplay).some((key) => ui_abstract_1.TimeDisplay[key] === type);
    }
    static isAlternateDateFormats(type) {
        return Object.keys(ui_abstract_1.AlternateDateFormats).some((key) => ui_abstract_1.AlternateDateFormats[key] === type);
    }
    // supported options:
    //    timeDisplay?: TimeDisplay
    //    alternateDateFormat?: AlternateDateFormats
    convertToStringWithOptions(value, options) {
        if (value === undefined)
            return "";
        if (options) {
            let timeDisplay = (this.getTimeFormat() === ui_core_1.TimeFormat.Long) ? ui_abstract_1.TimeDisplay.H12MC : undefined;
            let alternateDateFormat = ui_abstract_1.AlternateDateFormats.None;
            // istanbul ignore else
            if ("alternateDateFormat" in options && DateTimeTypeConverterBase.isAlternateDateFormats(options.alternateDateFormat)) {
                alternateDateFormat = options.alternateDateFormat;
            }
            if (this.getTimeFormat() === ui_core_1.TimeFormat.Long) {
                // istanbul ignore else
                if ("timeDisplay" in options) {
                    // istanbul ignore if
                    if (alternateDateFormat) {
                        bentleyjs_core_1.Logger.logInfo(UiComponents_1.UiComponents.loggerCategory(this), `Invalid specification of timeDisplay with alternateDateFormat specification`);
                    }
                    else {
                        // istanbul ignore else
                        if (DateTimeTypeConverterBase.isValidTimeDisplay(options.timeDisplay))
                            timeDisplay = options.timeDisplay;
                    }
                }
            }
            // istanbul ignore else
            if (typeof value === "string") {
                value = new Date(value); // this value will be based on local time zone
                // istanbul ignore else
                if (value instanceof Date && alternateDateFormat) {
                    // alternateDateFormat displays UTC time, so assume string is specifying UTC Date and Time
                    value = DateUtils_1.adjustDateToTimezone(value, value.getTimezoneOffset() * -1);
                }
            }
            // istanbul ignore else
            if (value instanceof Date) {
                // Ensure if alternateDateFormat is specified make sure it matches proper Long/Short format.
                if (this.getTimeFormat() === ui_core_1.TimeFormat.Long) {
                    if (alternateDateFormat === ui_abstract_1.AlternateDateFormats.IsoShort)
                        alternateDateFormat = ui_abstract_1.AlternateDateFormats.IsoDateTime;
                    if (alternateDateFormat === ui_abstract_1.AlternateDateFormats.UtcShort)
                        alternateDateFormat = ui_abstract_1.AlternateDateFormats.UtcDateTime;
                    if (alternateDateFormat === ui_abstract_1.AlternateDateFormats.UtcShortWithDay)
                        alternateDateFormat = ui_abstract_1.AlternateDateFormats.UtcDateTimeWithDay;
                }
                else {
                    // short time format
                    if (alternateDateFormat === ui_abstract_1.AlternateDateFormats.IsoDateTime)
                        alternateDateFormat = ui_abstract_1.AlternateDateFormats.IsoShort;
                    if (alternateDateFormat === ui_abstract_1.AlternateDateFormats.UtcDateTime)
                        alternateDateFormat = ui_abstract_1.AlternateDateFormats.UtcShort;
                    if (alternateDateFormat === ui_abstract_1.AlternateDateFormats.UtcDateTimeWithDay)
                        alternateDateFormat = ui_abstract_1.AlternateDateFormats.UtcShortWithDay;
                }
                const formattedDateTime = DateField_1.formatInputDate(value, timeDisplay, undefined, alternateDateFormat);
                // istanbul ignore else
                if (formattedDateTime)
                    return formattedDateTime;
            }
        }
        return this.convertToString(value);
    }
    /** Default implementation just calls convertFromString with no options */
    convertFromStringWithOptions(value, options) {
        if (!value)
            return undefined;
        if (options) {
            let alternateDateFormat = ui_abstract_1.AlternateDateFormats.None;
            // istanbul ignore else
            if ("alternateDateFormat" in options && DateTimeTypeConverterBase.isAlternateDateFormats(options.alternateDateFormat)) {
                alternateDateFormat = options.alternateDateFormat;
            }
            // istanbul ignore else
            let date = new Date(value); // this value will be based on local time zone
            // istanbul ignore else
            if (date instanceof Date && alternateDateFormat) {
                // alternateDateFormat displays UTC time, so assume string is specifying UTC Date and Time
                date = DateUtils_1.adjustDateToTimezone(date, date.getTimezoneOffset() * -1);
            }
        }
        return this.convertFromString(value);
    }
    isDateValid(date) {
        return date instanceof Date && !isNaN(+date);
    }
    convertFromString(value) {
        if (!value)
            return undefined;
        const dateValue = new Date(value);
        if (!this.isDateValid(dateValue))
            return undefined;
        return dateValue;
    }
    get isLessGreaterType() { return true; }
    sortCompare(valueA, valueB, _ignoreCase) {
        return valueA.valueOf() - valueB.valueOf();
    }
    isEqualTo(valueA, valueB) {
        return valueA.valueOf() === valueB.valueOf();
    }
    isNotEqualTo(valueA, valueB) {
        return valueA.valueOf() !== valueB.valueOf();
    }
    isLessThan(a, b) {
        return a.valueOf() < b.valueOf();
    }
    isLessThanOrEqualTo(a, b) {
        return a.valueOf() <= b.valueOf();
    }
    isGreaterThan(a, b) {
        return a.valueOf() > b.valueOf();
    }
    isGreaterThanOrEqualTo(a, b) {
        return a.valueOf() >= b.valueOf();
    }
}
exports.DateTimeTypeConverterBase = DateTimeTypeConverterBase;
/**
 * Short Date Type Converter.
 * @public
 */
class ShortDateTypeConverter extends DateTimeTypeConverterBase {
    getTimeFormat() { return ui_core_1.TimeFormat.Short; }
}
exports.ShortDateTypeConverter = ShortDateTypeConverter;
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.ShortDate, ShortDateTypeConverter);
/**
 * Date Time Type Converter.
 * @public
 */
class DateTimeTypeConverter extends DateTimeTypeConverterBase {
    getTimeFormat() { return ui_core_1.TimeFormat.Long; }
}
exports.DateTimeTypeConverter = DateTimeTypeConverter;
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.DateTime, DateTimeTypeConverter);
//# sourceMappingURL=DateTimeTypeConverter.js.map