"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module TypeConverters
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IntTypeConverter = exports.FloatTypeConverter = exports.NumericTypeConverterBase = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const TypeConverter_1 = require("./TypeConverter");
const TypeConverterManager_1 = require("./TypeConverterManager");
/**
 * Base Numeric Type Converter.
 * @public
 */
class NumericTypeConverterBase extends TypeConverter_1.TypeConverter {
    get isLessGreaterType() { return true; }
    isLessThan(a, b) {
        return a < b;
    }
    isLessThanOrEqualTo(a, b) {
        return a <= b;
    }
    isGreaterThan(a, b) {
        return a > b;
    }
    isGreaterThanOrEqualTo(a, b) {
        return a >= b;
    }
    sortCompare(a, b, _ignoreCase) {
        return (+a) - (+b);
    }
}
exports.NumericTypeConverterBase = NumericTypeConverterBase;
/**
 * Float Type Converter.
 * @public
 */
class FloatTypeConverter extends NumericTypeConverterBase {
    convertToString(value) {
        if (value === undefined)
            return "";
        let numericValue = 0;
        if (typeof value === "string") {
            if (value === "-" || value === "" || value === "-0.0" || value === "-0") {
                // handle these semi-valid values as 0
                numericValue = 0;
            }
            else {
                numericValue = parseFloat(value);
            }
        }
        else {
            numericValue = value;
        }
        // this is close to calling `toFixed(2)`, but cuts of any trailing zeros
        let stringValue = (Math.round(100 * numericValue) / 100).toString();
        // because this is a _float_ converter, we want to emphasize the number is a float - make
        // sure there's a decimal part
        if (stringValue.indexOf(".") === -1)
            stringValue += ".0";
        return stringValue;
    }
    convertFromString(value) {
        return parseFloat(value);
    }
}
exports.FloatTypeConverter = FloatTypeConverter;
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Float, FloatTypeConverter);
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Double, FloatTypeConverter);
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Number, FloatTypeConverter);
/**
 * Int Type Converter.
 * @public
 */
class IntTypeConverter extends NumericTypeConverterBase {
    convertToString(value) {
        if (value === undefined)
            return "";
        let numericValue = 0;
        if (typeof value === "string") {
            if (value === "-" || value === "" || value === "-0") {
                // handle these semi-valid values as 0
                numericValue = 0;
            }
            else {
                numericValue = parseInt(value, 10);
            }
        }
        else {
            numericValue = value;
        }
        return Math.round(numericValue).toString();
    }
    convertFromString(value) {
        return parseInt(value, 10);
    }
}
exports.IntTypeConverter = IntTypeConverter;
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Int, IntTypeConverter);
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Integer, IntTypeConverter);
//# sourceMappingURL=NumericTypeConverter.js.map