"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module TypeConverters
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Point3dTypeConverter = exports.Point2dTypeConverter = exports.BasePointTypeConverter = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
const TypeConverter_1 = require("./TypeConverter");
const TypeConverterManager_1 = require("./TypeConverterManager");
// cSpell:ignore valuetypes
/**
 * Point type converter.
 * @public
 */
class BasePointTypeConverter extends TypeConverter_1.TypeConverter {
    constructor(componentConverterName = ui_abstract_1.StandardTypeNames.Double) {
        super();
        this.componentConverterName = componentConverterName;
    }
    formatValue(value) {
        if (typeof value === "string")
            value = parseFloat(value);
        return TypeConverterManager_1.TypeConverterManager.getConverter(this.componentConverterName).convertToString(value);
    }
    convertToString(value) {
        if (!value)
            return "";
        let components = new Array();
        if (Array.isArray(value)) {
            if (value.length === 0)
                return "";
            components = value.map((c) => this.formatValue(c));
        }
        else {
            components = [this.formatValue(value.x), this.formatValue(value.y)];
            if (undefined !== value.z)
                components.push(this.formatValue(value.z));
        }
        const hasAsyncComponents = components.some(ui_core_1.isPromiseLike);
        if (hasAsyncComponents) {
            return Promise.all(components.map(async (c) => ui_core_1.isPromiseLike(c) ? c : Promise.resolve(c))).then((c) => c.join(", "));
        }
        return components.join(", ");
    }
    convertFromString(value) {
        return this.constructPoint(value.split(","));
    }
    sortCompare(a, b, _ignoreCase) {
        const aLength = this.getVectorLength(a);
        const bLength = this.getVectorLength(b);
        if (aLength === bLength)
            return 0;
        if (aLength === undefined)
            return -1;
        if (bLength === undefined)
            return 1;
        return aLength - bLength;
    }
}
exports.BasePointTypeConverter = BasePointTypeConverter;
/**
 * Point2d type converter.
 * @public
 */
class Point2dTypeConverter extends BasePointTypeConverter {
    constructor(componentConverterName) {
        super(componentConverterName);
    }
    getVectorLength(point) {
        const derivedPoint = this.constructPoint(point);
        if (derivedPoint === undefined)
            return undefined;
        return Math.sqrt(Math.pow(derivedPoint.x, 2) + Math.pow(derivedPoint.y, 2));
    }
    constructPoint(values) {
        if (Array.isArray(values)) {
            if (values.length !== 2 || isNaN(+values[0]) || isNaN(+values[1]))
                return undefined;
            return { x: +values[0], y: +values[1] };
        }
        return values;
    }
}
exports.Point2dTypeConverter = Point2dTypeConverter;
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Point2d, Point2dTypeConverter);
/**
 * Point3d type converter.
 * @public
 */
class Point3dTypeConverter extends BasePointTypeConverter {
    constructor(componentConverterName) {
        super(componentConverterName);
    }
    getVectorLength(point) {
        const derivedPoint = this.constructPoint(point);
        if (derivedPoint === undefined)
            return undefined;
        return Math.sqrt(Math.pow(derivedPoint.x, 2) + Math.pow(derivedPoint.y, 2) + Math.pow(derivedPoint.z, 2));
    }
    constructPoint(values) {
        if (Array.isArray(values)) {
            if (values.length !== 3 || isNaN(+values[0]) || isNaN(+values[1]) || isNaN(+values[2]))
                return undefined;
            return { x: +values[0], y: +values[1], z: +values[2] };
        }
        const z = values.z;
        return Object.assign(Object.assign({}, values), { z: z ? z : 0 });
    }
}
exports.Point3dTypeConverter = Point3dTypeConverter;
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Point3d, Point3dTypeConverter);
//# sourceMappingURL=PointTypeConverter.js.map