"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module TypeConverters
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringTypeConverter = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const TypeConverter_1 = require("./TypeConverter");
const TypeConverterManager_1 = require("./TypeConverterManager");
/**
 * String Type Converter.
 * @public
 */
class StringTypeConverter extends TypeConverter_1.TypeConverter {
    convertToString(value) {
        return value ? value.toString() : "";
    }
    convertFromString(value) {
        return value;
    }
    sortCompare(valueA, valueB, ignoreCase) {
        if (!this.checkArgTypes(valueA, valueB))
            return 0;
        if (ignoreCase)
            return valueA.toLocaleLowerCase().localeCompare(valueB.toLocaleLowerCase());
        else
            return valueA.localeCompare(valueB);
    }
    get isStringType() { return true; }
    startsWith(valueA, valueB, caseSensitive) {
        if (!valueA || !valueB || !this.checkArgTypes(valueA, valueB))
            return false;
        if (caseSensitive)
            return (valueA.substr(0, valueB.length) === valueB);
        return (valueA.toLocaleUpperCase().substr(0, valueB.length) === valueB.toLocaleUpperCase());
    }
    endsWith(valueA, valueB, caseSensitive) {
        if (!valueA || !valueB || !this.checkArgTypes(valueA, valueB))
            return false;
        const position = valueA.length - valueB.length;
        if (position < 0)
            return false;
        let lastIndex;
        if (caseSensitive)
            lastIndex = valueA.indexOf(valueB, position);
        else
            lastIndex = valueA.toLocaleUpperCase().indexOf(valueB.toLocaleUpperCase(), position);
        return lastIndex !== -1 && lastIndex === position;
    }
    contains(valueA, valueB, caseSensitive) {
        if (!valueA || !valueB || !this.checkArgTypes(valueA, valueB))
            return false;
        if (valueB.length > valueA.length)
            return false;
        if (caseSensitive)
            return valueA.indexOf(valueB, 0) !== -1;
        return valueA.toLocaleUpperCase().indexOf(valueB.toLocaleUpperCase(), 0) !== -1;
    }
    doesNotContain(valueA, valueB, caseSensitive) {
        return !this.contains(valueA, valueB, caseSensitive);
    }
    isContainedIn(valueA, valueB, caseSensitive) {
        return this.contains(valueB, valueA, caseSensitive);
    }
    isNotContainedIn(valueA, valueB, caseSensitive) {
        return this.doesNotContain(valueB, valueA, caseSensitive);
    }
    isEmpty(valueA) {
        if (!this.checkArgTypes(valueA))
            return true;
        return valueA.length === 0;
    }
    isNotEmpty(valueA) {
        return !this.isEmpty(valueA);
    }
    checkArgTypes(valueA, valueB) {
        if (typeof valueA !== "string")
            return false;
        if (valueB && typeof valueB !== "string")
            return false;
        return true;
    }
}
exports.StringTypeConverter = StringTypeConverter;
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.Text, StringTypeConverter);
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.String, StringTypeConverter);
TypeConverterManager_1.TypeConverterManager.registerConverter(ui_abstract_1.StandardTypeNames.URL, StringTypeConverter);
//# sourceMappingURL=StringTypeConverter.js.map