"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module TypeConverters
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeConverter = void 0;
// cSpell:ignore valuetypes shortdate
const ui_abstract_1 = require("@bentley/ui-abstract");
/**
 * Type Converter base class.
 * @public
 */
class TypeConverter {
    /** Converts a primitive value to a string */
    convertToString(value) {
        if (value === undefined)
            return "";
        return value.toString();
    }
    /** Default implementation just calls convertToString with no options */
    convertToStringWithOptions(value, _options) {
        return this.convertToString(value);
    }
    /** Converts a string to a primitive value */
    convertFromString(_value) {
        return undefined;
    }
    /** Default implementation just calls convertFromString with no options */
    convertFromStringWithOptions(value, _options) {
        return this.convertFromString(value);
    }
    /** Converts a value associated with a property description to a string */
    convertPropertyToString(propertyDescription, value) {
        var _a;
        return this.convertToStringWithOptions(value, (_a = propertyDescription.converter) === null || _a === void 0 ? void 0 : _a.options);
    }
    /** Converts a string with a property record to a property value */
    async convertFromStringToPropertyValue(value, propertyRecord) {
        const converterOptions = (propertyRecord && propertyRecord.property.converter) ? propertyRecord.property.converter.options : undefined;
        const stringValue = await this.convertFromStringWithOptions(value, converterOptions);
        const propertyValue = {
            valueFormat: ui_abstract_1.PropertyValueFormat.Primitive,
            value: stringValue ? stringValue : "",
            displayValue: "",
        };
        return propertyValue;
    }
    /** Determines if two primitive values are equal */
    isEqualTo(valueA, valueB) {
        return valueA === valueB;
    }
    /** Determines if two primitive values are not equal */
    isNotEqualTo(valueA, valueB) {
        return valueA !== valueB;
    }
    /** Determines if a primitive value is null or undefined */
    isNull(value) {
        return value === null || value === undefined;
    }
    /** Determines if a primitive value is not null or not undefined */
    isNotNull(value) {
        return value !== null && value !== undefined;
    }
    /** Determines if the converter is for a string type */
    get isStringType() { return false; }
    /** Determines if the converter is for a numeric type */
    get isLessGreaterType() { return false; }
    /** Determines if the converter is for a boolean type */
    get isBooleanType() { return false; }
    /** Determines if the converter is for a nullable type */
    get isNullableType() { return true; }
}
exports.TypeConverter = TypeConverter;
//# sourceMappingURL=TypeConverter.js.map