"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Date
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DateField = exports.formatInputDate = void 0;
const React = __importStar(require("react"));
const classnames_1 = __importDefault(require("classnames"));
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
require("./DateField.scss");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
/** Function to format a Date object that is used in control, type editor, and type converter
 * to produce a consistent formatted Date string.
 * @internal
 */
function formatInputDate(inputDate, timeDisplay, customFormatter, alternateDateFormat) {
    if (customFormatter) {
        return customFormatter.formateDate(inputDate);
    }
    if (alternateDateFormat) {
        switch (alternateDateFormat) {
            case ui_abstract_1.AlternateDateFormats.IsoDateTime:
                return inputDate.toISOString();
            case ui_abstract_1.AlternateDateFormats.IsoShort:
                return inputDate.toISOString().slice(0, 10);
            case ui_abstract_1.AlternateDateFormats.UtcDateTime:
                return inputDate.toUTCString().slice(5);
            case ui_abstract_1.AlternateDateFormats.UtcShort:
                return inputDate.toUTCString().slice(5, 16);
            case ui_abstract_1.AlternateDateFormats.UtcDateTimeWithDay:
                return inputDate.toUTCString();
            case ui_abstract_1.AlternateDateFormats.UtcShortWithDay:
                return inputDate.toUTCString().slice(0, 16);
        }
    }
    else {
        const dateString = inputDate.toLocaleDateString(undefined, { day: "2-digit", month: "2-digit", year: "numeric" });
        let timeString = "";
        if (timeDisplay) {
            switch (timeDisplay) {
                case ui_abstract_1.TimeDisplay.H12MC:
                    timeString = inputDate.toLocaleTimeString(undefined, { hour12: true, hour: "2-digit", minute: "2-digit" });
                    break;
                case ui_abstract_1.TimeDisplay.H12MSC:
                    timeString = (inputDate.toLocaleTimeString(undefined, { hour12: true, hour: "2-digit", minute: "2-digit", second: "2-digit" }));
                    break;
                case ui_abstract_1.TimeDisplay.H24M:
                    timeString = (inputDate.toLocaleTimeString(undefined, { hour12: false, hour: "2-digit", minute: "2-digit" }));
                    // istanbul ignore next
                    if (timeString === "24:00")
                        timeString = "00:00";
                    break;
                case ui_abstract_1.TimeDisplay.H24MS:
                    timeString = (inputDate.toLocaleTimeString(undefined, { hour12: false, hour: "2-digit", minute: "2-digit", second: "2-digit" }));
                    // istanbul ignore next
                    if (timeString === "24:00:00")
                        timeString = "00:00:00";
                    break;
            }
        }
        return timeString.length ? `${dateString}, ${timeString}` : dateString;
    }
}
exports.formatInputDate = formatInputDate;
/** Date input component. This component is typically used by the [[DatePickerPopupButton]] to display and edit date and time.
 * @internal
 */
function DateField({ initialDate, onDateChange, readOnly, dateFormatter, timeDisplay, style, className }) {
    const initialDateRef = React.useRef(initialDate);
    const [hasBadInput, setHasBadInput] = React.useState(false);
    const [inputValue, setInputValue] = React.useState(formatInputDate(initialDate, timeDisplay, dateFormatter));
    // See if new initialDate props have changed since component mounted
    React.useEffect(() => {
        // istanbul ignore else
        if (initialDate.getTime() !== initialDateRef.current.getTime()) {
            initialDateRef.current = initialDate;
        }
        setInputValue(formatInputDate(initialDate, timeDisplay, dateFormatter));
    }, [initialDate, dateFormatter, timeDisplay]);
    const handleInputChange = React.useCallback((event) => {
        setInputValue(event.currentTarget.value);
    }, []);
    const parseDate = React.useCallback((dateString) => {
        try {
            if (dateFormatter && dateFormatter.parseDate)
                return dateFormatter.parseDate(dateString);
            const newDateValue = Date.parse(dateString);
            // istanbul ignore else
            if (newDateValue)
                return new Date(newDateValue);
        }
        catch (_error) /* istanbul ignore next */ {
            bentleyjs_core_1.Logger.logInfo("DateField", `Encountered error parsing input value '${dateString}' as a date.`);
            return undefined;
        }
        // istanbul ignore next
        return undefined;
    }, [dateFormatter]);
    const updateInputDate = React.useCallback((dateString) => {
        try {
            const newDateValue = parseDate(dateString);
            if (newDateValue) {
                const newDate = new Date(newDateValue);
                onDateChange && onDateChange(newDate);
                setHasBadInput(false);
            }
            else {
                setHasBadInput(true);
            }
        }
        catch (_error) {
            // istanbul ignore next
            setHasBadInput(true);
        }
    }, [onDateChange, parseDate]);
    const handleOnBlur = React.useCallback((event) => {
        updateInputDate(event.target.value);
    }, [updateInputDate]);
    function onInputKeyDown(event) {
        // istanbul ignore else
        if (event.key === "Enter") {
            updateInputDate(event.currentTarget.value);
            event.preventDefault();
        }
    }
    const classNames = classnames_1.default(className, "components-date-input", hasBadInput && "components-date-has-error");
    return React.createElement(ui_core_1.Input, { "data-testid": "components-date-input", style: style, className: classNames, onKeyDown: onInputKeyDown, onBlur: handleOnBlur, onChange: handleInputChange, value: inputValue, disabled: readOnly || (dateFormatter && !(dateFormatter.parseDate)) });
}
exports.DateField = DateField;
//# sourceMappingURL=DateField.js.map