"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Date
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimeField = void 0;
const React = __importStar(require("react"));
const ui_core_1 = require("@bentley/ui-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const UiComponents_1 = require("../UiComponents");
require("./TimeField.scss");
function getValidInt(intText, min, max, defaultValue) {
    try {
        const newValue = parseInt(intText, 10);
        if (newValue >= min && newValue <= max)
            return newValue;
    }
    catch (_e) {
        // istanbul ignore next
        return defaultValue;
    }
    return defaultValue;
}
function isSameTime(a, b) {
    return a.hours === b.hours && a.minutes === b.minutes && a.seconds === b.seconds;
}
/** Field used to set the Hour:Minutes:Seconds in the [[DatePicker]] Popup panel. The user may key-in the value or use up/down keys to
 * change the time.
 * @internal
 */
function TimeField({ time, timeDisplay, readOnly, onTimeChange }) {
    const initialDateRef = React.useRef(time);
    const amLabelRef = React.useRef(UiComponents_1.UiComponents.translate("timepicker.day-period-am"));
    const pmLabelRef = React.useRef(UiComponents_1.UiComponents.translate("timepicker.day-period-pm"));
    const [timeSpec, setTimeSpec] = React.useState(time);
    const { hours, minutes, seconds } = timeSpec;
    const showDayPeriod = -1 !== timeDisplay.search("aa");
    const getDisplayHours = React.useCallback((timeHours) => {
        if (!showDayPeriod) {
            return timeHours;
        }
        const outHours = (0 === timeHours) ? 12 : timeHours > 12 ? timeHours - 12 : timeHours;
        return outHours;
    }, [showDayPeriod]);
    // See if new initialDate props have changed since component mounted
    React.useEffect(() => {
        if (!isSameTime(time, initialDateRef.current)) {
            setTimeSpec(time);
            initialDateRef.current = time;
        }
    }, [time]);
    const displayHour = getDisplayHours(hours);
    const [dayPeriodText, setDayPeriodText] = React.useState(showDayPeriod ? (hours >= 12 ? pmLabelRef.current : amLabelRef.current) : undefined);
    const [hoursText, setHoursText] = React.useState(displayHour.toString().padStart(2, "0"));
    const [minutesText, setMinutesText] = React.useState(minutes.toString().padStart(2, "0"));
    const [secondsText, setSecondsText] = React.useState(seconds.toString().padStart(2, "0"));
    const updateTimeSpec = React.useCallback((newTime) => {
        if (newTime.hours !== timeSpec.hours || newTime.minutes !== timeSpec.minutes || newTime.seconds !== timeSpec.seconds) {
            setTimeSpec(newTime);
            onTimeChange && onTimeChange(newTime);
        }
    }, [onTimeChange, timeSpec.hours, timeSpec.minutes, timeSpec.seconds]);
    const handleHoursChange = React.useCallback((event) => {
        setHoursText(event.currentTarget.value);
    }, []);
    const handleMinutesChange = React.useCallback((event) => {
        setMinutesText(event.currentTarget.value);
    }, []);
    const handleSecondsChange = React.useCallback((event) => {
        setSecondsText(event.currentTarget.value);
    }, []);
    const handleDayPeriodChange = React.useCallback((event) => {
        setDayPeriodText(event.currentTarget.value);
    }, []);
    const handleHoursOnBlur = React.useCallback(() => {
        const newHours = getValidInt(hoursText, 0, 24, hours);
        setHoursText(getDisplayHours(newHours).toString().padStart(2, "0"));
        updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { hours: newHours }));
        showDayPeriod && setDayPeriodText(newHours >= 12 ? pmLabelRef.current : amLabelRef.current);
    }, [getDisplayHours, hours, hoursText, showDayPeriod, timeSpec, updateTimeSpec]);
    const handleMinutesOnBlur = React.useCallback(() => {
        const newMinutes = getValidInt(minutesText, 0, 59, minutes);
        setMinutesText(newMinutes.toString().padStart(2, "0"));
        updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { minutes: newMinutes }));
    }, [minutes, minutesText, timeSpec, updateTimeSpec]);
    const handleSecondsOnBlur = React.useCallback(() => {
        const newSeconds = getValidInt(secondsText, 0, 59, seconds);
        setSecondsText(newSeconds.toString().padStart(2, "0"));
        updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { seconds: newSeconds }));
    }, [seconds, secondsText, timeSpec, updateTimeSpec]);
    const handleDayPeriodOnBlur = React.useCallback(() => {
        let newPeriodText;
        // istanbul ignore else
        if (dayPeriodText === "AM" || dayPeriodText === "am" || dayPeriodText === amLabelRef.current)
            newPeriodText = amLabelRef.current;
        else if (dayPeriodText === "PM" || dayPeriodText === "pm" || dayPeriodText === pmLabelRef.current)
            newPeriodText = pmLabelRef.current;
        // istanbul ignore else
        if (undefined !== newPeriodText) {
            setDayPeriodText(newPeriodText);
            if (newPeriodText === amLabelRef.current && hours > 12) {
                updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { hours: hours - 12 }));
            }
            else if (newPeriodText === pmLabelRef.current && hours <= 11) {
                updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { hours: hours + 12 }));
            }
        }
    }, [dayPeriodText, hours, timeSpec, updateTimeSpec]);
    const handleHoursOnKeyDown = React.useCallback((event) => {
        // istanbul ignore else
        if (event.key === ui_abstract_1.SpecialKey.ArrowDown || event.key === ui_abstract_1.SpecialKey.ArrowUp) {
            let newHours = hours + (event.key === ui_abstract_1.SpecialKey.ArrowDown ? -1 : 1);
            if (newHours < 0)
                newHours = 24;
            if (newHours > 24 || (newHours > 23 && ((minutes + seconds) > 0)))
                newHours = 0;
            setHoursText(getDisplayHours(newHours).toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { hours: newHours }));
            showDayPeriod && setDayPeriodText(newHours >= 12 ? pmLabelRef.current : amLabelRef.current);
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.Enter) {
            const newHours = getValidInt(hoursText, 0, 24, hours);
            setHoursText(getDisplayHours(newHours).toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { hours: newHours }));
            showDayPeriod && setDayPeriodText(newHours >= 12 ? pmLabelRef.current : amLabelRef.current);
            event.preventDefault();
        }
    }, [getDisplayHours, hours, hoursText, minutes, seconds, showDayPeriod, timeSpec, updateTimeSpec]);
    const handleMinutesOnKeyDown = React.useCallback((event) => {
        if (event.key === ui_abstract_1.SpecialKey.ArrowDown) {
            const newMinutes = minutes === 0 ? 59 : minutes - 1;
            setMinutesText(newMinutes.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { minutes: newMinutes }));
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.ArrowUp) {
            const newMinutes = minutes === 59 ? 0 : minutes + 1;
            setMinutesText(newMinutes.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { minutes: newMinutes }));
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.Home || event.key === ui_abstract_1.SpecialKey.End) {
            const newMinutes = event.key === ui_abstract_1.SpecialKey.Home ? 0 : 59;
            setMinutesText(newMinutes.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { minutes: newMinutes }));
            event.preventDefault();
        }
        else /* istanbul ignore next */ if (event.key === ui_abstract_1.SpecialKey.Enter) {
            const newMinutes = getValidInt(minutesText, 0, 59, minutes);
            setMinutesText(newMinutes.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { minutes: newMinutes }));
            event.preventDefault();
        }
    }, [minutesText, minutes, timeSpec, updateTimeSpec]);
    const handleSecondsOnKeyDown = React.useCallback((event) => {
        // istanbul ignore else
        if (event.key === ui_abstract_1.SpecialKey.ArrowDown) {
            const newSeconds = seconds === 0 ? 59 : seconds - 1;
            setSecondsText(newSeconds.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { seconds: newSeconds }));
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.ArrowUp) {
            const newSeconds = seconds === 59 ? 0 : seconds + 1;
            setSecondsText(newSeconds.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { seconds: newSeconds }));
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.Home || event.key === ui_abstract_1.SpecialKey.End) {
            const newSeconds = event.key === ui_abstract_1.SpecialKey.Home ? 0 : 59;
            setSecondsText(newSeconds.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { seconds: newSeconds }));
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.Enter) {
            const newSeconds = getValidInt(secondsText, 0, 59, seconds);
            setSecondsText(newSeconds.toString().padStart(2, "0"));
            updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { seconds: newSeconds }));
            event.preventDefault();
        }
    }, [seconds, secondsText, timeSpec, updateTimeSpec]);
    const handleDayPeriodOnKeyDown = React.useCallback((event) => {
        let newPeriodText;
        // istanbul ignore else
        if (event.key === ui_abstract_1.SpecialKey.ArrowDown || event.key === ui_abstract_1.SpecialKey.Home || event.key === "a" || event.key === "A") {
            newPeriodText = amLabelRef.current;
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.ArrowUp || event.key === ui_abstract_1.SpecialKey.End || event.key === "p" || event.key === "P") {
            newPeriodText = pmLabelRef.current;
            event.preventDefault();
        }
        else if (event.key === ui_abstract_1.SpecialKey.Enter) {
            if (dayPeriodText === "AM" || dayPeriodText === "am" || dayPeriodText === amLabelRef.current)
                newPeriodText = amLabelRef.current;
            else if (dayPeriodText === "PM" || dayPeriodText === "pm" || dayPeriodText === pmLabelRef.current)
                newPeriodText = pmLabelRef.current;
            event.preventDefault();
        }
        // istanbul ignore else
        if (undefined !== newPeriodText) {
            setDayPeriodText(newPeriodText);
            /* istanbul ignore else */
            if (newPeriodText === amLabelRef.current && hours > 12) {
                updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { hours: hours - 12 }));
            }
            else if (newPeriodText === pmLabelRef.current && hours <= 11) {
                updateTimeSpec(Object.assign(Object.assign({}, timeSpec), { hours: hours + 12 }));
            }
        }
    }, [dayPeriodText, hours, timeSpec, updateTimeSpec]);
    const showSeconds = -1 !== timeDisplay.search(":ss");
    return React.createElement("div", { "data-testid": "components-time-input", className: "components-time" },
        React.createElement(ui_core_1.Input, { className: "components-time-input", onKeyDown: handleHoursOnKeyDown, onBlur: handleHoursOnBlur, onChange: handleHoursChange, value: hoursText, disabled: readOnly }),
        React.createElement("span", { className: "component-time-separator" }, ":"),
        React.createElement(ui_core_1.Input, { className: "components-time-input", onKeyDown: handleMinutesOnKeyDown, onBlur: handleMinutesOnBlur, onChange: handleMinutesChange, value: minutesText, disabled: readOnly }),
        showSeconds &&
            React.createElement(React.Fragment, null,
                React.createElement("span", { className: "component-time-separator" }, ":"),
                React.createElement(ui_core_1.Input, { className: "components-time-input", onKeyDown: handleSecondsOnKeyDown, onBlur: handleSecondsOnBlur, onChange: handleSecondsChange, value: secondsText, disabled: readOnly })),
        dayPeriodText &&
            React.createElement(ui_core_1.Input, { className: "components-time-period-input", onKeyDown: handleDayPeriodOnKeyDown, onBlur: handleDayPeriodOnBlur, onChange: handleDayPeriodChange, value: dayPeriodText, disabled: readOnly }));
}
exports.TimeField = TimeField;
//# sourceMappingURL=TimeField.js.map