"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyEditors
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EditorContainer = void 0;
require("./EditorContainer.scss");
const React = __importStar(require("react"));
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_abstract_1 = require("@bentley/ui-abstract");
const PropertyEditorManager_1 = require("./PropertyEditorManager");
/**
 * EditorContainer React component used by the Table, Tree and PropertyGrid for cell editing.
 * @beta
 */
class EditorContainer extends React.PureComponent {
    constructor() {
        super(...arguments);
        this._spanRef = React.createRef();
        this._handleEditorBlur = (_e) => {
            // istanbul ignore else
            if (!this.props.ignoreEditorBlur && this._propertyEditor && this._propertyEditor.containerHandlesBlur)
                this._handleContainerCommit(); // eslint-disable-line @typescript-eslint/no-floating-promises
        };
        this._handleContainerBlur = (e) => {
            e.stopPropagation();
        };
        this._handleClick = (e) => {
            e.stopPropagation();
        };
        this._handleKeyDown = (e) => {
            switch (e.key) {
                case ui_abstract_1.SpecialKey.Escape:
                    this.onPressEscape(e);
                    break;
                case ui_abstract_1.SpecialKey.Enter:
                    this.onPressEnter(e);
                    break;
                case ui_abstract_1.SpecialKey.Tab:
                    this.onPressTab(e);
                    break;
                default:
                    e.stopPropagation();
            }
        };
        this._handleRightClick = (e) => {
            e.stopPropagation();
        };
        this._handleEditorCommit = (args) => {
            this._commit(args); // eslint-disable-line @typescript-eslint/no-floating-promises
        };
        this._handleContainerCommit = async () => {
            const newValue = this._editorRef && await this._editorRef.getPropertyValue();
            // istanbul ignore else
            if (newValue !== undefined) {
                this._commit({ propertyRecord: this.props.propertyRecord, newValue }); // eslint-disable-line @typescript-eslint/no-floating-promises
            }
        };
        this._commit = async (args) => {
            const newValue = args.newValue;
            const isValid = await this.isNewValueValid(newValue);
            if (isValid) {
                let doCommit = true;
                // istanbul ignore else
                if (this._propertyEditor && args.propertyRecord) {
                    const commitResult = await this._propertyEditor.commitValue(newValue, args.propertyRecord);
                    if (commitResult.encounteredError) {
                        this.displayInputFieldMessage(commitResult.errorMessage);
                        doCommit = false;
                    }
                }
                if (doCommit) {
                    this.props.onCommit(args);
                }
            }
        };
        this._handleEditorCancel = () => {
            this._commitCancel();
        };
        this._commitCancel = () => {
            this.props.onCancel();
        };
    }
    createEditor() {
        const editorRef = (ref) => this._editorRef = ref;
        const editorProps = {
            ref: editorRef,
            onCommit: this._handleEditorCommit,
            onCancel: this._handleEditorCancel,
            onBlur: this._handleEditorBlur,
            propertyRecord: this.props.propertyRecord,
            setFocus: this.props.setFocus !== undefined ? this.props.setFocus : true,
            className: this.props.className,
            style: this.props.style,
        };
        const propDescription = this.props.propertyRecord.property;
        const editorName = propDescription.editor !== undefined ? propDescription.editor.name : undefined;
        this._propertyEditor = PropertyEditorManager_1.PropertyEditorManager.createEditor(propDescription.typename, editorName, propDescription.dataController);
        const editorNode = this._propertyEditor.reactNode;
        let clonedNode = null;
        // istanbul ignore else
        if (React.isValidElement(editorNode)) {
            clonedNode = React.cloneElement(editorNode, editorProps);
        }
        return clonedNode;
    }
    componentDidMount() {
        var _a;
        (_a = this._spanRef.current) === null || _a === void 0 ? void 0 : _a.addEventListener("keydown", this._handleKeyDown, true);
    }
    componentWillUnmount() {
        // istanbul ignore next
        if (this._spanRef.current)
            this._spanRef.current.removeEventListener("keydown", this._handleKeyDown, true);
    }
    onPressEscape(_e) {
        // istanbul ignore else
        if (this._propertyEditor && this._propertyEditor.containerHandlesEscape) {
            this._commitCancel();
        }
    }
    onPressEnter(e) {
        // istanbul ignore else
        if (this._propertyEditor && this._propertyEditor.containerHandlesEnter) {
            // istanbul ignore else
            if (this._editorRef && this._editorRef.hasFocus)
                e.stopPropagation();
            this._handleContainerCommit(); // eslint-disable-line @typescript-eslint/no-floating-promises
        }
    }
    onPressTab(e) {
        // istanbul ignore else
        if (this._propertyEditor && this._propertyEditor.containerHandlesTab) {
            e.stopPropagation();
            this._handleContainerCommit(); // eslint-disable-line @typescript-eslint/no-floating-promises
        }
    }
    displayInputFieldMessage(errorMessage) {
        // istanbul ignore else
        if (errorMessage && this._editorRef) {
            const details = new imodeljs_frontend_1.NotifyMessageDetails(errorMessage.priority, errorMessage.briefMessage, errorMessage.detailedMessage);
            const htmlElement = this._editorRef && this._editorRef.htmlElement;
            // istanbul ignore else
            if (htmlElement)
                details.setInputFieldTypeDetails(htmlElement);
            if (imodeljs_frontend_1.IModelApp.notifications)
                imodeljs_frontend_1.IModelApp.notifications.outputMessage(details);
        }
    }
    async isNewValueValid(value) {
        let isValid = true;
        // istanbul ignore else
        if (this._propertyEditor && this.props.propertyRecord) {
            const validateResult = await this._propertyEditor.validateValue(value, this.props.propertyRecord);
            if (validateResult.encounteredError) {
                this.displayInputFieldMessage(validateResult.errorMessage);
                isValid = false;
            }
        }
        else {
            isValid = false;
        }
        return isValid;
    }
    /** @internal */
    render() {
        return (React.createElement("span", { className: "components-editor-container", onBlur: this._handleContainerBlur, onClick: this._handleClick, onContextMenu: this._handleRightClick, title: this.props.title, "data-testid": "editor-container", role: "presentation", ref: this._spanRef }, this.createEditor()));
    }
}
exports.EditorContainer = EditorContainer;
//# sourceMappingURL=EditorContainer.js.map