"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyEditors
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BasicPropertyEditor = exports.PropertyEditorManager = exports.DataControllerBase = exports.PropertyEditorBase = void 0;
const React = __importStar(require("react"));
const TextEditor_1 = require("./TextEditor");
const ui_abstract_1 = require("@bentley/ui-abstract");
/** PropertyEditor is the base class for all property editors.
 * @beta
 */
class PropertyEditorBase {
    constructor() {
        this.customDataController = undefined;
    }
    get containerHandlesBlur() {
        return true;
    }
    get containerHandlesEscape() {
        return true;
    }
    get containerHandlesEnter() {
        return true;
    }
    get containerHandlesTab() {
        return true;
    }
    applyEditorParams(_property, _record) { }
    async commitValue(newValue, record) {
        if (this.customDataController)
            return this.customDataController.commitValue(newValue, record);
        return { encounteredError: false };
    }
    async validateValue(newValue, record) {
        if (this.customDataController)
            return this.customDataController.validateValue(newValue, record);
        return { encounteredError: false };
    }
}
exports.PropertyEditorBase = PropertyEditorBase;
/** DataControllerBase is the base class for all Data Controllers.
 * @beta
 */
class DataControllerBase {
    async commitValue(_newValue, _record) {
        return { encounteredError: false };
    }
    async validateValue(_newValue, _record) {
        return { encounteredError: false };
    }
}
exports.DataControllerBase = DataControllerBase;
/** Manages Property Editors. Property Editors are registered with and created by the manager.
 * @beta
 */
class PropertyEditorManager {
    static registerEditor(editType, editor, editorName) {
        const fullEditorName = PropertyEditorManager.getFullEditorName(editType, editorName);
        if (PropertyEditorManager._editors.hasOwnProperty(fullEditorName)) {
            const nameOfEditor = PropertyEditorManager._editors[fullEditorName].name;
            throw Error(`PropertyEditorManager.registerEditor error: type '${fullEditorName}' already registered to '${nameOfEditor}'`);
        }
        PropertyEditorManager._editors[fullEditorName] = editor;
    }
    static getFullEditorName(editType, editorName) {
        let fullEditorName = editType;
        if (editorName)
            fullEditorName += `:${editorName}`;
        return fullEditorName;
    }
    static registerDataController(controllerName, controller) {
        if (PropertyEditorManager._dataControllers.hasOwnProperty(controllerName)) {
            throw Error(`PropertyEditorManager.registerDataController error: type '${controllerName}' already registered to '${(typeof PropertyEditorManager._dataControllers[controllerName]).toString()}'`);
        }
        PropertyEditorManager._dataControllers[controllerName] = controller;
    }
    /** @internal */
    static deregisterDataController(controllerName) {
        // istanbul ignore else
        if (PropertyEditorManager._dataControllers.hasOwnProperty(controllerName)) {
            delete PropertyEditorManager._dataControllers[controllerName];
        }
    }
    static createEditor(editType, editorName, dataControllerName) {
        const fullEditorName = PropertyEditorManager.getFullEditorName(editType, editorName);
        let editor;
        if (PropertyEditorManager._editors.hasOwnProperty(fullEditorName))
            editor = new PropertyEditorManager._editors[fullEditorName]();
        else if (PropertyEditorManager._editors.hasOwnProperty(editType))
            editor = new PropertyEditorManager._editors[editType]();
        else
            editor = new BasicPropertyEditor();
        if (dataControllerName) {
            if (PropertyEditorManager._dataControllers.hasOwnProperty(dataControllerName))
                editor.customDataController = new PropertyEditorManager._dataControllers[dataControllerName]();
            else
                throw Error(`PropertyEditorManager.createEditor error: data controller '${dataControllerName}' is not registered`);
        }
        return editor;
    }
    static hasCustomEditor(editType, editorName) {
        const fullEditorName = PropertyEditorManager.getFullEditorName(editType, editorName);
        return PropertyEditorManager._editors.hasOwnProperty(fullEditorName);
    }
}
exports.PropertyEditorManager = PropertyEditorManager;
PropertyEditorManager._editors = {};
PropertyEditorManager._dataControllers = {};
/** Basic Property Editor registered for the "text" and "string" type names.
 * It uses the [[TextEditor]] React component.
 * @beta
 */
class BasicPropertyEditor extends PropertyEditorBase {
    get reactNode() {
        return React.createElement(TextEditor_1.TextEditor, null);
    }
}
exports.BasicPropertyEditor = BasicPropertyEditor;
PropertyEditorManager.registerEditor(ui_abstract_1.StandardTypeNames.Text, BasicPropertyEditor);
PropertyEditorManager.registerEditor(ui_abstract_1.StandardTypeNames.String, BasicPropertyEditor);
//# sourceMappingURL=PropertyEditorManager.js.map