"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Properties
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyValueRendererManager = exports.PropertyContainerType = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const ArrayPropertyValueRenderer_1 = require("./renderers/value/ArrayPropertyValueRenderer");
const DoublePropertyValueRenderer_1 = require("./renderers/value/DoublePropertyValueRenderer");
const MergedPropertyValueRenderer_1 = require("./renderers/value/MergedPropertyValueRenderer");
const MultilineTextPropertyValueRenderer_1 = require("./renderers/value/MultilineTextPropertyValueRenderer");
const NavigationPropertyValueRenderer_1 = require("./renderers/value/NavigationPropertyValueRenderer");
const PrimitivePropertyValueRenderer_1 = require("./renderers/value/PrimitivePropertyValueRenderer");
const StructPropertyValueRenderer_1 = require("./renderers/value/StructPropertyValueRenderer");
const UrlPropertyValueRenderer_1 = require("./renderers/value/UrlPropertyValueRenderer");
/** Types of property containers
 * @public
 */
var PropertyContainerType;
(function (PropertyContainerType) {
    PropertyContainerType["PropertyPane"] = "pane";
    PropertyContainerType["Table"] = "table";
    PropertyContainerType["Tree"] = "tree";
})(PropertyContainerType = exports.PropertyContainerType || (exports.PropertyContainerType = {}));
/** Default implementation of property value renderer manager
 * @public
 */
class PropertyValueRendererManager {
    constructor() {
        this._propertyRenderers = new Map();
        this._defaultPrimitiveValueRenderer = new PrimitivePropertyValueRenderer_1.PrimitivePropertyValueRenderer();
        this._defaultArrayValueRenderer = new ArrayPropertyValueRenderer_1.ArrayPropertyValueRenderer();
        this._defaultStructValueRenderer = new StructPropertyValueRenderer_1.StructPropertyValueRenderer();
        this._defaultMergedValueRenderer = new MergedPropertyValueRenderer_1.MergedPropertyValueRenderer();
    }
    selectRenderer(record) {
        if (record.property.renderer && this._propertyRenderers.has(record.property.renderer.name))
            return this._propertyRenderers.get(record.property.renderer.name);
        if (this._defaultMergedValueRenderer.canRender(record))
            return this._defaultMergedValueRenderer;
        if (this._propertyRenderers.has(record.property.typename))
            return this._propertyRenderers.get(record.property.typename);
        // Use one of default renderers
        switch (record.value.valueFormat) {
            case ui_abstract_1.PropertyValueFormat.Primitive:
                return this._defaultPrimitiveValueRenderer;
            case ui_abstract_1.PropertyValueFormat.Array:
                return this._defaultArrayValueRenderer;
            case ui_abstract_1.PropertyValueFormat.Struct:
                return this._defaultStructValueRenderer;
            default:
                return undefined;
        }
    }
    /** Render property into JSX element */
    render(record, context) {
        const selectedRenderer = this.selectRenderer(record);
        if (!selectedRenderer || !selectedRenderer.canRender(record, context))
            return undefined;
        return selectedRenderer.render(record, context);
    }
    /** Register a specified property type renderer */
    registerRenderer(rendererType, propertyRenderer, overwrite = false) {
        if (!overwrite && this._propertyRenderers.has(rendererType)) {
            throw Error(`PropertyValueRendererManager.registerRenderer error: type '${rendererType}' already registered to '${propertyRenderer.constructor.name}'`);
        }
        this._propertyRenderers.set(rendererType, propertyRenderer);
    }
    /** Unregister a specified property type renderer */
    unregisterRenderer(rendererType) {
        this._propertyRenderers.delete(rendererType);
    }
    /** Get the specified property type renderer instance */
    getRegisteredRenderer(rendererType) {
        return this._propertyRenderers.get(rendererType);
    }
    /** Returns default PropertyValueRendererManager instance */
    static get defaultManager() {
        if (!this._defaultRendererManager)
            this._defaultRendererManager = new PropertyValueRendererManager();
        return this._defaultRendererManager;
    }
}
exports.PropertyValueRendererManager = PropertyValueRendererManager;
PropertyValueRendererManager.defaultManager.registerRenderer("navigation", new NavigationPropertyValueRenderer_1.NavigationPropertyValueRenderer());
PropertyValueRendererManager.defaultManager.registerRenderer("url", new UrlPropertyValueRenderer_1.UrlPropertyValueRenderer());
PropertyValueRendererManager.defaultManager.registerRenderer("double", new DoublePropertyValueRenderer_1.DoublePropertyValueRenderer());
PropertyValueRendererManager.defaultManager.registerRenderer("multiline", new MultilineTextPropertyValueRenderer_1.MultilineTextPropertyValueRenderer());
//# sourceMappingURL=ValueRendererManager.js.map