"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Properties
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyGridColumnStyleProvider = void 0;
const ui_core_1 = require("@bentley/ui-core");
/** @internal */
class PropertyGridColumnStyleProvider {
    constructor(columnInfo) {
        this._minLabelWidth = 0;
        this._minValueWidth = 0;
        this._actionButtonWidth = 0;
        if (!columnInfo) {
            this._isMinimumColumnSizeEnabled = false;
            return;
        }
        this._minLabelWidth = columnInfo.minLabelWidth;
        this._minValueWidth = columnInfo.minValueWidth;
        this._actionButtonWidth = columnInfo.actionButtonWidth;
        this._isMinimumColumnSizeEnabled = columnInfo.isMinimumColumnSizeEnabled;
    }
    get minLabelWidth() {
        return this._minLabelWidth;
    }
    get minValueWidth() {
        return this._minValueWidth;
    }
    get actionButtonWidth() {
        return this._actionButtonWidth;
    }
    get isMinimumColumnSizeEnabled() {
        return this._isMinimumColumnSizeEnabled;
    }
    getStyle(orientation, needActionButtons, ratio, needElementSeparator) {
        switch (orientation) {
            case ui_core_1.Orientation.Horizontal:
                return this.getHorizontalStyle(needActionButtons, ratio, needElementSeparator);
            case ui_core_1.Orientation.Vertical:
                return this.getVerticalStyle(needActionButtons);
            /* istanbul ignore next */
            default:
                const unhandledOrientationType = orientation; // Compile time check that all cases handled
                throw new Error(`Unhandled orientation type: ${unhandledOrientationType}. Was new orientation added ? `);
        }
    }
    /** Join columns together in sequence, filtering out undefined column definitions */
    columnStyleBuilder(columns) {
        columns = columns.filter((el) => el !== undefined);
        return { gridTemplateColumns: columns.join(" ") };
    }
    getHorizontalStyle(needActionButtons, ratio, needElementSeparator) {
        const separatorColumn = needElementSeparator ? "1px" : undefined;
        if (!this.isMinimumColumnSizeEnabled)
            return this.columnStyleBuilder([`${ratio * 100}%`, separatorColumn, "auto", needActionButtons ? "auto" : undefined]);
        const labelColumn = `minmax(${this.minLabelWidth}px, ${ratio * 100}%)`;
        const valueColumn = `minmax(${this.minValueWidth}px, 1fr)`;
        const actionButtonColumn = needActionButtons ? `${this.actionButtonWidth}px` : undefined;
        return this.columnStyleBuilder([labelColumn, separatorColumn, valueColumn, actionButtonColumn]);
    }
    getVerticalStyle(needActionButtons) {
        return this.columnStyleBuilder(["auto", needActionButtons ? "auto" : undefined]);
    }
}
exports.PropertyGridColumnStyleProvider = PropertyGridColumnStyleProvider;
//# sourceMappingURL=PropertyGridColumns.js.map