"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilteringPropertyDataProvider = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const PropertyDataProvider_1 = require("../PropertyDataProvider");
/**
 * IPropertyDataProvider implementation which will filter wrapped provider PropertyData using passed IPropertyDataFilterer.
 * @beta
 */
class FilteringPropertyDataProvider {
    constructor(_dataProvider, _filterer) {
        this._dataProvider = _dataProvider;
        this._filterer = _filterer;
        this.onDataChanged = new PropertyDataProvider_1.PropertyDataChangeEvent();
        this._filteredPropertyData = undefined;
        this._disposeFilterChangedListener = this._filterer.onFilterChanged.addListener(() => { this._filteredPropertyData = undefined; this.onDataChanged.raiseEvent(); });
        this._disposeDataChangedListener = this._dataProvider.onDataChanged.addListener(() => { this._filteredPropertyData = undefined; this.onDataChanged.raiseEvent(); });
    }
    dispose() {
        this._disposeDataChangedListener();
        this._disposeFilterChangedListener();
    }
    async getData() {
        if (this._filteredPropertyData) {
            return this._filteredPropertyData;
        }
        this._filteredPropertyData = this._dataProvider.getData().then(async (propertyData) => {
            return ((this._filterer.isActive) ?
                this.filterPropertyData(propertyData) : propertyData);
        });
        return this._filteredPropertyData;
    }
    async filterPropertyData(propertyData) {
        const { categories, records, label, description } = Object.assign({}, propertyData);
        const { filteredCategories, filteredRecords, matchesCount, filteredResultMatches, filteredTypes } = await matchHierarchy(this._filterer, categories, records, undefined);
        const uniqueTypes = [...new Set(filteredTypes)];
        const getMatchByIndex = (index) => {
            let activeMatch;
            if (index <= 0)
                return undefined;
            let i = 1;
            for (const record of filteredResultMatches) {
                const fullMatchesCount = record.matchesCount;
                if (index < i + fullMatchesCount) {
                    activeMatch = {
                        highlightedItemIdentifier: record.id,
                        highlightIndex: index - i,
                    };
                    break;
                }
                i += fullMatchesCount;
            }
            return activeMatch;
        };
        return {
            label,
            description,
            categories: filteredCategories,
            records: filteredRecords,
            reusePropertyDataState: false,
            matchesCount,
            getMatchByIndex,
            filteredTypes: uniqueTypes,
        };
    }
}
exports.FilteringPropertyDataProvider = FilteringPropertyDataProvider;
async function matchHierarchy(filterer, categories, records, newRecords, parentMatchInfo) {
    var _a, _b, _c, _d, _e;
    const newCategories = [];
    newRecords = newRecords !== null && newRecords !== void 0 ? newRecords : {};
    let matchesCount = 0;
    let allFilteredResultMatches = [];
    let allFilteredTypes = [];
    for (const category of categories) {
        const matchInfo = await filterer.categoryMatchesFilter(category, []);
        if (matchInfo.matchesFilter) {
            allFilteredResultMatches.push({ id: category.name, matchesCount: (_a = matchInfo.matchesCount) !== null && _a !== void 0 ? _a : 0 });
            allFilteredTypes = allFilteredTypes.concat((_b = matchInfo.filteredTypes) !== null && _b !== void 0 ? _b : []);
            matchesCount += (_c = matchInfo.matchesCount) !== null && _c !== void 0 ? _c : 0;
        }
        const childRecords = (_d = records[category.name]) !== null && _d !== void 0 ? _d : [];
        const { filteredRecords, matchesCount: count, filteredResultMatches, filteredTypes } = await matchRecordHierarchy(filterer, childRecords, []);
        allFilteredResultMatches = allFilteredResultMatches.concat(filteredResultMatches);
        allFilteredTypes = allFilteredTypes.concat(filteredTypes);
        matchesCount += count;
        const parentInfo = (parentMatchInfo === null || parentMatchInfo === void 0 ? void 0 : parentMatchInfo.matchesFilter) ? parentMatchInfo : matchInfo;
        const childCategories = (_e = category.childCategories) !== null && _e !== void 0 ? _e : [];
        const { filteredCategories, matchesCount: childCount, filteredResultMatches: childFilteredResultMatches, filteredTypes: childFilteredTypes } = await matchHierarchy(filterer, childCategories, records, newRecords, parentInfo);
        allFilteredResultMatches = allFilteredResultMatches.concat(childFilteredResultMatches);
        allFilteredTypes = allFilteredTypes.concat(childFilteredTypes);
        matchesCount += childCount;
        let expand = !(count === 0 && childCount === 0);
        if (matchInfo.matchesFilter || ((parentMatchInfo === null || parentMatchInfo === void 0 ? void 0 : parentMatchInfo.matchesFilter) && parentMatchInfo.shouldForceIncludeDescendants)) {
            newRecords[category.name] = records[category.name];
            if (matchInfo.matchesFilter) {
                expand = true;
            }
            const newCategory = copyPropertyCategory(category, filteredCategories, expand);
            newCategories.push(newCategory);
        }
        if ((filteredRecords.length !== 0 || filteredCategories.length !== 0) && !matchInfo.matchesFilter && !((parentMatchInfo === null || parentMatchInfo === void 0 ? void 0 : parentMatchInfo.shouldExpandNodeParents) && parentMatchInfo.matchesFilter)) {
            const newCategory = copyPropertyCategory(category, filteredCategories, expand);
            newRecords[newCategory.name] = filteredRecords;
            newCategories.push(newCategory);
        }
    }
    return { filteredCategories: newCategories, filteredRecords: newRecords, matchesCount, filteredResultMatches: allFilteredResultMatches, filteredTypes: allFilteredTypes };
}
async function matchRecordHierarchy(filterer, records, parents, forceIncludeItem = false) {
    var _a, _b;
    const newMatchedRecords = [];
    let expandParent = false; // property passed back to parent to indicate whether it should be expanded
    let matchesCount = 0;
    let filteredResultMatches = [];
    let filteredTypes = [];
    for (const record of records) {
        const matchInfo = await filterer.recordMatchesFilter(record, parents);
        const shouldForceIncludeDescendants = forceIncludeItem || matchInfo.shouldForceIncludeDescendants;
        const { filteredRecords: filteredChildren, shouldExpandNodeParents, matchesCount: childrenMatchesCount, filteredResultMatches: childrenResultMatches, filteredTypes: childrenFilteredTypes } = await matchRecordHierarchy(filterer, record.getChildrenRecords(), [...parents, record], shouldForceIncludeDescendants);
        filteredResultMatches = filteredResultMatches.concat(childrenResultMatches);
        filteredTypes = [...filteredTypes, ...childrenFilteredTypes];
        matchesCount += childrenMatchesCount;
        // if child hierarchy indicates parents should be expanded, then it should be passed back up to the top parent
        expandParent = expandParent || shouldExpandNodeParents;
        let newRecord;
        if (matchInfo.matchesFilter) {
            filteredTypes = [...filteredTypes, ...((_a = matchInfo.filteredTypes) !== null && _a !== void 0 ? _a : [])];
            const recordMatchesCount = (_b = matchInfo.matchesCount) !== null && _b !== void 0 ? _b : 0;
            matchesCount += recordMatchesCount;
            // Item is included if it matches filter
            // If at least one matched record from records list needs its parent expanded,
            // then `expandParent` should be set to true
            expandParent = expandParent || !!matchInfo.shouldExpandNodeParents;
            newRecord = copyPropertyRecord(record, filteredChildren, shouldExpandNodeParents);
            filteredResultMatches.push({ id: newRecord.property.name, matchesCount: recordMatchesCount });
        }
        else if (filteredChildren.length !== 0 || forceIncludeItem) {
            // Item is also included if it has at least one matched child or is forcefully included by its parent
            newRecord = copyPropertyRecord(record, filteredChildren, shouldExpandNodeParents);
        }
        if (newRecord)
            newMatchedRecords.push(newRecord);
    }
    return { filteredRecords: newMatchedRecords, shouldExpandNodeParents: expandParent, matchesCount, filteredResultMatches, filteredTypes };
}
function copyPropertyCategory(propertyCategory, newChildren, expand) {
    return Object.assign(Object.assign({}, propertyCategory), { childCategories: newChildren, expand });
}
function copyPropertyRecord(record, newChildren, autoExpand) {
    const r = record.copyWithNewValue(createNewPropertyValue(record.value, newChildren));
    r.autoExpand = autoExpand;
    return r;
}
function createNewPropertyValue(value, newChildren) {
    switch (value.valueFormat) {
        case ui_abstract_1.PropertyValueFormat.Primitive:
            return Object.assign({}, value);
        case ui_abstract_1.PropertyValueFormat.Array:
            return {
                valueFormat: ui_abstract_1.PropertyValueFormat.Array,
                itemsTypeName: value.itemsTypeName,
                items: newChildren,
            };
        case ui_abstract_1.PropertyValueFormat.Struct:
            const members = {};
            newChildren.forEach((child) => members[child.property.name] = child);
            return {
                valueFormat: ui_abstract_1.PropertyValueFormat.Struct,
                members,
            };
    }
}
//# sourceMappingURL=FilteringDataProvider.js.map