"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyGrid
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompositePropertyDataFilterer = exports.CompositeFilterType = void 0;
const PropertyDataFiltererBase_1 = require("./PropertyDataFiltererBase");
/**
 * Logical operator for composite filterer.
 * @beta
 */
var CompositeFilterType;
(function (CompositeFilterType) {
    CompositeFilterType[CompositeFilterType["And"] = 0] = "And";
    CompositeFilterType[CompositeFilterType["Or"] = 1] = "Or";
})(CompositeFilterType = exports.CompositeFilterType || (exports.CompositeFilterType = {}));
/**
 * Composite PropertyData filter which can join two filters using logic operators
 * @beta
 */
class CompositePropertyDataFilterer extends PropertyDataFiltererBase_1.PropertyDataFiltererBase {
    constructor(_leftFilterer, _operator, _rightFilterer) {
        super();
        this._leftFilterer = _leftFilterer;
        this._operator = _operator;
        this._rightFilterer = _rightFilterer;
        this._leftFilterer.onFilterChanged.addListener(() => this.onFilterChanged.raiseEvent());
        this._rightFilterer.onFilterChanged.addListener(() => this.onFilterChanged.raiseEvent());
    }
    get isActive() { return this._leftFilterer.isActive || this._rightFilterer.isActive; }
    async recordMatchesFilter(node, parents) {
        const [lhs, rhs] = await Promise.all([
            this._leftFilterer.recordMatchesFilter(node, parents),
            this._rightFilterer.recordMatchesFilter(node, parents),
        ]);
        return this.getFiltererResult(lhs, rhs);
    }
    async categoryMatchesFilter(node, parents) {
        const [lhs, rhs] = await Promise.all([
            this._leftFilterer.categoryMatchesFilter(node, parents),
            this._rightFilterer.categoryMatchesFilter(node, parents),
        ]);
        return this.getFiltererResult(lhs, rhs);
    }
    getFiltererResult(lhs, rhs) {
        const matchesFilter = (this._operator === CompositeFilterType.And) ? (lhs.matchesFilter && rhs.matchesFilter) : (lhs.matchesFilter || rhs.matchesFilter);
        if (!matchesFilter)
            return { matchesFilter: false };
        const filteredTypes = joinNullableArrays(lhs.matchesFilter ? lhs.filteredTypes : undefined, rhs.matchesFilter ? rhs.filteredTypes : undefined);
        return {
            matchesFilter: true,
            shouldExpandNodeParents: lhs.shouldExpandNodeParents || rhs.shouldExpandNodeParents,
            shouldForceIncludeDescendants: lhs.shouldForceIncludeDescendants || rhs.shouldForceIncludeDescendants,
            matchesCount: (sumNullableNumbers(lhs.matchesCount, rhs.matchesCount)),
            filteredTypes,
        };
    }
}
exports.CompositePropertyDataFilterer = CompositePropertyDataFilterer;
function sumNullableNumbers(lhs, rhs) {
    if (undefined === lhs && undefined === rhs)
        return undefined;
    return (lhs !== null && lhs !== void 0 ? lhs : 0) + (rhs !== null && rhs !== void 0 ? rhs : 0);
}
function joinNullableArrays(lhs, rhs) {
    if (undefined === lhs && undefined === rhs)
        return undefined;
    return [...lhs !== null && lhs !== void 0 ? lhs : [], ...rhs !== null && rhs !== void 0 ? rhs : []];
}
//# sourceMappingURL=CompositePropertyDataFilterer.js.map