"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyGrid
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePropertyGridModel = exports.usePropertyGridEventHandler = exports.usePropertyGridModelSource = exports.usePropertyData = void 0;
const react_1 = require("react");
const UseDebouncedAsyncValue_1 = require("../../common/UseDebouncedAsyncValue");
const PropertyGridCommons_1 = require("../component/PropertyGridCommons");
const MutableGridItemFactory_1 = require("./flat-items/MutableGridItemFactory");
const PropertyGridEventHandler_1 = require("./PropertyGridEventHandler");
const PropertyGridModelSource_1 = require("./PropertyGridModelSource");
/**
 * Custom hook that gets propertyData from data provider and subscribes to further data changes.
 * Returned property data has links.onClick replaced by passed onPropertyLinkClick or default implementation
 * @beta
 */
function usePropertyData(props) {
    const { dataProvider, onPropertyLinkClick } = Object.assign({}, props);
    const [forcedUpdate, triggerForcedUpdate] = react_1.useReducer(() => ({}), {});
    react_1.useEffect(() => {
        return dataProvider.onDataChanged.addListener(() => {
            triggerForcedUpdate();
        });
    }, [dataProvider]);
    // ForcedUpdate is added to dependency list to re-memo getData promise when onDataChanged emits an event.
    // eslint-disable-next-line react-hooks/exhaustive-deps
    const delayedReturn = UseDebouncedAsyncValue_1.useDebouncedAsyncValue(react_1.useCallback(async () => dataProvider.getData(), [dataProvider, forcedUpdate]));
    const { value, inProgress } = delayedReturn;
    return react_1.useMemo(() => {
        if (value) {
            for (const categoryName in value.records) {
                // Support for deprecated onPropertyLinkClick
                // istanbul ignore else
                if (onPropertyLinkClick && value.records.hasOwnProperty(categoryName))
                    PropertyGridCommons_1.PropertyGridCommons.assignRecordClickHandlers(value.records[categoryName], onPropertyLinkClick);
            }
        }
        return { value, inProgress };
    }, [value, inProgress, onPropertyLinkClick]);
}
exports.usePropertyData = usePropertyData;
/**
 * Custom hook that creates a PropertyGridModelSource and subscribes it to data updates from the data provider.
 * @beta
 */
function usePropertyGridModelSource(props) {
    const { value: propertyData } = usePropertyData(props);
    const { dataProvider } = Object.assign({}, props);
    // Model source needs to be recreated if data provider changes
    // eslint-disable-next-line react-hooks/exhaustive-deps
    const modelSource = react_1.useMemo(() => new PropertyGridModelSource_1.PropertyGridModelSource(new MutableGridItemFactory_1.MutableGridItemFactory()), [dataProvider]);
    react_1.useEffect(() => {
        if (propertyData)
            modelSource.setPropertyData(propertyData);
    }, [modelSource, propertyData]);
    return modelSource;
}
exports.usePropertyGridModelSource = usePropertyGridModelSource;
/**
 * Custom hook that creates memoized version of PropertyGridEventHandler that modifies given modelSource
 * @beta
 */
function usePropertyGridEventHandler(props) {
    return react_1.useMemo(() => new PropertyGridEventHandler_1.PropertyGridEventHandler(props.modelSource), [props.modelSource]);
}
exports.usePropertyGridEventHandler = usePropertyGridEventHandler;
/**
 * Custom hook that automatically listens and retrieves latest model from model source
 * @beta
 */
function usePropertyGridModel(props) {
    const { modelSource } = Object.assign({}, props);
    const [model, setModel] = react_1.useState();
    react_1.useEffect(() => {
        const modelChanged = () => {
            setModel(modelSource.getModel());
        };
        modelChanged();
        return modelSource.onModelChanged.addListener(modelChanged);
    }, [modelSource]);
    return model;
}
exports.usePropertyGridModel = usePropertyGridModel;
//# sourceMappingURL=PropertyGridHooks.js.map