"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MutablePropertyGridModel = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyGrid
 */
const immer_1 = require("immer");
/**
 * Implementation of PropertyGridModel for working with and converting PropertyData to mutable FlatGridItems
 * @beta
 */
class MutablePropertyGridModel {
    constructor(propertyData, _gridItemFactory) {
        this._gridItemFactory = _gridItemFactory;
        this[_a] = true;
        this._categories = propertyData.categories.map((category) => this._gridItemFactory.createGridCategory(category, propertyData.records));
    }
    /**
     * Retrieves grid item from model.
     * @param selectionKey unique key for identifying item to retrieve.
     */
    getItem(selectionKey) {
        const item = this.findItem(this._categories, selectionKey);
        if (!item)
            throw new Error(`Grid item at provided key not found: ${selectionKey}`);
        return item;
    }
    /**
     * Walks the grid item hierarchy and finds item matching key
     * @param items items and their descendants to check
     * @param selectionKey unique key for identifying item
     * @returns FlatGridItem if items with key exists, undefined otherwise
     */
    findItem(items, selectionKey) {
        for (const item of items) {
            // Each items key is prefixed with it's parent key, so we can ignore the ones that aren't prefixed to reduce checking unnecessary branches.
            if (!selectionKey.startsWith(item.selectionKey))
                continue;
            if (item.selectionKey === selectionKey)
                return item;
            const foundItem = this.findItem(item.getChildren(), selectionKey);
            if (foundItem)
                return foundItem;
        }
        return undefined;
    }
    /**
     * Gets all GridCategoryItems that do not have parent categories.
     * @returns array of GridCategoryItems
     */
    getRootCategories() {
        return this._categories;
    }
    /**
     * Gets an array of all FlatGridItems.
     * @returns 1-Dimensional array of GridCategories and CategorizedProperties
     */
    getFlatGrid() {
        const flatGrid = [];
        this._categories.forEach((category) => flatGrid.push(...category.getDescendantsAndSelf()));
        return flatGrid;
    }
    /**
     * Gets an array of all currently visible FlatGridItems.
     * @returns 1-Dimensional array of GridCategories and CategorizedProperties
     */
    getVisibleFlatGrid() {
        const visibleItems = [];
        this._categories.forEach((category) => visibleItems.push(...category.getVisibleDescendantsAndSelf()));
        return visibleItems;
    }
}
exports.MutablePropertyGridModel = MutablePropertyGridModel;
_a = immer_1.immerable;
//# sourceMappingURL=PropertyGridModel.js.map