"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyGrid
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyGridModelSource = void 0;
const immer_1 = require("immer");
const PropertyGridModel_1 = require("./PropertyGridModel");
const PropertyGridModelChangeEvent_1 = require("./PropertyGridModelChangeEvent");
/**
 * Controls property grid model.
 * It is used to modify property grid model model and inform when the model changes.
 * @beta
 */
class PropertyGridModelSource {
    constructor(_gridFactory) {
        this._gridFactory = _gridFactory;
        /** Event that is emitted every time property model is changed. */
        this.onModelChanged = new PropertyGridModelChangeEvent_1.PropertyGridModelChangeEvent();
    }
    getFullModelMap(fullModel) {
        const fullModelMap = new Map();
        for (const item of fullModel) {
            const key = item.selectionKey;
            fullModelMap.set(key, item);
        }
        return fullModelMap;
    }
    moveOldModelState(oldModel, newModel) {
        const oldModelMap = this.getFullModelMap(oldModel.getFlatGrid());
        const flatGrid = newModel.getFlatGrid();
        flatGrid.forEach((gridItem) => {
            const oldGridItem = oldModelMap.get(gridItem.selectionKey);
            // istanbul ignore else
            if (oldGridItem)
                gridItem.isExpanded = oldGridItem.isExpanded;
        });
    }
    setPropertyData(data) {
        const newModel = new PropertyGridModel_1.MutablePropertyGridModel(data, this._gridFactory);
        if (this._model !== undefined && data.reusePropertyDataState)
            this.moveOldModelState(this._model, newModel);
        this._model = newModel;
        this.onModelChanged.raiseEvent();
    }
    /**
     * Modifies property grid model using provided callback.
     * If changes to model are detected, onModelChanged emits an event to all subscribers.
     */
    modifyModel(callback) {
        if (!this._model)
            return;
        this._model = immer_1.produce(this._model, callback);
        this.onModelChanged.raiseEvent();
    }
    /** Returns property grid model. */
    getModel() {
        return this._model;
    }
}
exports.PropertyGridModelSource = PropertyGridModelSource;
//# sourceMappingURL=PropertyGridModelSource.js.map