"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MutableCategorizedProperty = exports.MutableFlatPropertyGridItem = exports.FlatGridItemType = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyGrid
 */
const immer_1 = require("immer");
const shortid_1 = __importDefault(require("shortid"));
const ui_abstract_1 = require("@bentley/ui-abstract");
/**
 * Types of FlatGridItems for differentiating what property FlatGridItem is wrapping.
 * Category for PropertyCategory
 * (Primitive, Array, Struct) for PropertyRecord with valueFormats: (Primitive, Array, Struct)
 * @beta
 */
var FlatGridItemType;
(function (FlatGridItemType) {
    FlatGridItemType[FlatGridItemType["Category"] = 0] = "Category";
    FlatGridItemType[FlatGridItemType["Primitive"] = 1] = "Primitive";
    FlatGridItemType[FlatGridItemType["Array"] = 2] = "Array";
    FlatGridItemType[FlatGridItemType["Struct"] = 3] = "Struct";
})(FlatGridItemType = exports.FlatGridItemType || (exports.FlatGridItemType = {}));
/**
 * Base class for all FlatPropertyGrid items
 * @beta
 */
class MutableFlatPropertyGridItem {
    constructor(_depth, _parentSelectionKey, _parentCategorySelectionKey) {
        this._depth = _depth;
        this._parentSelectionKey = _parentSelectionKey;
        this._parentCategorySelectionKey = _parentCategorySelectionKey;
        this[_a] = true;
        this.key = shortid_1.default.generate();
        this._isExpanded = false;
        this._lastInNumberOfCategories = 0;
        this._isLastInRootCategory = false;
        if (this._depth < 0)
            throw new Error("Depth cannot be negative");
    }
    get depth() {
        return this._depth;
    }
    get parentSelectionKey() {
        return this._parentSelectionKey;
    }
    get parentCategorySelectionKey() {
        return this._parentCategorySelectionKey;
    }
    get isExpanded() {
        return this._isExpanded;
    }
    set isExpanded(value) {
        this._isExpanded = value;
    }
    getDescendants() {
        const descendants = [];
        this.getChildren().forEach((child) => descendants.push(...child.getDescendantsAndSelf()));
        return descendants;
    }
    /**
     * Gets a flat list of all FlatGridItems beneath this flat grid item including itself in depth first visiting order.
     */
    getDescendantsAndSelf() {
        return [this.getSelf(), ...this.getDescendants()];
    }
    /**
     * Gets a flat list of visible FlatGridItems beneath this flat grid item.
     */
    getVisibleDescendants() {
        const descendants = [];
        if (this.isExpanded)
            this.getChildren().forEach((child) => descendants.push(...child.getVisibleDescendantsAndSelf()));
        return descendants;
    }
    /**
     * Gets a flat list of visible FlatGridItems beneath this flat grid item and itself in depth first visiting order.
     */
    getVisibleDescendantsAndSelf() {
        return [this.getSelf(), ...this.getVisibleDescendants()];
    }
    /**
     * @returns self if item is not expanded, last visible descendant of this item otherwise.
     */
    getLastVisibleDescendantOrSelf() {
        if (!this.isExpanded)
            return this.getSelf();
        const children = this.getChildren();
        if (children.length < 1)
            return this.getSelf();
        return children[children.length - 1].getLastVisibleDescendantOrSelf();
    }
    /**
   * Sets lastInNumberOfCategories value and sends it down to this items last child
   * @internal
   */
    get lastInNumberOfCategories() {
        if (this.isExpanded && this.getChildren().length > 0)
            return 0;
        return this._lastInNumberOfCategories;
    }
    set lastInNumberOfCategories(value) {
        this._lastInNumberOfCategories = value;
        const children = this.getChildren();
        if (children.length < 1)
            return;
        children[children.length - 1].lastInNumberOfCategories = value;
    }
    /**
     * Gets and sets isLastInRootCategory value and sends it down to this items last child
     * @internal
     */
    get isLastInRootCategory() {
        if (this.isExpanded && this.getChildren().length > 0)
            return false;
        return this._isLastInRootCategory;
    }
    set isLastInRootCategory(value) {
        this._isLastInRootCategory = value;
        const children = this.getChildren();
        if (children.length < 1)
            return;
        children[children.length - 1].isLastInRootCategory = value;
    }
}
exports.MutableFlatPropertyGridItem = MutableFlatPropertyGridItem;
_a = immer_1.immerable;
/**
 * Base class for all Mutable CategorizedProperties
 * @beta
 */
class MutableCategorizedProperty extends MutableFlatPropertyGridItem {
    constructor(type, record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel) {
        super(depth, parentSelectionKey, parentCategorySelectionKey);
        const recordType = this.valueTypeToFlatGridType(record.value.valueFormat);
        if (recordType !== type) {
            const expectedTypeStr = FlatGridItemType[type];
            const actualTypeStr = FlatGridItemType[recordType];
            throw Error(`Record with incorrect value format passed to property: expected ${expectedTypeStr}, got ${actualTypeStr}`);
        }
        const overridenProperty = this.makeOverridenProperty(record.property, overrideName, overrideDisplayLabel);
        this._derivedRecord = this.makeDerivedRecord(record, overridenProperty);
        this._selectionKey = `${this.parentSelectionKey}_${this.derivedRecord.property.name}`;
        this._isExpanded = !!record.autoExpand;
    }
    /**
     * Maps PropertyRecord valueFormat to FlatGridItemType
     * @param valueType valueFormat to map
     * @returns mapped FlatGridItemType
     */
    valueTypeToFlatGridType(valueType) {
        switch (valueType) {
            case ui_abstract_1.PropertyValueFormat.Primitive:
                return FlatGridItemType.Primitive;
            case ui_abstract_1.PropertyValueFormat.Array:
                return FlatGridItemType.Array;
            case ui_abstract_1.PropertyValueFormat.Struct:
                return FlatGridItemType.Struct;
            /* istanbul ignore next */
            default:
                const unhandledType = valueType;
                throw Error(`Property Value Format not handled: ${unhandledType}`);
        }
    }
    /**
     * Make new property description with overriden fields.
     * @param propertyDescription property description to override.
     * @param overrideName property description name to override.
     * @param overrideDisplay  property description display name to override.
     */
    makeOverridenProperty(propertyDescription, overrideName, overrideDisplay) {
        const _b = Object.assign({}, propertyDescription), { name, displayLabel } = _b, property = __rest(_b, ["name", "displayLabel"]);
        const newName = overrideName !== null && overrideName !== void 0 ? overrideName : name;
        const newDisplayLabel = overrideDisplay !== null && overrideDisplay !== void 0 ? overrideDisplay : displayLabel;
        return Object.assign(Object.assign({}, property), { name: newName, displayLabel: newDisplayLabel });
    }
    /**
     * Gets derived property record that has it's property description field overriden
     */
    makeDerivedRecord(record, overridenPropertyDescription) {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const { value, property } = record, others = __rest(record, ["value", "property"]);
        const newRecord = new ui_abstract_1.PropertyRecord(value, overridenPropertyDescription);
        return Object.assign(newRecord, others);
    }
    getSelf() {
        return this;
    }
    get parentCategorySelectionKey() {
        return super.parentCategorySelectionKey;
    }
    get parentSelectionKey() {
        return super.parentSelectionKey;
    }
    /**
     * Unique selection key made of parent selectionKey and this property name.
     */
    get selectionKey() {
        return this._selectionKey;
    }
    get label() {
        return this._derivedRecord.property.displayLabel;
    }
    /**
     * Record with overriden property description.
     */
    get derivedRecord() {
        return this._derivedRecord;
    }
}
exports.MutableCategorizedProperty = MutableCategorizedProperty;
//# sourceMappingURL=MutableFlatGridItem.js.map