"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.MutableGridCategory = void 0;
const MutableFlatGridItem_1 = require("./MutableFlatGridItem");
/**
 * Mutable wrapper object for PropertyCategory which provides methods for working with and managing category and record children hierarchies
 * @beta
 */
class MutableGridCategory extends MutableFlatGridItem_1.MutableFlatPropertyGridItem {
    constructor(category, recordsDict, gridItemFactory, parentSelectionKey, depth = 0) {
        var _a, _b;
        super(depth, parentSelectionKey, parentSelectionKey);
        this._category = { name: category.name, label: category.label, expand: category.expand };
        if (parentSelectionKey !== undefined)
            this._selectionKey = `${parentSelectionKey}_${category.name}`;
        else
            this._selectionKey = category.name;
        this._isExpanded = category.expand;
        // Even though categories are nested and have their own depth, categorized properties depth is counted starting with the parent category.
        const categoryRecords = (_a = recordsDict[category.name]) !== null && _a !== void 0 ? _a : [];
        this._children = categoryRecords.map((value) => gridItemFactory.createCategorizedProperty(value, this.selectionKey, this.selectionKey, 0));
        const childCategories = (_b = category.childCategories) !== null && _b !== void 0 ? _b : [];
        this._childCategories = childCategories.map((childCategory) => gridItemFactory.createGridCategory(childCategory, recordsDict, this.selectionKey, this.depth + 1));
        this._children.push(...this._childCategories);
        // Assign lastInNumberOfCategories and isLastInRootCategory for entire children hierarchy
        this.lastInNumberOfCategories = 0;
        this.isLastInRootCategory = this.isRootCategory;
    }
    /**
     * Category is considered to be root category if its depth is 0
     */
    get isRootCategory() {
        return this.depth === 0;
    }
    get selectionKey() {
        return this._selectionKey;
    }
    get type() {
        return MutableFlatGridItem_1.FlatGridItemType.Category;
    }
    get name() {
        return this._category.name;
    }
    get label() {
        return this._category.label;
    }
    get derivedCategory() {
        return Object.assign(Object.assign({}, this._category), { expand: this.isExpanded });
    }
    getSelf() {
        return this;
    }
    getChildren() {
        return this._children;
    }
    getChildCategories() {
        return this._childCategories;
    }
    /**
     * Gets a flat list of all categories beneath this category and itself in depth first visiting order.
     */
    getDescendantCategoriesAndSelf() {
        const descendants = [];
        this._childCategories.forEach((value) => descendants.push(...value.getDescendantCategoriesAndSelf()));
        return [this, ...descendants];
    }
    /**
     * Gets and Sets lastInNumberOfCategories.
     * Setter increments set value by one to account self as category.
     * New value is sent down to this items last child
     * @internal
     */
    get lastInNumberOfCategories() {
        return super.lastInNumberOfCategories;
    }
    set lastInNumberOfCategories(value) {
        super.lastInNumberOfCategories = value + 1;
    }
}
exports.MutableGridCategory = MutableGridCategory;
//# sourceMappingURL=MutableGridCategory.js.map