"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MutableGridItemFactory = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module PropertyGrid
 */
const ui_abstract_1 = require("@bentley/ui-abstract");
const MutableCategorizedArrayProperty_1 = require("./MutableCategorizedArrayProperty");
const MutableCategorizedPrimitiveProperty_1 = require("./MutableCategorizedPrimitiveProperty");
const MutableCategorizedStructProperty_1 = require("./MutableCategorizedStructProperty");
const MutableCustomGridCategory_1 = require("./MutableCustomGridCategory");
const MutableGridCategory_1 = require("./MutableGridCategory");
/**
 * Implementation of IMutableGridItemFactory for creating MutableGridItem objects.
 * @beta
 */
class MutableGridItemFactory {
    createPrimitiveProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel) {
        return new MutableCategorizedPrimitiveProperty_1.MutableCategorizedPrimitiveProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel);
    }
    createArrayProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel) {
        return new MutableCategorizedArrayProperty_1.MutableCategorizedArrayProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, this, overrideName, overrideDisplayLabel);
    }
    createStructProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel) {
        return new MutableCategorizedStructProperty_1.MutableCategorizedStructProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, this, overrideName, overrideDisplayLabel);
    }
    /**
     * Converts record into FlatGridItem.
     * @param record record to convert to a FlatGridItem.
     * @param parentSelectionKey parent selection key of provided `record`.
     * @param parentCategorySelectionKey parent category selection key of provided `record`.
     * @param depth current depth counting from parent category.
     * @param overrideName property name that overrides original record property name.
     * @param overrideDisplayLabel property display label that overrides original record property display label.
     * @returns converted CategorizedProperty.
     */
    createCategorizedProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel) {
        const valueFormat = record.value.valueFormat;
        switch (valueFormat) {
            case ui_abstract_1.PropertyValueFormat.Primitive:
                return this.createPrimitiveProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel);
            case ui_abstract_1.PropertyValueFormat.Array:
                return this.createArrayProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel);
            case ui_abstract_1.PropertyValueFormat.Struct:
                return this.createStructProperty(record, parentSelectionKey, parentCategorySelectionKey, depth, overrideName, overrideDisplayLabel);
            /* istanbul ignore next */
            default:
                const unhandledRecordType = valueFormat;
                throw new Error(`Unhandled property record type. Was a new type added? ${unhandledRecordType}`);
        }
    }
    /**
     * Converts category into GridCategoryItem.
     * @param category PropertyCategory to convert.
     * @param recordsDict dictionary of category records.
     * @param parentSelectionKey parent selection key of this category.
     * @param depth current depth counting from root category.
     * @returns converted GridCategory.
     */
    createGridCategory(category, recordsDict, parentSelectionKey, depth) {
        if (category.renderer !== undefined)
            return new MutableCustomGridCategory_1.MutableCustomGridCategory(category, recordsDict, this, parentSelectionKey, depth !== null && depth !== void 0 ? depth : 0);
        if (parentSelectionKey !== undefined && depth !== undefined)
            return new MutableGridCategory_1.MutableGridCategory(category, recordsDict, this, parentSelectionKey, depth);
        return new MutableGridCategory_1.MutableGridCategory(category, recordsDict, this);
    }
}
exports.MutableGridItemFactory = MutableGridItemFactory;
//# sourceMappingURL=MutableGridItemFactory.js.map