"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module QuantityFormat
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnitDescr = void 0;
const React = __importStar(require("react"));
const ui_core_1 = require("@bentley/ui-core");
async function getUnitConversionData(possibleUnits, toUnit, unitsProvider) {
    const unitConversionEntries = possibleUnits.map(async (unit) => {
        const conversion = await unitsProvider.getConversion(unit, toUnit);
        return { conversion, unitProps: unit };
    });
    return unitConversionEntries;
}
async function getPossibleUnits(parentUnit, unitsProvider, ensureCompatibleComposite) {
    const phenomenon = parentUnit.phenomenon;
    const possibleUnits = await unitsProvider.getUnitsByFamily(phenomenon);
    if (!ensureCompatibleComposite)
        return possibleUnits;
    const conversionPromises = await getUnitConversionData(possibleUnits, parentUnit, unitsProvider);
    const conversionEntries = await Promise.all(conversionPromises);
    // sort the entries so the best potential sub unit will be the first one in the array
    return conversionEntries.filter((entry) => ((entry.unitProps.system === parentUnit.system) && (entry.conversion.factor < 1)))
        .sort((a, b) => b.conversion.factor - a.conversion.factor)
        .map((value) => value.unitProps);
}
function getUnitName(fullUnitName) {
    const nameParts = fullUnitName.split(/[.:]/);
    // istanbul ignore else
    if (nameParts.length > 0)
        return nameParts[nameParts.length - 1];
    // istanbul ignore next
    throw Error("Bad unit name encountered");
}
/** Component use to display dropdown list of possible units.
 * @internal
 */
function UnitDescr(props) {
    const { name, label, parentUnitName, index, onUnitChange, onLabelChange, readonly, unitsProvider } = props;
    const [unitOptions, setUnitOptions] = React.useState([{ value: name, label: getUnitName(name) }]);
    const [currentUnit, setCurrentUnit] = React.useState({ name, label });
    const isMounted = React.useRef(false);
    React.useEffect(() => {
        isMounted.current = true;
        return () => {
            isMounted.current = false;
        };
    });
    React.useEffect(() => {
        async function fetchAllowableUnitSelections() {
            const currentUnitProps = await unitsProvider.findUnitByName(name);
            const parentUnit = await unitsProvider.findUnitByName(parentUnitName ? parentUnitName : name);
            // istanbul ignore else
            if (parentUnit && currentUnitProps) {
                let potentialSubUnit;
                const potentialUnits = await getPossibleUnits(parentUnit, unitsProvider, index !== 0);
                // istanbul ignore else
                if (index < 3) {
                    const potentialSubUnits = await getPossibleUnits(currentUnitProps, unitsProvider, true);
                    if (potentialSubUnits.length)
                        potentialSubUnit = potentialSubUnits[0];
                }
                const options = (potentialUnits.length > 0) ?
                    potentialUnits.map((unitValue) => {
                        return { value: `${unitValue.name}:${unitValue.label}`, label: getUnitName(unitValue.name) };
                    }).sort((a, b) => a.label.localeCompare(b.label))
                    : /* istanbul ignore next */
                        [{ value: `${currentUnitProps.name}:${currentUnitProps.label}`, label: getUnitName(name) }];
                if (potentialSubUnit) {
                    // construct an entry that will provide the name and label of the unit to add
                    options.push({ value: `ADDSUBUNIT:${potentialSubUnit.name}:${potentialSubUnit.label}`, label: "Add sub-unit" });
                }
                if (index !== 0)
                    options.push({ value: "REMOVEUNIT", label: "Remove" });
                // istanbul ignore else
                if (isMounted.current) {
                    setUnitOptions(options);
                    setCurrentUnit(currentUnitProps);
                }
            }
        }
        fetchAllowableUnitSelections(); // eslint-disable-line @typescript-eslint/no-floating-promises
    }, [index, label, name, parentUnitName, unitsProvider]);
    const handleOnUnitChange = React.useCallback((e) => {
        e.preventDefault();
        onUnitChange && onUnitChange(e.target.value, index);
    }, [index, onUnitChange]);
    const handleOnLabelChange = React.useCallback((e) => {
        e.preventDefault();
        onLabelChange && onLabelChange(e.target.value, index);
    }, [index, onLabelChange]);
    return (React.createElement(React.Fragment, null,
        React.createElement(ui_core_1.Select, { options: unitOptions, "data-testid": `unit-${currentUnit.name}`, value: `${currentUnit.name}:${currentUnit.label}`, onChange: handleOnUnitChange, disabled: readonly }),
        React.createElement(ui_core_1.Input, { "data-testid": `unit-label-${currentUnit.name}`, value: label, onChange: handleOnLabelChange })));
}
exports.UnitDescr = UnitDescr;
//# sourceMappingURL=UnitDescr.js.map