"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Table
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleTableDataProvider = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
const TypeConverterManager_1 = require("../converters/TypeConverterManager");
const UiComponents_1 = require("../UiComponents");
const ColumnFiltering_1 = require("./columnfiltering/ColumnFiltering");
const TableDataProvider_1 = require("./TableDataProvider");
// cSpell:ignore columnfiltering
/**
 * A Table Data Provider using an array of items.
 * @public
 */
class SimpleTableDataProvider {
    constructor(columns) {
        this._sortDirection = ui_core_1.SortDirection.NoSort;
        this._sortColumnIndex = -1;
        /** Event emitted by the data provider when column data changes */
        this.onColumnsChanged = new TableDataProvider_1.TableDataChangeEvent();
        /** Event emitted by the data provider when row data changes */
        this.onRowsChanged = new TableDataProvider_1.TableDataChangeEvent();
        this._columns = columns;
        this._items = [];
        this._rowItemIndices = [];
        this._secondarySortColumnStack = [];
    }
    /** Sets the row items based on an array */
    setItems(items) {
        this._items = items;
        this._rowItemIndices = [];
        this.resetRowIndices();
        this.onRowsChanged.raiseEvent();
    }
    resetRowIndices() {
        this._rowItemIndices.splice(0);
        for (let rowIndex = 0; rowIndex < this._items.length; rowIndex++) {
            this._rowItemIndices.push(rowIndex);
        }
    }
    /** Retrieves the column descriptions */
    async getColumns() {
        return this._columns;
    }
    /** Retrieves the row count */
    async getRowsCount() {
        return this._rowItemIndices.length;
    }
    /** Retrieves a specific row by index */
    async getRow(rowIndex, unfiltered) {
        let realRowIndex = -1;
        if (unfiltered) {
            // istanbul ignore else
            if (0 <= rowIndex && rowIndex < this._items.length)
                realRowIndex = rowIndex;
        }
        else {
            if (0 <= rowIndex && rowIndex < this._rowItemIndices.length)
                realRowIndex = this.getRowIndexFromFilteredRowIndex(rowIndex);
        }
        if (0 <= realRowIndex && realRowIndex < this._items.length)
            return this._items[realRowIndex];
        throw new ui_abstract_1.UiError(UiComponents_1.UiComponents.loggerCategory(this), `getRow: Invalid row index - ${rowIndex}`);
    }
    getRowIndexFromFilteredRowIndex(filteredRowIndex) {
        // istanbul ignore next
        if (filteredRowIndex === undefined)
            return filteredRowIndex;
        // istanbul ignore next
        if (filteredRowIndex < 0 || filteredRowIndex >= this._rowItemIndices.length)
            return -1;
        return this._rowItemIndices[filteredRowIndex];
    }
    /** Sorts the rows based on the value in a specific column */
    async sort(columnIndex, sortDirection) {
        // istanbul ignore next
        if (columnIndex < 0 || columnIndex >= this._columns.length)
            return;
        this._sortColumnIndex = columnIndex;
        this._sortDirection = sortDirection;
        if (sortDirection === ui_core_1.SortDirection.NoSort) {
            if (this._filterDescriptors)
                await this.applyFilterDescriptors(this._filterDescriptors);
            else
                this.resetRowIndices();
            return;
        }
        this._secondarySortColumnStack = [];
        this._secondarySortColumnStack.push(columnIndex);
        // Sort by the column
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const dataProvider = this;
        this._rowItemIndices.sort((a, b) => {
            return dataProvider.sortDispatcher(dataProvider._items[a], dataProvider._items[b], columnIndex, sortDirection);
        });
        delete this._secondarySortColumnStack;
        return;
    }
    sortDispatcher(a, b, columnIndex, sortDirection) {
        let result = this.sortRows(a, b, columnIndex, sortDirection);
        if (result !== 0)
            return result;
        // Secondary sort column support
        // istanbul ignore else
        const column = this._columns[columnIndex];
        if (this._secondarySortColumnStack !== undefined &&
            column.secondarySortColumn !== undefined &&
            column.secondarySortColumn !== columnIndex &&
            this._secondarySortColumnStack.indexOf(column.secondarySortColumn) < 0) {
            this._secondarySortColumnStack.push(column.secondarySortColumn);
            result = this.sortDispatcher(a, b, column.secondarySortColumn, sortDirection);
            this._secondarySortColumnStack.pop();
        }
        return result;
    }
    getPrimitiveValue(record) {
        let primitiveValue;
        const recordValue = record.value;
        // istanbul ignore else
        if (recordValue.valueFormat === ui_abstract_1.PropertyValueFormat.Primitive)
            primitiveValue = recordValue.value;
        return primitiveValue;
    }
    sortRows(a, b, columnIndex, sortDirection) {
        var _a;
        const aCell = a.cells[columnIndex];
        const bCell = b.cells[columnIndex];
        let result = 0;
        const column = this._columns[columnIndex];
        const propertyDescription = column.propertyDescription;
        const valueA = this.getPrimitiveValue(aCell.record);
        const valueB = this.getPrimitiveValue(bCell.record);
        // istanbul ignore else
        if (valueA !== undefined && valueB !== undefined && propertyDescription)
            result = TypeConverterManager_1.TypeConverterManager.getConverter(propertyDescription.typename, (_a = propertyDescription.converter) === null || _a === void 0 ? void 0 : _a.name).sortCompare(valueA, valueB, column.sortIgnoreCase);
        if (sortDirection === ui_core_1.SortDirection.Descending)
            result *= -1;
        return result;
    }
    /** Gets distinct values in a column
     * @beta
     */
    async getDistinctValues(columnKey, maximumValueCount) {
        var _a;
        const distinctValues = new ColumnFiltering_1.DistinctValueCollection();
        const columnIndex = this._columns.findIndex((description) => description.key === columnKey);
        if (columnIndex < 0 || columnIndex >= this._columns.length)
            return distinctValues;
        const columnDescription = this._columns[columnIndex];
        const propertyDescription = columnDescription.propertyDescription;
        const uniqueKeysMap = {};
        const uniqueValues = [];
        // istanbul ignore next
        if (!propertyDescription)
            return distinctValues;
        for (const row of this._items) {
            const cell = row.cells[columnIndex];
            const record = cell.record;
            // istanbul ignore next
            if (record === undefined)
                continue;
            const value = this.getPrimitiveValue(record);
            const displayValue = await TypeConverterManager_1.TypeConverterManager
                .getConverter(propertyDescription.typename, (_a = propertyDescription.converter) === null || _a === void 0 ? void 0 : _a.name)
                .convertPropertyToString(propertyDescription, value);
            // istanbul ignore next
            if (value === undefined || typeof displayValue !== "string")
                continue;
            const valueKey = value.toString();
            if (!uniqueKeysMap.hasOwnProperty(valueKey)) {
                const propertyValue = { value, label: displayValue };
                uniqueKeysMap[valueKey] = propertyValue;
                uniqueValues.push(propertyValue);
            }
        }
        distinctValues.values = uniqueValues;
        distinctValues.values.sort((a, b) => {
            var _a;
            if (a.value && b.value)
                return TypeConverterManager_1.TypeConverterManager.getConverter(propertyDescription.typename, (_a = propertyDescription.converter) === null || _a === void 0 ? void 0 : _a.name).sortCompare(a.value, b.value);
            return 0;
        });
        if (maximumValueCount !== undefined && distinctValues.values.length > maximumValueCount)
            distinctValues.values.splice(maximumValueCount);
        return distinctValues;
    }
    /** Apply a filter descriptor collection
     * @beta
     */
    async applyFilterDescriptors(filterDescriptors) {
        this._filterDescriptors = filterDescriptors;
        this._rowItemIndices.splice(0);
        this._items.forEach((row, i) => {
            if (filterDescriptors.evaluateRow(row))
                this._rowItemIndices.push(i);
        });
        if (this._sortDirection !== ui_core_1.SortDirection.NoSort) {
            await this.sort(this._sortColumnIndex, this._sortDirection);
        }
    }
    /** Adds a row to the end */
    addRow(rowItem) {
        return this.insertRow(rowItem, -1);
    }
    /** Inserts a row at a given row index */
    insertRow(rowItem, index) {
        let newIdx = index;
        if (index < 0 || index >= this._items.length)
            newIdx = this._items.push(rowItem) - 1;
        else
            this._items.splice(newIdx, 0, rowItem);
        this.resetRowIndices();
        this.onRowsChanged.raiseEvent();
        return newIdx;
    }
    /** Deletes a row */
    deleteRow(rowItem, raiseRowsChangedEvent = true) {
        const rowIndex = this.findRowIndex(rowItem);
        if (rowIndex >= 0) {
            this._items.splice(rowIndex, 1);
            this.resetRowIndices();
            if (raiseRowsChangedEvent)
                this.onRowsChanged.raiseEvent();
        }
    }
    /** Moves a row to a new row index */
    moveRow(rowItem, newIndex) {
        const oldIndex = this.findRowIndex(rowItem);
        if (oldIndex >= 0) {
            this.deleteRow(rowItem, false);
            if (newIndex > oldIndex)
                newIndex--; // removing the row will shrink the count by 1
            return this.insertRow(rowItem, newIndex);
        }
        return -1;
    }
    findRowIndex(item) {
        for (let i = 0; i < this._items.length; i++) {
            if (item === this._items[i])
                return i;
        }
        return -1;
    }
}
exports.SimpleTableDataProvider = SimpleTableDataProvider;
//# sourceMappingURL=SimpleTableDataProvider.js.map