"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Table
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataGridFilterParser = exports.FILTER_PARSER_TIMER_TIMEOUT = exports.NumericFilterType = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_core_1 = require("@bentley/ui-core");
const UiComponents_1 = require("../../UiComponents");
const TableDataProvider_1 = require("../TableDataProvider");
const ColumnFiltering_1 = require("./ColumnFiltering");
/** ReactDataGrid NumericFilter type
 * @internal
 */
var NumericFilterType;
(function (NumericFilterType) {
    NumericFilterType[NumericFilterType["ExactMatch"] = 1] = "ExactMatch";
    NumericFilterType[NumericFilterType["Range"] = 2] = "Range";
    NumericFilterType[NumericFilterType["GreaterThan"] = 3] = "GreaterThan";
    NumericFilterType[NumericFilterType["LessThan"] = 4] = "LessThan";
})(NumericFilterType = exports.NumericFilterType || (exports.NumericFilterType = {}));
/** @internal */
exports.FILTER_PARSER_TIMER_TIMEOUT = 250;
/** ReactDataGrid Filter Parser
 * @internal
 */
class DataGridFilterParser {
    static get timerTimeout() { return DataGridFilterParser._timerTimeout; }
    static set timerTimeout(v) { DataGridFilterParser._timerTimeout = v; }
    static async handleFilterChange(filter, filterDescriptor, columnDescription, onApplyFilter) {
        filterDescriptor.clear();
        if (columnDescription.filterRenderer !== undefined) {
            switch (columnDescription.filterRenderer) {
                case TableDataProvider_1.FilterRenderer.MultiSelect:
                    await DataGridFilterParser.parseMultiSelect(filter, filterDescriptor, onApplyFilter);
                    break;
                case TableDataProvider_1.FilterRenderer.MultiValue:
                    await DataGridFilterParser.parseMultiValue(filter, filterDescriptor, onApplyFilter);
                    break;
                case TableDataProvider_1.FilterRenderer.SingleSelect:
                    await DataGridFilterParser.parseSingleSelect(filter, filterDescriptor, onApplyFilter);
                    break;
                case TableDataProvider_1.FilterRenderer.Numeric:
                    await DataGridFilterParser.parseNumeric(filter, filterDescriptor, onApplyFilter);
                    break;
                case TableDataProvider_1.FilterRenderer.Text:
                    await DataGridFilterParser.parseText(filter, filterDescriptor, onApplyFilter);
                    break;
            }
        }
        else {
            await DataGridFilterParser.parseText(filter, filterDescriptor, onApplyFilter);
        }
    }
    static async parseMultiSelect(filter, filterDescriptor, onApplyFilter) {
        /*
        MultiSelect filters
        0: {value: "Title 1", label: "Title 1"}
        1: {value: "Title 100", label: "Title 100"}
        2: {value: "Title 10000", label: "Title 10000"}
        length: 3
        */
        // istanbul ignore else
        if (filter.filterTerm) {
            const filterData = filter.filterTerm;
            if (filterData.length) {
                filterData.forEach((distinctValue) => {
                    filterDescriptor.distinctFilter.addDistinctValue(distinctValue.value);
                });
            }
        }
        await onApplyFilter();
    }
    static async parseMultiValue(filter, filterDescriptor, onApplyFilter) {
        /*
        MultiValue filters
          distinctValues
            length: 3
            0: {value: "Title 1", label: "Title 1"}
            1: {value: "Title 100", label: "Title 100"}
            2: {value: "Title 10000", label: "Title 10000"}
          fieldValues
            length: 1
            0: {value: any, operator: FilterOperator, isCaseSensitive?: boolean}
          logicalOperator: FilterCompositionLogicalOperator
        */
        // istanbul ignore else
        if (filter.filterTerm) {
            const filterData = filter.filterTerm;
            if (filterData.distinctValues.length) {
                filterData.distinctValues.forEach((distinctValue) => {
                    filterDescriptor.distinctFilter.addDistinctValue(distinctValue.value);
                });
            }
            if (filterData.fieldValues.length) {
                filterData.fieldValues.forEach((fieldData) => {
                    filterDescriptor.fieldFilter.addFieldValue(fieldData.fieldValue, fieldData.operator, fieldData.isCaseSensitive);
                });
                filterDescriptor.fieldFilter.logicalOperator = filterData.fieldLogicalOperator;
            }
        }
        await onApplyFilter();
    }
    static async parseSingleSelect(filter, filterDescriptor, onApplyFilter) {
        /*
        SingleSelect filters
        filter.filterTerm: {value: 1, label: "Red"}
        */
        // istanbul ignore else
        if (filter.filterTerm) {
            const filterData = filter.filterTerm;
            filterDescriptor.distinctFilter.addDistinctValue(filterData.value);
        }
        await onApplyFilter();
    }
    static async parseNumeric(filter, filterDescriptor, onApplyFilter) {
        /*
        NumericFilter
        0: {type: 2, begin: 1, end: 10} // range
        1: {type: 1, value: 15} // Exact match
        2: {type: 3, value: 30} // > 30
        3: {type: 4, value: 5}  // < 5
        length: 4
        */
        if (filter.filterTerm) {
            this._numericTimer.setOnExecute(async () => {
                const filterData = filter.filterTerm;
                if (filterData.length) {
                    filterDescriptor.fieldFilter.logicalOperator = ColumnFiltering_1.FilterCompositionLogicalOperator.Or;
                    filterData.forEach((numericFilterData) => {
                        switch (numericFilterData.type) {
                            case NumericFilterType.ExactMatch:
                                const exactMatchData = numericFilterData;
                                filterDescriptor.fieldFilter.addFieldValue(exactMatchData.value, ColumnFiltering_1.FilterOperator.IsEqualTo);
                                break;
                            case NumericFilterType.GreaterThan:
                                const greaterThanData = numericFilterData;
                                filterDescriptor.fieldFilter.addFieldValue(greaterThanData.value, ColumnFiltering_1.FilterOperator.IsGreaterThan);
                                break;
                            case NumericFilterType.LessThan:
                                const lessThanData = numericFilterData;
                                filterDescriptor.fieldFilter.addFieldValue(lessThanData.value, ColumnFiltering_1.FilterOperator.IsLessThan);
                                break;
                            case NumericFilterType.Range:
                                const rangeData = numericFilterData;
                                filterDescriptor.fieldFilter.addFieldValue(rangeData, ColumnFiltering_1.FilterOperator.Range);
                                break;
                            default:
                                bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `parseNumeric: Unknown numeric filter data type - ${numericFilterData.type}`);
                                break;
                        }
                    });
                    await onApplyFilter();
                }
            });
            this._numericTimer.delay = DataGridFilterParser._timerTimeout;
            this._numericTimer.start();
        }
        else {
            this._numericTimer.stop();
            await onApplyFilter();
        }
    }
    static async parseText(filter, filterDescriptor, onApplyFilter) {
        /*
        Input filter
        filter.filterTerm: 1000
        */
        if (filter.filterTerm) {
            this._textTimer.setOnExecute(async () => {
                const filterData = filter.filterTerm;
                filterDescriptor.fieldFilter.addFieldValue(filterData, ColumnFiltering_1.FilterOperator.Contains, false);
                await onApplyFilter();
            });
            this._textTimer.delay = DataGridFilterParser._timerTimeout;
            this._textTimer.start();
        }
        else {
            this._textTimer.stop();
            await onApplyFilter();
        }
    }
}
exports.DataGridFilterParser = DataGridFilterParser;
DataGridFilterParser._timerTimeout = exports.FILTER_PARSER_TIMER_TIMEOUT;
DataGridFilterParser._textTimer = new ui_core_1.Timer(DataGridFilterParser._timerTimeout);
DataGridFilterParser._numericTimer = new ui_core_1.Timer(DataGridFilterParser._timerTimeout);
//# sourceMappingURL=DataGridFilterParser.js.map