"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Table
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterableColumnBase = void 0;
const TableColumnFilterDescriptor_1 = require("./TableColumnFilterDescriptor");
const TableFilterDescriptor_1 = require("./TableFilterDescriptor");
/** Filterable Column base class.
 * @internal
 */
class FilterableColumnBase {
    constructor(filterableTable, description) {
        this._filterable = false;
        this._filterableTable = filterableTable;
        this._columnDescription = description;
        this._filterable = (description.filterable === undefined) ? false : description.filterable;
        this._showFieldFilters = (description.showFieldFilters === undefined) ? true : description.showFieldFilters;
        this._showDistinctValueFilters = (description.showDistinctValueFilters === undefined) ? true : description.showDistinctValueFilters;
        this._filterCaseSensitive = (description.filterCaseSensitive === undefined) ? false : description.filterCaseSensitive;
        if (!this._columnDescription.propertyDescription) {
            this._columnDescription.propertyDescription = {
                name: "",
                displayLabel: "",
                typename: "",
            };
        }
    }
    /** Indicates whether column is filterable.
     */
    get filterable() {
        return this._filterable;
    }
    /** Gets the owning filterable Table.
     */
    get filterableTable() {
        return this._filterableTable;
    }
    /** Gets the Column Description.
     */
    get columnDescription() {
        return this._columnDescription;
    }
    /** Gets the column filter descriptor.
     * @return A class implementing IColumnFilterDescriptor.
     */
    get columnFilterDescriptor() {
        let columnFilterDescriptor = this._filterableTable.filterDescriptors.getColumnFilterDescriptor(this.filterMemberKey);
        if (undefined === columnFilterDescriptor) {
            columnFilterDescriptor = this.createColumnFilterDescriptor();
            this._filterableTable.filterDescriptors.add(columnFilterDescriptor);
        }
        return columnFilterDescriptor;
    }
    /** Creates the column filter descriptor.
     * @return A new class implementing ColumnFilterDescriptor.
     */
    createColumnFilterDescriptor() {
        return new TableColumnFilterDescriptor_1.TableColumnFilterDescriptor(this, this.filterMemberKey, this.filterMemberType);
    }
    /** Creates a basic filter descriptor for use .
     * @return A new FilterDescriptor.
     */
    createSimpleFilterDescriptor(value, filterOperator) {
        return new TableFilterDescriptor_1.TableFilterDescriptor(this._filterableTable, this.filterMemberKey, this.filterMemberType, filterOperator, value, this._filterCaseSensitive);
    }
    /** Gets the filter member key.
     */
    get filterMemberKey() {
        return this._columnDescription.key;
    }
    /** Gets the filter member type.
     */
    get filterMemberType() {
        let memberType = "";
        // istanbul ignore else
        if (this._columnDescription.propertyDescription)
            memberType = this._columnDescription.propertyDescription.typename;
        return memberType;
    }
    /** Determines if a filter is active. */
    get isFilterActive() {
        return this._filterableTable.filterDescriptors.isColumnFilterActive(this.filterMemberKey);
    }
    /** Determines if the filter should show distinct values as a filter option */
    get showDistinctValueFilters() {
        return this._showDistinctValueFilters;
    }
    /** Determines if the filter should show field filter options */
    get showFieldFilters() {
        return this._showFieldFilters;
    }
    /** Determines if the filter is case-sensitive */
    get filterCaseSensitive() {
        return this._filterCaseSensitive;
    }
}
exports.FilterableColumnBase = FilterableColumnBase;
//# sourceMappingURL=FilterableColumnBase.js.map