"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Table
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableFilterDescriptor = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const BooleanTypeConverter_1 = require("../../converters/BooleanTypeConverter");
const TypeConverterManager_1 = require("../../converters/TypeConverterManager");
const UiComponents_1 = require("../../UiComponents");
const ColumnFiltering_1 = require("./ColumnFiltering");
const ColumnFilteringUtilities_1 = require("./ColumnFilteringUtilities");
/** Represents a filtering descriptor.
 * @internal
 */
class TableFilterDescriptor {
    /** Initializes a new instance of the FilterDescriptor class.
     * @param filterableTable The owning Table.
     * @param memberKey       The member (key).
     * @param memberType      The Type of the member.
     * @param filterOperator  The filter operator.
     * @param filterValue     The filter value.
     * @param caseSensitive   Indicates that this filter descriptor will be case sensitive.
     * @param typeConverterName The Type of the member.
     *
     */
    constructor(filterableTable, memberKey, memberType, filterOperator = ColumnFiltering_1.FilterOperator.IsEqualTo, filterValue = TableFilterDescriptor.unsetValue, caseSensitive = false, typeConverterName) {
        this._memberKey = "";
        this._memberType = "";
        this._operator = ColumnFiltering_1.FilterOperator.IsEqualTo;
        this._caseSensitive = false;
        this._filterableTable = filterableTable;
        this.memberKey = memberKey;
        this.operator = filterOperator;
        this.value = filterValue;
        this.isCaseSensitive = caseSensitive;
        this.memberType = memberType;
        this._typeConverterName = typeConverterName;
    }
    /** Gets the member (key).
     */
    get memberKey() {
        return this._memberKey;
    }
    /** Sets the member (key).
     */
    set memberKey(value) {
        this._memberKey = value;
    }
    /** Gets the member type.
     */
    get memberType() {
        return this._memberType;
    }
    /** Sets the member type.
     */
    set memberType(value) {
        this._memberType = value;
    }
    /** Gets the type Converter Name.
     */
    get typeConverterName() {
        return this._typeConverterName;
    }
    /** Sets the member type.
     */
    set typeConverterName(value) {
        this._typeConverterName = value;
    }
    /** Gets the operator.
     */
    get operator() {
        return this._operator;
    }
    /** Sets the operator.
     */
    set operator(operator) {
        this._operator = operator;
    }
    /** Gets the value.
     */
    get value() {
        return this._value;
    }
    /** Sets the value.
     */
    set value(value) {
        this._value = value;
    }
    /** Gets whether the filter is case sensitive.
     */
    get isCaseSensitive() {
        return this._caseSensitive;
    }
    /** Sets whether the filter is case sensitive.
     */
    set isCaseSensitive(caseSensitive) {
        this._caseSensitive = caseSensitive;
    }
    /** Clears the filter descriptor.
     */
    clear() {
        this.value = TableFilterDescriptor.unsetValue;
    }
    /** Determines if the filter descriptor is active.
     */
    get isActive() {
        if (!this._memberKey) {
            return false;
        }
        if (this.isOperatorUnary(this.operator)) {
            return true;
        }
        return this.value !== TableFilterDescriptor.unsetValue;
    }
    /** Evaluates a row for filtering
     */
    evaluateRow(row) {
        if (this._memberKey) {
            let cellValue;
            if (row.getValueFromCell !== undefined)
                cellValue = row.getValueFromCell(this._memberKey);
            else
                cellValue = ColumnFilteringUtilities_1.ColumnFilteringUtilities.getValueFromCell(row, this._memberKey);
            return this.processFilterOperator(cellValue);
        }
        return true;
    }
    /** Determines if this filter is for a particular column.
     */
    isFilterForColumn(columnKey) {
        return this._memberKey === columnKey;
    }
    getJulianDaysFromValue(isUpperBound) {
        // istanbul ignore next
        if (this._value.constructor.name !== "Date") {
            this._value = new Date(this._value);
            this._value.setMilliseconds(0);
        }
        // Credits : https://stackoverflow.com/a/11760121/7797060
        const dayLengthInMilliseconds = 86400000;
        const daysCountFromJulianDayToUnixEpoch = 2440587.5;
        const dayLengthInMinutes = 1440;
        if (isUpperBound)
            this._value.setSeconds(this._value.getSeconds() + 1);
        return (this._value.getTime() / dayLengthInMilliseconds) - (this._value.getTimezoneOffset() / dayLengthInMinutes) + daysCountFromJulianDayToUnixEpoch;
    }
    constructComparisonFilterExpression(filterOperator) {
        if (this._value == null)
            return "";
        if (this.memberType === ui_abstract_1.StandardTypeNames.DateTime)
            return `(${this.memberKey} ${filterOperator}${this.getJulianDaysFromValue(false)})`;
        return `${this.memberKey} ${filterOperator} "${this._value.toString()}"`;
    }
    constructRangeExpression(rangeData) {
        return `(${rangeData.begin.toString()} <= ${this.memberKey}) AND (${this.memberKey} <= ${rangeData.end.toString()})`;
    }
    getEqualsExpression(inverse) {
        let prefix = "";
        let operator = " = ";
        if (inverse) {
            prefix = "NOT ";
            operator = " <> ";
        }
        switch (this.memberType) {
            case ui_abstract_1.StandardTypeNames.DateTime: {
                if (this._value == null)
                    return `${this.memberKey + operator} Null`;
                return `${prefix}(${this.memberKey} >= ${this.getJulianDaysFromValue(false)}) AND (${this.memberKey} < ${this.getJulianDaysFromValue(true)})`;
            }
            case ui_abstract_1.StandardTypeNames.Bool:
            case ui_abstract_1.StandardTypeNames.Boolean:
                if (typeof this._value === "string")
                    return this.memberKey + operator + new BooleanTypeConverter_1.BooleanTypeConverter().convertFromString(this._value).toString();
                if (typeof this._value === "boolean")
                    return this.memberKey + operator + this._value.toString();
                bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `getEqualsExpression - invalid value type: ${typeof this._value}`);
                break;
            case ui_abstract_1.StandardTypeNames.Point2d: {
                if (typeof this._value === "object")
                    return `${prefix} (ArePointsEqualByValue(${this.memberKey}, ${this._value.x}, ${this._value.y}) = 1)`;
                if (typeof this._value === "string")
                    return `${prefix + this._filterableTable.getPropertyDisplayValueExpression(this.memberKey)} = "${this.value}"`;
                bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `getEqualsExpression - invalid value type: ${typeof this._value}`);
                break;
            }
            case ui_abstract_1.StandardTypeNames.Point3d: {
                if (typeof this._value === "object")
                    return `${prefix} (ArePointsEqualByValue(${this.memberKey}, ${this._value.x}, ${this._value.y}, ${this._value.z}) = 1)`;
                if (typeof this._value === "string")
                    return `${prefix + this._filterableTable.getPropertyDisplayValueExpression(this.memberKey)} = "${this.value}"`;
                bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `getEqualsExpression - invalid value type: ${typeof this._value}`);
                break;
            }
            case ui_abstract_1.StandardTypeNames.Double:
            case ui_abstract_1.StandardTypeNames.Float: {
                if (typeof this._value === "number")
                    return `${prefix} (AreDoublesEqualByValue(${this.memberKey}, ${this._value}) = 1)`;
                if (typeof this._value === "string")
                    return `${prefix + this._filterableTable.getPropertyDisplayValueExpression(this.memberKey)} = "${this.value}"`;
                bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `getEqualsExpression - invalid value type: ${typeof this._value}`);
                break;
            }
        }
        return `${this.memberKey + operator}"${this._value.toString()}"`;
    }
    /** Returns filter as ECExpression */
    getFilterExpression() {
        switch (this.operator) {
            case ColumnFiltering_1.FilterOperator.IsEqualTo:
                return this.getEqualsExpression(false);
            case ColumnFiltering_1.FilterOperator.IsNotEqualTo:
                return this.getEqualsExpression(true);
            case ColumnFiltering_1.FilterOperator.IsNull:
            case ColumnFiltering_1.FilterOperator.IsEmpty:
                return `${this.memberKey} = Null`;
            case ColumnFiltering_1.FilterOperator.IsNotNull:
            case ColumnFiltering_1.FilterOperator.IsNotEmpty:
                return `${this.memberKey} <> Null`;
            case ColumnFiltering_1.FilterOperator.IsLessThan:
                return this.constructComparisonFilterExpression("<");
            case ColumnFiltering_1.FilterOperator.IsLessThanOrEqualTo:
                return this.constructComparisonFilterExpression("<=");
            case ColumnFiltering_1.FilterOperator.IsGreaterThan:
                return this.constructComparisonFilterExpression(">");
            case ColumnFiltering_1.FilterOperator.IsGreaterThanOrEqualTo:
                return this.constructComparisonFilterExpression(">=");
            case ColumnFiltering_1.FilterOperator.Range:
                return this.constructRangeExpression(this._value);
            case ColumnFiltering_1.FilterOperator.StartsWith:
                return `${this.memberKey} ~ ${null === this._value ? "Null" : `"${this._value.toString()}%"`}`;
            case ColumnFiltering_1.FilterOperator.EndsWith:
                return `${this.memberKey} ~ ${null === this._value ? "Null" : `"%${this._value.toString()}"`}`;
            case ColumnFiltering_1.FilterOperator.Contains:
                return `${this.memberKey} ~ ${null === this._value ? "Null" : `"%${this._value.toString()}%"`}`;
            case ColumnFiltering_1.FilterOperator.DoesNotContain:
                return `NOT(${this.memberKey} ~ ${null === this._value ? "Null" : `"%${this._value.toString()}%"`})`;
            case ColumnFiltering_1.FilterOperator.IsContainedIn:
                return `${null === this._value ? "Null" : `"${this._value.toString()}"`} ~ "%" & ${this.memberKey} & "%" `;
            case ColumnFiltering_1.FilterOperator.IsNotContainedIn:
                return `NOT(${null === this._value ? "Null" : `"${this._value.toString()}"`} ~ "%" & ${this.memberKey} & "%" )`;
        }
        // istanbul ignore next
        return "";
    }
    isOperatorUnary(op) {
        return (op === ColumnFiltering_1.FilterOperator.IsNull ||
            op === ColumnFiltering_1.FilterOperator.IsNotNull ||
            op === ColumnFiltering_1.FilterOperator.IsEmpty ||
            op === ColumnFiltering_1.FilterOperator.IsNotEmpty);
    }
    processFilterOperator(cellValue) {
        let result = false;
        const converter = TypeConverterManager_1.TypeConverterManager.getConverter(this.memberType, this._typeConverterName);
        switch (this.operator) {
            case ColumnFiltering_1.FilterOperator.IsEqualTo: {
                result = converter.isEqualTo(cellValue, this.value);
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsNotEqualTo: {
                result = converter.isNotEqualTo(cellValue, this.value);
                break;
            }
            // Numeric types, Dates, all types that implement these methods
            case ColumnFiltering_1.FilterOperator.IsLessThan: {
                // istanbul ignore else
                if (converter.isLessGreaterType) {
                    const lgConverter = converter;
                    result = lgConverter.isLessThan(cellValue, this.value);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsLessThanOrEqualTo: {
                // istanbul ignore else
                if (converter.isLessGreaterType) {
                    const lgConverter = converter;
                    result = lgConverter.isLessThanOrEqualTo(cellValue, this.value);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsGreaterThan: {
                // istanbul ignore else
                if (converter.isLessGreaterType) {
                    const lgConverter = converter;
                    result = lgConverter.isGreaterThan(cellValue, this.value);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsGreaterThanOrEqualTo: {
                // istanbul ignore else
                if (converter.isLessGreaterType) {
                    const lgConverter = converter;
                    result = lgConverter.isGreaterThanOrEqualTo(cellValue, this.value);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.Range: {
                // istanbul ignore else
                if (converter.isLessGreaterType) {
                    const lgConverter = converter;
                    const rangeData = this.value;
                    const passed = lgConverter.isGreaterThanOrEqualTo(cellValue, rangeData.begin);
                    if (!passed)
                        result = false;
                    else
                        result = lgConverter.isLessThanOrEqualTo(cellValue, rangeData.end);
                }
                break;
            }
            // Strings
            case ColumnFiltering_1.FilterOperator.StartsWith: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.startsWith(cellValue, this.value, this.isCaseSensitive);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.EndsWith: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.endsWith(cellValue, this.value, this.isCaseSensitive);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.Contains: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.contains(cellValue, this.value, this.isCaseSensitive);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.DoesNotContain: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.doesNotContain(cellValue, this.value, this.isCaseSensitive);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsContainedIn: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.isContainedIn(cellValue, this.value, this.isCaseSensitive);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsNotContainedIn: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.isNotContainedIn(cellValue, this.value, this.isCaseSensitive);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsEmpty: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.isEmpty(cellValue);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsNotEmpty: {
                // istanbul ignore else
                if (converter.isStringType) {
                    const strConverter = converter;
                    result = strConverter.isNotEmpty(cellValue);
                }
                break;
            }
            // All filterable null-able types
            case ColumnFiltering_1.FilterOperator.IsNull: {
                // istanbul ignore else
                if (converter.isNullableType) {
                    const nullConverter = converter;
                    result = nullConverter.isNull(cellValue);
                }
                break;
            }
            case ColumnFiltering_1.FilterOperator.IsNotNull: {
                // istanbul ignore else
                if (converter.isNullableType) {
                    const nullConverter = converter;
                    result = nullConverter.isNotNull(cellValue);
                }
                break;
            }
        }
        return result;
    }
}
exports.TableFilterDescriptor = TableFilterDescriptor;
TableFilterDescriptor.unsetValue = "UNSET";
//# sourceMappingURL=TableFilterDescriptor.js.map