"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Table
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableFilterDescriptorCollection = void 0;
const ColumnFiltering_1 = require("./ColumnFiltering");
/** Represents collection of FilterDescriptor objects composed together by a logical operator.
 * @internal
 */
class TableFilterDescriptorCollection extends ColumnFiltering_1.FilterDescriptorCollection {
    constructor() {
        super();
        this._logicalOperator = ColumnFiltering_1.FilterCompositionLogicalOperator.And;
    }
    /** Gets the logical operator.
     */
    get logicalOperator() {
        return this._logicalOperator;
    }
    /** Sets the logical operator.
     */
    set logicalOperator(value) {
        if (this._logicalOperator !== value) {
            this._logicalOperator = value;
        }
    }
    /** Gets filter descriptors that will be used for composition.
     */
    get filterDescriptorCollection() {
        return this;
    }
    /** Evaluates a row for filtering
     */
    evaluateRow(row) {
        let passed = true;
        for (const fd of this.filterDescriptorCollection.descriptors) {
            if (!fd.isActive)
                continue;
            passed = fd.evaluateRow(row);
            if (passed && ColumnFiltering_1.FilterCompositionLogicalOperator.Or === this.logicalOperator)
                return true;
            else if (!passed && ColumnFiltering_1.FilterCompositionLogicalOperator.And === this.logicalOperator)
                return false;
        }
        return passed;
    }
    /** Determines if a filter is active for a particular column
     */
    isColumnFilterActive(columnKey) {
        const columnFilterDescriptor = this.getColumnFilterDescriptor(columnKey);
        return columnFilterDescriptor ? true : false;
    }
    /** Gets the filter for a particular column
     */
    getColumnFilterDescriptor(columnKey) {
        for (const fd of this.filterDescriptorCollection.descriptors) {
            if (!fd.isActive)
                continue;
            if (fd.isFilterForColumn(columnKey))
                return fd;
        }
        return undefined;
    }
    getFilterExpression() {
        let result = "";
        for (let i = 0; i < this.filterDescriptorCollection.descriptors.length; i++) {
            if (!this.filterDescriptorCollection.descriptors[i].isActive)
                continue;
            if (i > 0 && result !== "") {
                switch (this.logicalOperator) {
                    case ColumnFiltering_1.FilterCompositionLogicalOperator.And:
                        result += (" And ");
                        break;
                    case ColumnFiltering_1.FilterCompositionLogicalOperator.Or:
                        result += (" Or ");
                        break;
                }
            }
            result += (`(${this.filterDescriptorCollection.descriptors[i].getFilterExpression()})`);
        }
        return result;
    }
}
exports.TableFilterDescriptorCollection = TableFilterDescriptorCollection;
//# sourceMappingURL=TableFilterDescriptorCollection.js.map