"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Table
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiValueFilter = void 0;
require("./MultiValueFilter.scss");
const react_1 = __importDefault(require("react"));
const ui_core_1 = require("@bentley/ui-core");
const PopupButton_1 = require("../../../editors/PopupButton");
const UiComponents_1 = require("../../../UiComponents");
const ColumnFiltering_1 = require("../ColumnFiltering");
function contains(valueA, valueB, caseSensitive) {
    // istanbul ignore next
    if (!valueA || !valueB || valueB.length > valueA.length)
        return false;
    if (caseSensitive)
        return valueA.indexOf(valueB, 0) !== -1;
    return valueA.toLocaleUpperCase().indexOf(valueB.toLocaleUpperCase(), 0) !== -1;
}
/** @internal */
function MultiValueFilter(props) {
    const [clearLabel] = react_1.default.useState(() => ui_core_1.UiCore.translate("general.clear"));
    const [filterLabel] = react_1.default.useState(() => UiComponents_1.UiComponents.translate("button.label.filter"));
    const [cancelLabel] = react_1.default.useState(() => ui_core_1.UiCore.translate("dialog.cancel"));
    const [searchLabel] = react_1.default.useState(() => UiComponents_1.UiComponents.translate("button.label.search"));
    const [selectAllLabel] = react_1.default.useState(() => UiComponents_1.UiComponents.translate("button.label.selectAll"));
    const [distinctFilter] = react_1.default.useState(() => props.column.filterableColumn.columnFilterDescriptor.distinctFilter);
    const [buttonLabel, setButtonLabel] = react_1.default.useState(undefined);
    const [searchText, setSearchText] = react_1.default.useState("");
    const [selectAllState, setSelectAllState] = react_1.default.useState(ui_core_1.CheckBoxState.Off);
    const [filterCaseSensitive] = react_1.default.useState(() => !!props.column.filterableColumn.filterCaseSensitive);
    const [showDistinctValueFilters] = react_1.default.useState(() => !!props.column.filterableColumn.showDistinctValueFilters);
    const popupButtonRef = react_1.default.useRef(null);
    const distinctValues = react_1.default.useMemo(() => {
        let options = props.getValidFilterValues(props.column.key);
        options = options.map((o) => {
            // istanbul ignore next
            if (typeof o === "string") {
                return { value: o, label: o };
            }
            return o;
        });
        return options;
    }, [props]);
    const checkedValues = react_1.default.useMemo(() => {
        const values = new Array();
        for (const distinctValue of distinctValues) {
            if (distinctFilter.tryFindDescriptor(distinctValue.value) !== undefined)
                values.push(distinctValue);
        }
        return values;
    }, [distinctValues, distinctFilter]);
    const [checkedDistinctValues, setCheckedDistinctValues] = react_1.default.useState(() => checkedValues);
    react_1.default.useEffect(() => {
        setCheckedDistinctValues(checkedValues);
    }, [checkedValues]);
    const handleClear = react_1.default.useCallback(() => {
        checkedDistinctValues.length = 0;
        const newValues = checkedDistinctValues.slice();
        setCheckedDistinctValues(newValues);
        setSelectAllState(ui_core_1.CheckBoxState.Off);
    }, [checkedDistinctValues]);
    const closePopup = () => {
        // istanbul ignore else
        if (popupButtonRef.current)
            popupButtonRef.current.closePopup();
    };
    const handleApplyFilter = react_1.default.useCallback(() => {
        const filterData = {
            distinctValues: new Array(),
            fieldValues: new Array(),
            fieldLogicalOperator: ColumnFiltering_1.FilterCompositionLogicalOperator.And,
        };
        for (const checkedDistinctValue of checkedDistinctValues) {
            filterData.distinctValues.push(checkedDistinctValue);
        }
        closePopup();
        setSearchText("");
        let label;
        if (checkedDistinctValues.length === 1)
            label = checkedDistinctValues[0].label;
        else if (checkedDistinctValues.length > 1)
            label = `${checkedDistinctValues[0].label} (+${checkedDistinctValues.length - 1})`;
        setButtonLabel(label);
        props.onChange({ filterTerm: filterData, column: props.column });
    }, [props, checkedDistinctValues]);
    const handleCancel = react_1.default.useCallback(() => {
        closePopup();
        setSearchText("");
    }, []);
    const handleSearchValueChanged = react_1.default.useCallback((value) => {
        setSearchText(value);
    }, []);
    const handleSelectAllChanged = react_1.default.useCallback((_e) => {
        let newState = ui_core_1.CheckBoxState.On;
        if (selectAllState === ui_core_1.CheckBoxState.Off || selectAllState === ui_core_1.CheckBoxState.Partial) {
            const newValues = distinctValues.slice();
            setCheckedDistinctValues(newValues);
            newState = ui_core_1.CheckBoxState.On;
        }
        else {
            checkedDistinctValues.length = 0;
            const newValues = checkedDistinctValues.slice();
            setCheckedDistinctValues(newValues);
            newState = ui_core_1.CheckBoxState.Off;
        }
        setSelectAllState(newState);
    }, [checkedDistinctValues, distinctValues, selectAllState]);
    const handleCheckboxChanged = react_1.default.useCallback((e, distinctValue) => {
        const index = checkedDistinctValues.indexOf(distinctValue);
        const newCheckedValues = checkedDistinctValues.slice();
        if (e.target.checked) {
            // istanbul ignore else
            if (index === -1)
                newCheckedValues.push(distinctValue);
        }
        else {
            // istanbul ignore else
            if (index !== -1)
                newCheckedValues.splice(index, 1);
        }
        setCheckedDistinctValues(newCheckedValues);
        let newSelectAllState = ui_core_1.CheckBoxState.Partial;
        if (newCheckedValues.length === distinctValues.length)
            newSelectAllState = ui_core_1.CheckBoxState.On;
        else if (newCheckedValues.length === 0)
            newSelectAllState = ui_core_1.CheckBoxState.Off;
        setSelectAllState(newSelectAllState);
    }, [checkedDistinctValues, distinctValues]);
    const checkBoxItems = react_1.default.useMemo(() => {
        const items = distinctValues
            .filter((distinctValue) => {
            return (searchText.length === 0 || contains(distinctValue.label, searchText, filterCaseSensitive));
        })
            .map((distinctValue, i) => {
            const checked = checkedDistinctValues.find((checkedDistinctValue) => checkedDistinctValue.value === distinctValue.value) !== undefined;
            return (react_1.default.createElement(ui_core_1.CheckListBoxItem, { key: i, label: distinctValue.label, checked: checked, onChange: (e) => handleCheckboxChanged(e, distinctValue) }));
        });
        return items;
    }, [checkedDistinctValues, distinctValues, handleCheckboxChanged, searchText, filterCaseSensitive]);
    const disableSelectAll = checkBoxItems.length !== distinctValues.length;
    return (react_1.default.createElement("div", { "data-testid": "components-multi-value-filter" },
        react_1.default.createElement(PopupButton_1.PopupButton, { placeholder: props.placeholder || filterLabel, label: buttonLabel, closeOnEnter: false, setFocus: false, moveFocus: false, ref: popupButtonRef },
            react_1.default.createElement(PopupButton_1.PopupContent, null,
                showDistinctValueFilters &&
                    react_1.default.createElement("div", null,
                        react_1.default.createElement("div", { className: "components-multi-value-filter-searchbox" },
                            react_1.default.createElement(ui_core_1.SearchBox, { placeholder: searchLabel, onValueChanged: handleSearchValueChanged, valueChangedDelay: 250 })),
                        react_1.default.createElement("div", { className: "components-multi-value-filter-selectAll" },
                            react_1.default.createElement(ui_core_1.Checkbox, { label: selectAllLabel, checked: selectAllState === ui_core_1.CheckBoxState.On, indeterminate: selectAllState === ui_core_1.CheckBoxState.Partial, onChange: handleSelectAllChanged, disabled: disableSelectAll, "data-testid": "components-multi-value-filter-selectAll" })),
                        react_1.default.createElement("div", { className: "components-multi-value-filter-distinct-values" },
                            react_1.default.createElement(ui_core_1.CheckListBox, null, checkBoxItems))),
                react_1.default.createElement("div", { className: "components-multi-value-buttons" },
                    react_1.default.createElement(ui_core_1.Button, { onClick: handleClear, "data-testid": "components-multi-value-button-clear" }, clearLabel),
                    react_1.default.createElement(ui_core_1.Button, { onClick: handleApplyFilter, "data-testid": "components-multi-value-button-filter" }, filterLabel),
                    react_1.default.createElement(ui_core_1.Button, { buttonType: ui_core_1.ButtonType.Hollow, onClick: handleCancel, "data-testid": "components-multi-value-button-cancel" }, cancelLabel))))));
}
exports.MultiValueFilter = MultiValueFilter;
//# sourceMappingURL=MultiValueFilter.js.map