"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Timeline
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseSolarDataProvider = void 0;
const geometry_core_1 = require("@bentley/geometry-core");
const imodeljs_common_1 = require("@bentley/imodeljs-common");
const millisecPerMinute = 1000 * 60;
const millisecPerHour = millisecPerMinute * 60;
// the interface and class are in alpha state - it may change after usability testing - test coverage not complete
/* istanbul ignore file */
/** Base class that provides solar data for animation control
 * @alpha
 */
class BaseSolarDataProvider {
    constructor(viewport, longitude, latitude) {
        this._shadowColor = imodeljs_common_1.ColorDef.create(imodeljs_common_1.ColorByName.gray);
        this.longitude = -75.17035; // long/lat of Philadelphia
        this.latitude = 39.954927;
        this.viewId = ""; // View Id used to determine sunrise and sunset
        this.supportsTimelineAnimation = false; // set to true when provider determines animation data is available.
        this.animationFraction = 0; // value from 0.0 to 1.0 that specifies the percentage complete for the animation.
        this._projectDayStartMS = 0;
        this._projectSunriseMs = 0;
        this._projectSunsetMs = 0;
        this._zoneOffsetMs = 0;
        this._projectSunrise = new Date();
        this._projectSunset = new Date();
        this._projectDateTime = new Date();
        this._projectDay = new Date();
        // instanbul ignore next
        this.onTimeChanged = (_time) => {
            // to be implemented by derived class
        };
        this._viewport = viewport;
        if (longitude)
            this.longitude = longitude;
        if (latitude)
            this.latitude = latitude;
        if (viewport)
            this.viewId = viewport.view.id;
        // project location
        this._cartographicCenter = imodeljs_common_1.Cartographic.fromDegrees(this.longitude, this.latitude, 0.0);
        this._projectTimeZoneOffset = this.getZone(this._cartographicCenter);
        this.initializeData(this._projectTimeZoneOffset);
    }
    // projectTimeZoneOffset = zone offset in hours
    initializeData(projectTimeZoneOffset, initialTime) {
        if (!initialTime)
            initialTime = new Date(Date.now()); // users local time
        this._zoneOffsetMs = projectTimeZoneOffset * millisecPerHour;
        const year = initialTime.getFullYear();
        const month = initialTime.getMonth();
        const date = initialTime.getDate();
        const hours = initialTime.getHours();
        const minutes = initialTime.getMinutes();
        const initialUtcMs = Date.UTC(year, month, date, hours, minutes, 0, 0);
        const initialProjectMs = initialUtcMs - this._zoneOffsetMs;
        this._projectDateTime = new Date(initialProjectMs);
        const utcDayMs = Date.UTC(year, month, date, 0, 0, 0, 0);
        const utcDay = new Date(utcDayMs);
        this._projectDayStartMS = utcDayMs - this._zoneOffsetMs;
        this._projectDay = new Date(this._projectDayStartMS);
        this._projectSunrise = imodeljs_common_1.calculateSunriseOrSunset(utcDay, this._cartographicCenter, true);
        this._projectSunset = imodeljs_common_1.calculateSunriseOrSunset(utcDay, this._cartographicCenter, false);
        this._projectSunriseMs = this._projectSunrise.getTime();
        this._projectSunsetMs = this._projectSunset.getTime();
    }
    get timeZoneOffset() {
        return this._projectTimeZoneOffset;
    }
    get shadowColor() {
        return this._shadowColor;
    }
    set shadowColor(color) {
        this._shadowColor = color;
    }
    get shouldShowTimeline() {
        return false;
    }
    /** Get time at project location */
    get timeOfDay() {
        return this._projectDateTime;
    }
    /** The date can be in project time or if isProjectDate is false, it is assumed to be users current local time
     *  which will be adjusted to the same time at project location. */
    setDateAndTime(date, isProjectDate) {
        let userDay = date;
        // convert date from user date to project location
        if (isProjectDate) {
            const userOffset = date.getTimezoneOffset() * millisecPerMinute;
            userDay = new Date(date.getTime() - userOffset - this._zoneOffsetMs);
        }
        this.initializeData(this._projectTimeZoneOffset, userDay);
    }
    /** Project Day Start */
    get day() {
        return this._projectDay;
    }
    /** Project Day Start in milliseconds */
    get dayStartMs() {
        return this._projectDayStartMS;
    }
    getCartographicCenter(iModel) {
        if (iModel.isGeoLocated) {
            const projectExtents = iModel.projectExtents;
            const projectCenter = geometry_core_1.Point3d.createAdd2Scaled(projectExtents.low, .5, projectExtents.high, .5);
            return iModel.spatialToCartographicFromEcef(projectCenter);
        }
        return imodeljs_common_1.Cartographic.fromDegrees(this.longitude, this.latitude, 0.0);
    }
    set viewport(viewport) {
        this._viewport = viewport;
        if (viewport)
            this.viewId = viewport.view.id;
        else
            this.viewId = "";
    }
    get viewport() {
        return this._viewport;
    }
    getZone(location) {
        return Math.floor(.5 + location.longitudeDegrees / 15.0);
    }
    get sunrise() {
        return this._projectSunrise;
    }
    get sunriseMs() {
        return this._projectSunriseMs;
    }
    get sunset() {
        return this._projectSunset;
    }
    get sunsetMs() {
        return this._projectSunsetMs;
    }
}
exports.BaseSolarDataProvider = BaseSolarDataProvider;
//# sourceMappingURL=BaseSolarDataProvider.js.map