"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Timeline
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimelineComponent = void 0;
require("./TimelineComponent.scss");
const classnames_1 = __importDefault(require("classnames"));
const React = __importStar(require("react"));
const ui_abstract_1 = require("@bentley/ui-abstract");
const UiComponents_1 = require("../UiComponents");
const ui_core_1 = require("@bentley/ui-core");
const InlineEdit_1 = require("./InlineEdit");
const interfaces_1 = require("./interfaces");
const PlayerButton_1 = require("./PlayerButton");
const Scrubber_1 = require("./Scrubber");
const DateUtils_1 = require("../common/DateUtils");
// cspell:ignore millisec
const slowSpeed = 60 * 1000;
const mediumSpeed = 20 * 1000;
const fastSpeed = 10 * 1000;
/**
 * [[TimelineComponent]] is used to playback timeline data
 * @public
 */
class TimelineComponent extends React.Component {
    constructor(props) {
        super(props);
        this._timeLastCycle = 0;
        this._requestFrame = 0;
        this._unmounted = false;
        this._settings = null;
        this._standardTimelineMenuItems = [
            { label: UiComponents_1.UiComponents.translate("timeline.slow"), timelineDuration: slowSpeed },
            { label: UiComponents_1.UiComponents.translate("timeline.medium"), timelineDuration: mediumSpeed },
            { label: UiComponents_1.UiComponents.translate("timeline.fast"), timelineDuration: fastSpeed },
        ];
        this._handleTimelinePausePlayEvent = (args) => {
            const timelineArgs = args;
            // istanbul ignore else
            if (!timelineArgs || this.props.componentId !== timelineArgs.uiComponentId || timelineArgs.timelineAction === undefined)
                return;
            let startPlaying;
            switch (timelineArgs.timelineAction) {
                case interfaces_1.TimelinePausePlayAction.Play:
                    startPlaying = true;
                    break;
                case interfaces_1.TimelinePausePlayAction.Pause:
                    startPlaying = false;
                    break;
                case interfaces_1.TimelinePausePlayAction.Toggle:
                    startPlaying = !this.state.isPlaying;
                    break;
                // istanbul ignore next
                default:
                    return; // throw error?
            }
            if (startPlaying)
                this._onPlay();
            else
                this._onPause();
        };
        // user clicked backward button
        this._onBackward = () => {
            // istanbul ignore else
            if (this.props.onJump)
                this.props.onJump(false);
        };
        // user clicked forward button
        this._onForward = () => {
            // istanbul ignore else
            if (this.props.onJump)
                this.props.onJump(true);
        };
        // user clicked play button
        this._onPlay = () => {
            // istanbul ignore if
            if (this.state.isPlaying)
                return;
            // timeline was complete, restart from the beginning
            if (this.state.currentDuration >= this.state.totalDuration) {
                this._replay();
            }
            else
                this._play();
            // istanbul ignore else
            if (this.props.onPlayPause)
                this.props.onPlayPause(true);
        };
        // user clicked pause button
        this._onPause = () => {
            // istanbul ignore if
            if (!this.state.isPlaying)
                return;
            // stop requesting frames
            window.cancelAnimationFrame(this._requestFrame);
            // stop playing
            this.setState({ isPlaying: false });
            // istanbul ignore else
            if (this.props.onPlayPause)
                this.props.onPlayPause(false);
        };
        // user clicked on timeline rail or dragged scrubber handle
        this._onTimelineChange = (values) => {
            const currentDuration = values[0];
            this._setDuration(currentDuration);
        };
        // set the current duration, which will call the OnChange callback
        this._setDuration = (currentDuration) => {
            this._timeLastCycle = new Date().getTime();
            if (!this._unmounted)
                this.setState({ currentDuration });
            // istanbul ignore else
            if (this.props.onChange) {
                const fraction = currentDuration / this.state.totalDuration;
                this.props.onChange(fraction);
            }
        };
        // recursively update the animation until we hit the end or the pause button is clicked
        this._updateAnimation = (_timestamp) => {
            // istanbul ignore else
            if (!this.state.isPlaying && !this._unmounted) {
                return;
            }
            // update duration
            const currentTime = new Date().getTime();
            const millisecondsElapsed = currentTime - this._timeLastCycle;
            const duration = this.state.currentDuration + millisecondsElapsed;
            this._setDuration(duration);
            // stop the animation!
            if (duration >= this.state.totalDuration) {
                window.cancelAnimationFrame(this._requestFrame);
                this.setState({ isPlaying: false });
                if (this.state.repeat)
                    this._replay();
                else {
                    // istanbul ignore else
                    if (this.props.onPlayPause)
                        this.props.onPlayPause(false);
                }
                return;
            }
            // continue the animation - request the next frame
            this._requestFrame = window.requestAnimationFrame(this._updateAnimation);
        };
        this._replay = () => {
            // timeline was complete, restart from the beginning
            this.setState({ currentDuration: 0 }, (() => this._play()));
        };
        // user changed the total duration
        this._onTotalDurationChange = (value) => {
            // NOTE: we should reset the current duration to 0
            const milliseconds = this._getMilliseconds(value);
            this._onSetTotalDuration(milliseconds);
        };
        this._onSettingsClick = () => {
            this.setState((prevState) => ({ isSettingsOpen: !prevState.isSettingsOpen }));
        };
        this._onCloseSettings = () => {
            this.setState({ isSettingsOpen: false });
        };
        this._changeRepeatSetting = (newValue) => {
            // istanbul ignore else
            if (newValue !== undefined) {
                this.setState(() => ({ repeat: newValue, isSettingsOpen: false }), () => {
                    // istanbul ignore else
                    if (this.props.onSettingsChange) {
                        this.props.onSettingsChange({ loop: this.state.repeat });
                    }
                });
            }
        };
        this._onRepeatChanged = () => {
            this.setState((prevState) => ({ repeat: !prevState.repeat, isSettingsOpen: false }), () => {
                // istanbul ignore else
                if (this.props.onSettingsChange) {
                    this.props.onSettingsChange({ loop: this.state.repeat });
                }
            });
        };
        this._currentDate = () => {
            const fraction = this.state.currentDuration / this.state.totalDuration;
            if (this.props.endDate && this.props.startDate) {
                const timeElapsed = (this.props.endDate.getTime() - this.props.startDate.getTime()) * fraction;
                return new Date(this.props.startDate.getTime() + timeElapsed);
            }
            return new Date();
        };
        this._onSetTotalDuration = (milliseconds) => {
            this.setState({ totalDuration: milliseconds, isSettingsOpen: false }, () => {
                // istanbul ignore else
                if (this.props.onSettingsChange) {
                    this.props.onSettingsChange({ duration: milliseconds });
                }
            });
        };
        this._renderSettings = () => {
            const { totalDuration } = this.state;
            let contextMenuItems = [];
            if (!this.props.appMenuItems) {
                contextMenuItems = this._standardTimelineMenuItems;
            }
            else {
                if (this.props.appMenuItemOption === "append") {
                    contextMenuItems = this._standardTimelineMenuItems.concat(this.props.appMenuItems);
                }
                else if (this.props.appMenuItemOption === "prefix") {
                    contextMenuItems = this.props.appMenuItems.concat(this._standardTimelineMenuItems);
                }
                else {
                    contextMenuItems = this.props.appMenuItems;
                }
            }
            return (React.createElement(React.Fragment, null,
                React.createElement("span", { "data-testid": "timeline-settings", className: "timeline-settings icon icon-more-vertical-2", ref: (element) => this._settings = element, onClick: this._onSettingsClick, role: "button", tabIndex: -1, title: UiComponents_1.UiComponents.translate("button.label.settings") }),
                React.createElement(ui_core_1.ContextMenu, { opened: this.state.isSettingsOpen, onOutsideClick: this._onCloseSettings, direction: ui_core_1.ContextMenuDirection.BottomRight, "data-testid": "timeline-contextmenu-div" },
                    this.state.includeRepeat && React.createElement(ui_core_1.ContextMenuItem, { icon: this.state.repeat && "icon icon-checkmark", onSelect: this._onRepeatChanged }, this._repeatLabel),
                    this.state.includeRepeat && React.createElement("div", { className: "separator", role: "separator" }),
                    this._createMenuItemNodes(contextMenuItems, totalDuration))));
        };
        this.state = {
            isSettingsOpen: false,
            isPlaying: false,
            minimized: true,
            currentDuration: props.initialDuration ? props.initialDuration : /* istanbul ignore next */ 0,
            totalDuration: this.props.totalDuration,
            repeat: this.props.repeat ? true : false,
            includeRepeat: this.props.includeRepeat || this.props.includeRepeat === undefined ? true : false,
        };
        this._repeatLabel = UiComponents_1.UiComponents.translate("timeline.repeat");
        // istanbul ignore else
        if (this.props.componentId) { // can't handle an event if we have no id
            this._removeListener = ui_abstract_1.UiAdmin.onGenericUiEvent.addListener(this._handleTimelinePausePlayEvent);
        }
    }
    componentWillUnmount() {
        if (this._removeListener)
            this._removeListener();
        window.cancelAnimationFrame(this._requestFrame);
        this._unmounted = true;
    }
    shouldComponentUpdate(nextProps, nextState) {
        let result = false;
        // istanbul ignore next
        if (nextState !== this.state || nextProps !== this.props ||
            nextProps.startDate !== this.props.startDate ||
            nextProps.endDate !== this.props.endDate ||
            nextProps.initialDuration !== this.props.initialDuration ||
            nextProps.repeat !== this.props.repeat)
            result = true;
        return result;
    }
    componentDidUpdate(prevProps) {
        // istanbul ignore else
        if (this.props.initialDuration !== prevProps.initialDuration) {
            this._setDuration(this.props.initialDuration ? this.props.initialDuration : /* istanbul ignore next */ 0);
        }
        // istanbul ignore else
        if (this.props.repeat !== prevProps.repeat) {
            this._changeRepeatSetting(this.props.repeat);
        }
        // istanbul ignore else
        if (this.props.totalDuration !== prevProps.totalDuration) {
            this._onSetTotalDuration(this.props.totalDuration);
        }
    }
    _play() {
        // start playing
        this.setState({ isPlaying: true }, () => {
            // request the next frame
            this._timeLastCycle = new Date().getTime();
            this._requestFrame = window.requestAnimationFrame(this._updateAnimation);
        });
    }
    _displayTime(millisec) {
        const addZero = (i) => {
            return (i < 10) ? `0${i}` : i;
        };
        const date = new Date(millisec);
        // const hours = date.getUTCHours();
        const minutes = date.getUTCMinutes();
        const seconds = date.getUTCSeconds();
        return `${addZero(minutes)}:${addZero(seconds)}`;
    }
    _getMilliseconds(time) {
        // istanbul ignore else - WIP
        if (time.indexOf(":") !== -1) {
            return (Number(time.split(":")[0]) * 60 + Number(time.split(":")[1])) * 1000;
        }
        else {
            return Number(time) * 1000;
        }
    }
    _createMenuItemNodes(itemList, currentTimelineDuration) {
        const itemNodes = [];
        itemList.forEach((item, index) => {
            const reactItem = this._createMenuItemNode(item, index, currentTimelineDuration);
            // istanbul ignore else
            if (reactItem)
                itemNodes.push(reactItem);
        });
        return itemNodes;
    }
    _createMenuItemNode(item, index, currentTimelineDuration) {
        let node = null;
        const label = item.label;
        const iconSpec = currentTimelineDuration === item.timelineDuration ? "icon icon-checkmark" : undefined;
        node = (React.createElement(ui_core_1.ContextMenuItem, { key: index, onSelect: () => this._onSetTotalDuration(item.timelineDuration), icon: iconSpec }, label));
        return node;
    }
    render() {
        const { startDate, endDate, showDuration, timeZoneOffset } = this.props;
        const { currentDuration, totalDuration, minimized } = this.state;
        const currentDate = this._currentDate();
        const durationString = this._displayTime(currentDuration);
        const totalDurationString = this._displayTime(totalDuration);
        const hasDates = !!startDate && !!endDate;
        return (React.createElement("div", { "data-testid": "timeline-component", className: classnames_1.default("timeline-component", !!minimized && "minimized", hasDates && "has-dates") },
            React.createElement("div", { className: "header" },
                React.createElement(PlayerButton_1.PlayButton, { className: "play-button", isPlaying: this.state.isPlaying, onPlay: this._onPlay, onPause: this._onPause, "data-testid": "timeline-play" }),
                React.createElement(PlayerButton_1.PlayerButton, { className: "play-backward", icon: "icon-caret-left", onClick: this._onBackward, title: UiComponents_1.UiComponents.translate("timeline.backward") }),
                React.createElement(PlayerButton_1.PlayerButton, { className: "play-button-step", icon: "icon-media-controls-circular-play", isPlaying: this.state.isPlaying, onPlay: this._onPlay, onPause: this._onPause, title: UiComponents_1.UiComponents.translate("timeline.step") }),
                React.createElement(PlayerButton_1.PlayerButton, { className: "play-forward", icon: "icon-caret-right", onClick: this._onForward, title: UiComponents_1.UiComponents.translate("timeline.backward") }),
                React.createElement("span", { className: "current-date" }, DateUtils_1.toDateString(currentDate, timeZoneOffset))),
            React.createElement("div", { className: "scrubber" },
                React.createElement(PlayerButton_1.PlayButton, { className: "play-button", isPlaying: this.state.isPlaying, onPlay: this._onPlay, onPause: this._onPause }),
                React.createElement("div", { className: "start-time-container" },
                    hasDates && React.createElement("span", { "data-testid": "test-start-date", className: "start-date" }, DateUtils_1.toDateString(startDate, timeZoneOffset)),
                    hasDates && !showDuration && React.createElement("span", { "data-testid": "test-start-time", className: "start-time" }, DateUtils_1.toTimeString(startDate, timeZoneOffset)),
                    showDuration && React.createElement("span", { className: "duration-start-time" }, durationString)),
                React.createElement(Scrubber_1.Scrubber, { className: "slider", currentDuration: this.state.currentDuration, totalDuration: totalDuration, startDate: startDate, endDate: endDate, isPlaying: this.state.isPlaying, inMiniMode: !!minimized, showTime: !showDuration, onChange: this._onTimelineChange, onUpdate: this._onTimelineChange, timeZoneOffset: this.props.timeZoneOffset }),
                React.createElement("div", { className: "end-time-container" },
                    hasDates && React.createElement("span", { className: "end-date" }, DateUtils_1.toDateString(endDate, timeZoneOffset)),
                    hasDates && !showDuration && React.createElement("span", { className: "end-time" }, DateUtils_1.toTimeString(endDate, timeZoneOffset)),
                    showDuration && React.createElement(InlineEdit_1.InlineEdit, { className: "duration-end-time", defaultValue: totalDurationString, onChange: this._onTotalDurationChange })),
                minimized && this._renderSettings())));
    }
}
exports.TimelineComponent = TimelineComponent;
//# sourceMappingURL=TimelineComponent.js.map