"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Toolbar
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDragInteraction = void 0;
const React = __importStar(require("react"));
const ui_core_1 = require("@bentley/ui-core");
/** @internal */
const getDragDistance = (from, to) => {
    return from.getDistanceTo(to);
};
function useDrag(args) {
    const { initialPosition, onDrag } = args;
    const handlePointerMove = React.useCallback((e) => {
        // istanbul ignore if
        if (!initialPosition.current)
            return;
        const newPosition = new ui_core_1.Point(e.clientX, e.clientY);
        const dragDistance = getDragDistance(initialPosition.current, newPosition);
        if (dragDistance < 20)
            return;
        onDrag();
    }, [initialPosition, onDrag]);
    return { handlePointerMove };
}
function useLongPress(args) {
    const longPressTimer = React.useRef(new ui_core_1.Timer(500));
    const handlePointerDown = React.useCallback(() => {
        longPressTimer.current.start();
    }, []);
    const handlePointerUp = React.useCallback(() => {
        longPressTimer.current.stop();
    }, []);
    const handlePointerMove = React.useCallback((e) => {
        if (!args.initialPosition.current)
            return;
        const newPosition = new ui_core_1.Point(e.clientX, e.clientY);
        const distance = args.initialPosition.current.getDistanceTo(newPosition);
        distance >= 10 && longPressTimer.current.stop();
    }, [args.initialPosition]);
    React.useEffect(() => {
        longPressTimer.current.setOnExecute(args.onLongPress);
        return () => {
            longPressTimer.current.setOnExecute(undefined); // eslint-disable-line react-hooks/exhaustive-deps
        };
    }, [args.onLongPress]);
    return { handlePointerDown, handlePointerMove, handlePointerUp };
}
/**
 * Hook used on expandable item that require drag or long press to open
 * @param onClick Function called when item is clicked.
 * @param onOpenPanel Function called when item is dragged or long pressed to open panel.
 */
function useDragInteraction(onClick, onOpenPanel) {
    const initialPosition = React.useRef(undefined);
    const skipClick = React.useRef(false);
    const handleOpenPanel = React.useCallback(() => {
        initialPosition.current = undefined;
        skipClick.current = true;
        onOpenPanel && onOpenPanel();
    }, [onOpenPanel]);
    const { handlePointerMove: dragPointerMove, } = useDrag({
        initialPosition,
        onDrag: handleOpenPanel,
    });
    const { handlePointerDown: longPressPointerDown, handlePointerMove: longPressPointerMove, handlePointerUp: longPressPointerUp, } = useLongPress({
        initialPosition,
        onLongPress: handleOpenPanel,
    });
    const handleButtonClick = React.useCallback(() => {
        // istanbul ignore if
        if (skipClick.current)
            return;
        onClick && onClick();
    }, [onClick]);
    const handlePointerDown = React.useCallback((e) => {
        (e.target instanceof Element) && e.target.releasePointerCapture(e.pointerId);
        initialPosition.current = new ui_core_1.Point(e.clientX, e.clientY);
        skipClick.current = false;
        longPressPointerDown();
    }, [longPressPointerDown]);
    const handlePointerMove = React.useCallback((e) => {
        dragPointerMove(e);
        longPressPointerMove(e);
    }, [dragPointerMove, longPressPointerMove]);
    const handlePointerUp = React.useCallback(() => {
        initialPosition.current = undefined;
        longPressPointerUp();
    }, [longPressPointerUp]);
    React.useEffect(() => {
        document.addEventListener("pointermove", handlePointerMove);
        document.addEventListener("pointerup", handlePointerUp);
        return () => {
            document.removeEventListener("pointermove", handlePointerMove);
            document.removeEventListener("pointerup", handlePointerUp);
        };
    }, [handlePointerMove, handlePointerUp]);
    return { handlePointerDown, handleButtonClick };
}
exports.useDragInteraction = useDragInteraction;
//# sourceMappingURL=useDragInteraction.js.map