"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeEventDispatcher = void 0;
const concat_1 = require("rxjs/internal/observable/concat");
const defer_1 = require("rxjs/internal/observable/defer");
const empty_1 = require("rxjs/internal/observable/empty");
const from_1 = require("rxjs/internal/observable/from");
const merge_1 = require("rxjs/internal/observable/merge");
const of_1 = require("rxjs/internal/observable/of");
const concatAll_1 = require("rxjs/internal/operators/concatAll");
const concatMap_1 = require("rxjs/internal/operators/concatMap");
const distinctUntilChanged_1 = require("rxjs/internal/operators/distinctUntilChanged");
const finalize_1 = require("rxjs/internal/operators/finalize");
const map_1 = require("rxjs/internal/operators/map");
const mergeAll_1 = require("rxjs/internal/operators/mergeAll");
const publishReplay_1 = require("rxjs/internal/operators/publishReplay");
const refCount_1 = require("rxjs/internal/operators/refCount");
const subscribeOn_1 = require("rxjs/internal/operators/subscribeOn");
const toArray_1 = require("rxjs/internal/operators/toArray");
const asap_1 = require("rxjs/internal/scheduler/asap");
const TreeSelectionManager_1 = require("./internal/TreeSelectionManager");
const TreeModel_1 = require("./TreeModel");
/**
 * Default event dispatcher that emits tree events according performed actions.
 * It converts low level tree events into TreeEvents.
 * @internal
 */
class TreeEventDispatcher {
    constructor(treeEvents, nodeLoader, selectionMode, getVisibleNodes) {
        this._activeSelections = new Set();
        this._treeEvents = treeEvents;
        this._nodeLoader = nodeLoader;
        this._getVisibleNodes = getVisibleNodes;
        this._selectionManager = new TreeSelectionManager_1.TreeSelectionManager(selectionMode, this._getVisibleNodes);
        this._selectionManager.onDragSelection.addListener(({ selectionChanges }) => {
            const modifications = from_1.from(selectionChanges)
                .pipe(map_1.map(({ selectedNodes, deselectedNodes }) => from_1.from(this.collectSelectionChanges(selectedNodes, []))
                .pipe(concatMap_1.concatMap(({ selectedNodeItems }) => from_1.from(selectedNodeItems)), toArray_1.toArray(), map_1.map((collectedIds) => ({ selectedNodeItems: collectedIds, deselectedNodeItems: this.collectNodeItems(deselectedNodes) })))), concatAll_1.concatAll(), publishReplay_1.publishReplay(), refCount_1.refCount());
            /* istanbul ignore else */
            if (this._treeEvents.onSelectionModified !== undefined)
                this._treeEvents.onSelectionModified({ modifications });
        });
        this._selectionManager.onSelectionChanged.addListener(({ selectedNodes, deselectedNodes }) => {
            const modifications = merge_1.merge(defer_1.defer(() => {
                this._activeSelections.add(modifications);
                return empty_1.EMPTY;
            }), this.collectSelectionChanges(selectedNodes, deselectedNodes))
                .pipe(finalize_1.finalize(() => {
                this._activeSelections.delete(modifications);
            }), publishReplay_1.publishReplay(), refCount_1.refCount());
            /* istanbul ignore else */
            if (this._treeEvents.onSelectionModified !== undefined)
                this._treeEvents.onSelectionModified({ modifications });
        });
        this._selectionManager.onSelectionReplaced.addListener(({ selectedNodeIds }) => {
            const replacements = merge_1.merge(defer_1.defer(() => {
                this._activeSelections.add(replacements);
                return empty_1.EMPTY;
            }), this.collectSelectionChanges(selectedNodeIds, []))
                .pipe(finalize_1.finalize(() => {
                this._activeSelections.delete(replacements);
            }), publishReplay_1.publishReplay(), refCount_1.refCount());
            /* istanbul ignore else */
            if (this._treeEvents.onSelectionReplaced !== undefined)
                this._treeEvents.onSelectionReplaced({ replacements });
        });
    }
    setVisibleNodes(visibleNodes) {
        this._getVisibleNodes = visibleNodes;
        this._selectionManager.setVisibleNodes(visibleNodes);
    }
    onNodeCheckboxClicked(nodeId, newState) {
        if (this._getVisibleNodes === undefined)
            return;
        const visibleNodes = this._getVisibleNodes();
        const clickedNode = visibleNodes.getModel().getNode(nodeId);
        if (clickedNode === undefined)
            return;
        const immediateStateChanges = [{ nodeItem: clickedNode.item, newState }];
        if (clickedNode.isSelected) {
            for (const node of visibleNodes) {
                if (TreeModel_1.isTreeModelNode(node) && node.id !== clickedNode.id && node.isSelected && node.checkbox.state !== newState) {
                    immediateStateChanges.push({ nodeItem: node.item, newState });
                }
            }
        }
        const stateChanges = concat_1.concat(of_1.of(immediateStateChanges), from_1.from(this._activeSelections)
            .pipe(mergeAll_1.mergeAll(), map_1.map(({ selectedNodeItems }) => selectedNodeItems.map((item) => ({ nodeItem: item, newState })))))
            .pipe(publishReplay_1.publishReplay(), refCount_1.refCount());
        /* istanbul ignore else */
        if (this._treeEvents.onCheckboxStateChanged !== undefined)
            this._treeEvents.onCheckboxStateChanged({ stateChanges });
    }
    onNodeExpanded(nodeId) {
        /* istanbul ignore else */
        if (this._treeEvents.onNodeExpanded !== undefined)
            this._treeEvents.onNodeExpanded({ nodeId });
    }
    onNodeCollapsed(nodeId) {
        /* istanbul ignore else */
        if (this._treeEvents.onNodeCollapsed !== undefined)
            this._treeEvents.onNodeCollapsed({ nodeId });
    }
    onNodeClicked(nodeId, event) {
        const node = this._getVisibleNodes ? this._getVisibleNodes().getModel().getNode(nodeId) : undefined;
        const isNodeSelected = node ? node.isSelected : false;
        this._selectionManager.onNodeClicked(nodeId, event);
        // if clicked node was already selected fire delayed click event
        if (isNodeSelected && this._treeEvents.onDelayedNodeClick !== undefined) {
            this._treeEvents.onDelayedNodeClick({ nodeId });
        }
    }
    onNodeMouseDown(nodeId) {
        this._selectionManager.onNodeMouseDown(nodeId);
    }
    onNodeMouseMove(nodeId) {
        this._selectionManager.onNodeMouseMove(nodeId);
    }
    onNodeEditorActivated(nodeId) {
        const node = this._getVisibleNodes ? this._getVisibleNodes().getModel().getNode(nodeId) : /* istanbul ignore next */ undefined;
        const isNodeSelected = node ? node.isSelected : false;
        // if node was already selected, fire onNodeEditorActivated event
        if (isNodeSelected && this._treeEvents.onNodeEditorActivated !== undefined) {
            this._treeEvents.onNodeEditorActivated({ nodeId });
        }
    }
    onTreeKeyDown(event) {
        this._selectionManager.onTreeKeyDown(event, this);
    }
    onTreeKeyUp(event) {
        this._selectionManager.onTreeKeyUp(event, this);
    }
    collectSelectionChanges(selection, deselection) {
        const deselectedItems = this.collectNodeItems(deselection);
        if (TreeSelectionManager_1.isRangeSelection(selection)) {
            let firstEmission = true;
            return this.collectNodesBetween(selection.from, selection.to)
                .pipe(map_1.map((selectedNodeItems) => {
                if (firstEmission) {
                    firstEmission = false;
                    return { selectedNodeItems, deselectedNodeItems: deselectedItems };
                }
                return { selectedNodeItems, deselectedNodeItems: [] };
            }));
        }
        const selectedItems = this.collectNodeItems(selection);
        return of_1.of({ selectedNodeItems: selectedItems, deselectedNodeItems: deselectedItems });
    }
    collectNodesBetween(nodeId1, nodeId2) {
        const [readyNodes, nodesToLoad] = TreeEventDispatcher.groupNodesByLoadingState(this.iterateNodesBetween(nodeId1, nodeId2));
        const loadedSelectedNodes = from_1.from(nodesToLoad.map((node) => {
            const parentNode = node.parentId ? this._getVisibleNodes().getModel().getNode(node.parentId) : this._getVisibleNodes().getModel().getRootNode();
            return parentNode ? this._nodeLoader.loadNode(parentNode, node.childIndex) : /* istanbul ignore next */ empty_1.EMPTY;
        }))
            .pipe(
        // We have requested multiple nodes that may belong to the same page.
        // When the page loads we only want to process the loaded nodes once.
        // Making assumption that loaded nodes from the same page will be emitted without interruptions.
        // Maybe we could simplify this to `this._nodeLoader.loadNodes(nodesToLoad)`?
        mergeAll_1.mergeAll(), distinctUntilChanged_1.distinctUntilChanged(), map_1.map((loadResult) => loadResult.loadedNodes));
        return concat_1.concat(of_1.of(readyNodes), loadedSelectedNodes)
            .pipe(
        // Give enough time for multiple subscribers to subscribe before any emissions begin
        subscribeOn_1.subscribeOn(asap_1.asapScheduler));
    }
    *iterateNodesBetween(nodeId1, nodeId2) {
        let firstNodeFound = false;
        if (this._getVisibleNodes === undefined) {
            return;
        }
        for (const node of this._getVisibleNodes()) {
            if (firstNodeFound) {
                yield node;
            }
            if (TreeModel_1.isTreeModelNode(node)) {
                if (nodeId1 === nodeId2 && node.id === nodeId1) {
                    yield node;
                    return;
                }
                if (node.id === nodeId1 || node.id === nodeId2) {
                    if (firstNodeFound) {
                        return;
                    }
                    firstNodeFound = true;
                    yield node;
                }
            }
        }
    }
    collectNodeItems(nodeIds) {
        const items = [];
        if (this._getVisibleNodes === undefined)
            return items;
        for (const nodeId of nodeIds) {
            const node = this._getVisibleNodes().getModel().getNode(nodeId);
            // istanbul ignore else
            if (node !== undefined)
                items.push(node.item);
        }
        return items;
    }
    static groupNodesByLoadingState(nodes) {
        const loadedNodeItems = [];
        const nodesToLoad = [];
        for (const node of nodes) {
            if (TreeModel_1.isTreeModelNode(node)) {
                loadedNodeItems.push(node.item);
            }
            else {
                nodesToLoad.push(node);
            }
        }
        return [loadedNodeItems, nodesToLoad];
    }
}
exports.TreeEventDispatcher = TreeEventDispatcher;
//# sourceMappingURL=TreeEventDispatcher.js.map