"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeEventHandler = void 0;
const takeUntil_1 = require("rxjs/internal/operators/takeUntil");
const Subject_1 = require("rxjs/internal/Subject");
const TreeModelMutator_1 = require("./internal/TreeModelMutator");
const Observable_1 = require("./Observable");
/**
 * Default tree event handler.
 * @public
 */
class TreeEventHandler {
    constructor(params) {
        this._disposed = new Subject_1.Subject();
        this._selectionReplaced = new Subject_1.Subject();
        this._modelMutator = new TreeModelMutator_1.TreeModelMutator(params.modelSource, params.nodeLoader, !!params.collapsedChildrenDisposalEnabled);
        this._editingParams = params.editingParams;
    }
    /** Disposes tree event handler. */
    dispose() {
        this._disposed.next();
    }
    get modelSource() { return this._modelMutator.modelSource; }
    /** Expands node and starts loading children. */
    onNodeExpanded({ nodeId }) {
        Observable_1.toRxjsObservable(this._modelMutator.expandNode(nodeId)).pipe(takeUntil_1.takeUntil(this._disposed)).subscribe();
    }
    /** Collapses node */
    onNodeCollapsed({ nodeId }) {
        this._modelMutator.collapseNode(nodeId);
    }
    /** Selects and deselects nodes until event is handled, handler is disposed or selection replaced event occurs. */
    onSelectionModified({ modifications }) {
        return Observable_1.toRxjsObservable(modifications)
            .pipe(takeUntil_1.takeUntil(this._disposed), takeUntil_1.takeUntil(this._selectionReplaced))
            .subscribe({
            next: ({ selectedNodeItems, deselectedNodeItems }) => {
                this._modelMutator.modifySelection(selectedNodeItems, deselectedNodeItems);
            },
        });
    }
    /** Replaces currently selected nodes until event is handled, handler is disposed or another selection replaced event occurs. */
    onSelectionReplaced({ replacements }) {
        this._selectionReplaced.next();
        let firstEmission = true;
        return Observable_1.toRxjsObservable(replacements)
            .pipe(takeUntil_1.takeUntil(this._disposed), takeUntil_1.takeUntil(this._selectionReplaced))
            .subscribe({
            next: ({ selectedNodeItems }) => {
                if (firstEmission) {
                    firstEmission = false;
                    this._modelMutator.replaceSelection(selectedNodeItems);
                }
                this._modelMutator.modifySelection(selectedNodeItems, []);
            },
        });
    }
    /** Changes nodes checkbox states. */
    onCheckboxStateChanged({ stateChanges }) {
        return stateChanges.subscribe((changes) => this._modelMutator.setCheckboxStates(changes));
    }
    /** Activates node editing if editing parameters are supplied and node is editable. */
    onDelayedNodeClick({ nodeId }) {
        this.activateEditor(nodeId);
    }
    /** Activates node editing if editing parameters are supplied and node is editable. */
    onNodeEditorActivated({ nodeId }) {
        this.activateEditor(nodeId);
    }
    /** Activates node editing if editing parameters are supplied and node is editable. */
    activateEditor(nodeId) {
        if (this._editingParams === undefined)
            return;
        this._modelMutator.activateEditing(nodeId, this._editingParams.onNodeUpdated);
    }
}
exports.TreeEventHandler = TreeEventHandler;
//# sourceMappingURL=TreeEventHandler.js.map