"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTreeEventsHandler = exports.useTreeModelSource = exports.usePagedTreeNodeLoader = exports.useTreeNodeLoader = exports.useVisibleTreeNodes = void 0;
const react_1 = require("react");
const ui_core_1 = require("@bentley/ui-core");
const TreeEventHandler_1 = require("./TreeEventHandler");
const TreeModelSource_1 = require("./TreeModelSource");
const TreeNodeLoader_1 = require("./TreeNodeLoader");
/**
 * Custom hook which returns a flat list of visible nodes from given `TreeModelSource` and subscribes
 * to onModelChanged event to update the list when model changes.
 *
 * @public
 */
function useVisibleTreeNodes(modelSource) {
    const [visibleNodes, setVisibleNodes] = react_1.useState(modelSource.getVisibleNodes());
    react_1.useEffect(() => {
        const onModelChanged = () => {
            setVisibleNodes(modelSource.getVisibleNodes());
        };
        onModelChanged();
        return modelSource.onModelChanged.addListener(onModelChanged);
    }, [modelSource]);
    return visibleNodes;
}
exports.useVisibleTreeNodes = useVisibleTreeNodes;
/**
 * Custom hook which creates a nodes' loader using the supplied data provider and model source. The
 * loader pulls nodes from the data provider and puts them into the model source.
 *
 * @public
 */
function useTreeNodeLoader(dataProvider, modelSource) {
    const createLoader = react_1.useCallback(() => new TreeNodeLoader_1.TreeNodeLoader(dataProvider, modelSource), [dataProvider, modelSource]);
    return ui_core_1.useDisposable(createLoader);
}
exports.useTreeNodeLoader = useTreeNodeLoader;
/**
 * Custom hook which creates a paging nodes' loader using the supplied data provider and model source. The
 * loader pulls nodes from the data provider and puts them into the model source.
 *
 * @beta
 */
function usePagedTreeNodeLoader(dataProvider, pageSize, modelSource) {
    const createLoader = react_1.useCallback(() => new TreeNodeLoader_1.PagedTreeNodeLoader(dataProvider, modelSource, pageSize), [dataProvider, modelSource, pageSize]);
    return ui_core_1.useDisposable(createLoader);
}
exports.usePagedTreeNodeLoader = usePagedTreeNodeLoader;
/**
 * Custom hook which creates a `TreeModelSource`.
 *
 * @note The model source has no direct dependency on the data provider, but we want a fresh model
 * source whenever the data provider changes - that's the reason the hook takes a data provider.
 *
 * @public
 */
function useTreeModelSource(dataProvider) {
    // need to create new model source every time data provider changes although it does not need data provider to be created.
    return react_1.useMemo(() => new TreeModelSource_1.TreeModelSource(), [dataProvider]); // eslint-disable-line react-hooks/exhaustive-deps
}
exports.useTreeModelSource = useTreeModelSource;
/**
 * Custom hook which creates and takes care of disposing a TreeEventsHandler. The input is either a factory method
 * for a custom `TreeEventHandler` implementation or parameters for the default implementation.
 *
 * @note Caller must ensure `factoryOrParams` changes only when a new handler needs to be created. `useCallback` or `useMemo` can
 * be used for that purpose based on whether the input is a factory function or params object.
 *
 * @public
 */
function useTreeEventsHandler(factoryOrParams) {
    const factory = react_1.useCallback(() => {
        if (typeof factoryOrParams === "function")
            return factoryOrParams();
        return new TreeEventHandler_1.TreeEventHandler(factoryOrParams);
    }, [factoryOrParams]);
    return ui_core_1.useDisposable(factory);
}
exports.useTreeEventsHandler = useTreeEventsHandler;
//# sourceMappingURL=TreeHooks.js.map