"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MutableTreeModel = exports.isTreeModelRootNode = exports.isTreeModelNodePlaceholder = exports.isTreeModelNode = void 0;
const immer_1 = require("immer");
const lodash_1 = __importDefault(require("lodash"));
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_core_1 = require("@bentley/ui-core");
const SparseTree_1 = require("./internal/SparseTree");
/**
 * Checks if object is [[TreeModelNode]]
 * @public
 */
function isTreeModelNode(obj) {
    return obj !== undefined && !isTreeModelNodePlaceholder(obj) && !isTreeModelRootNode(obj);
}
exports.isTreeModelNode = isTreeModelNode;
/**
 * Checks if object is [[TreeModelNodePlaceholder]]
 * @public
 */
function isTreeModelNodePlaceholder(obj) {
    return obj !== undefined && "childIndex" in obj;
}
exports.isTreeModelNodePlaceholder = isTreeModelNodePlaceholder;
/**
 * Checks if object is [[TreeModelRootNode]]
 * @public
 */
function isTreeModelRootNode(obj) {
    return obj !== undefined && obj.id === undefined && !("childIndex" in obj);
}
exports.isTreeModelRootNode = isTreeModelRootNode;
/**
 * Mutable tree model which holds nodes and allows adding or removing them.
 * @public
 */
class MutableTreeModel {
    constructor() {
        this[_a] = true;
        this._tree = new SparseTree_1.SparseTree();
        this._rootNode = { depth: -1, id: undefined, numChildren: undefined };
    }
    /** Returns root node of a tree. This node is not visible and is there to allow having multiple visible root nodes. */
    getRootNode() {
        return this._rootNode;
    }
    getNode(nodeId, childIndex) {
        if (childIndex === undefined) {
            return this._tree.getNode(nodeId);
        }
        const children = this._tree.getChildren(nodeId);
        const childId = children !== undefined ? children.get(childIndex) : undefined;
        if (childId !== undefined) {
            return this._tree.getNode(childId);
        }
        const parentNode = nodeId === undefined ? this._rootNode : this._tree.getNode(nodeId);
        if (parentNode !== undefined) {
            return {
                childIndex,
                depth: parentNode.depth + 1,
                parentId: nodeId,
            };
        }
        return undefined;
    }
    /** Returns children for specific parent.
     * @param parentId id of parent node.
     * @returns children of parent node or root nodes if undefined is passed as parent id.
     */
    getChildren(parentId) {
        return this._tree.getChildren(parentId);
    }
    /** Returns children offset in children array for specific parent. */
    getChildOffset(parentId, childId) {
        return this._tree.getChildOffset(parentId, childId);
    }
    /**
     * Sets children for parent node starting from the specific offset.
     * If offset overlaps with already added nodes, the overlapping nodes are overwritten.
     */
    setChildren(parentId, nodeInputs, offset) {
        const parentNode = parentId === undefined ? this._rootNode : this._tree.getNode(parentId);
        if (parentNode === undefined)
            return;
        const children = [];
        for (const input of nodeInputs) {
            const child = MutableTreeModel.createTreeModelNode(parentNode, input);
            children.push(child);
        }
        this._tree.setChildren(parentNode.id, children, offset);
        MutableTreeModel.setNumChildrenForNode(parentNode, this._tree.getChildren(parentNode.id));
    }
    /**
     * Inserts child in the specified position.
     * If offset is higher then current length of children array, the length is increased.
     */
    insertChild(parentId, childNodeInput, offset) {
        const parentNode = parentId === undefined ? this._rootNode : this._tree.getNode(parentId);
        if (parentNode === undefined)
            return;
        const child = MutableTreeModel.createTreeModelNode(parentNode, childNodeInput);
        this._tree.insertChild(parentNode.id, child, offset);
        MutableTreeModel.setNumChildrenForNode(parentNode, this._tree.getChildren(parentNode.id));
    }
    /**
     * Changes the id of target node.
     * @returns `true` on success, `false` otherwise.
     */
    changeNodeId(currentId, newId) {
        var _b, _c;
        const node = this.getNode(currentId);
        if (node === undefined) {
            return false;
        }
        if (currentId === newId) {
            return true;
        }
        const index = this.getChildOffset(node.parentId, currentId);
        bentleyjs_core_1.assert(index !== undefined);
        if (!this._tree.setNodeId(node.parentId, index, newId)) {
            return false;
        }
        node.id = newId; // eslint-disable-line @typescript-eslint/no-unnecessary-type-assertion
        for (const [childId] of (_c = (_b = this.getChildren(newId)) === null || _b === void 0 ? void 0 : _b.iterateValues()) !== null && _c !== void 0 ? _c : []) {
            const child = this.getNode(childId);
            bentleyjs_core_1.assert(child !== undefined);
            child.parentId = newId;
        }
        return true;
    }
    /**
     * Transfers node along with its children to a new location. Fails if destination has undefined child count.
     * @param sourceNodeId Node that is being moved.
     * @param targetParentId Node that will receive a new child.
     * @param targetIndex Insertion location among target's *current* children.
     * @returns `true` on success, `false` otherwise.
     */
    moveNode(sourceNodeId, targetParentId, targetIndex) {
        const sourceNode = this.getNode(sourceNodeId);
        if (sourceNode === undefined) {
            return false;
        }
        const targetParent = targetParentId === undefined ? this._rootNode : this.getNode(targetParentId);
        if (targetParent === undefined || targetParent.numChildren === undefined) {
            return false;
        }
        if (targetParentId !== undefined && this.areNodesRelated(sourceNodeId, targetParentId)) {
            return false;
        }
        this._tree.moveNode(sourceNode.parentId, sourceNodeId, targetParentId, targetIndex);
        const sourceParent = sourceNode.parentId === undefined ? this._rootNode : this.getNode(sourceNode.parentId);
        bentleyjs_core_1.assert(sourceParent !== undefined);
        MutableTreeModel.setNumChildrenForNode(sourceParent, this._tree.getChildren(sourceNode.parentId));
        MutableTreeModel.setNumChildrenForNode(targetParent, this._tree.getChildren(targetParentId));
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
        sourceNode.parentId = targetParentId;
        const updateDepths = (parentId, depth) => {
            var _b, _c;
            const node = this.getNode(parentId);
            bentleyjs_core_1.assert(node !== undefined);
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
            node.depth = depth;
            for (const [nodeId] of (_c = (_b = this.getChildren(parentId)) === null || _b === void 0 ? void 0 : _b.iterateValues()) !== null && _c !== void 0 ? _c : []) {
                updateDepths(nodeId, depth + 1);
            }
        };
        updateDepths(sourceNodeId, targetParent.depth + 1);
        return true;
    }
    areNodesRelated(ancestorNodeId, descendantNodeId) {
        const node = this.getNode(descendantNodeId);
        if (node === undefined || node.parentId === undefined) {
            return false;
        }
        return node.parentId === ancestorNodeId ? true : this.areNodesRelated(ancestorNodeId, node.parentId);
    }
    /**
     * Sets the number of child nodes a parent is expected to contain. All child nodes of this parent will be subsequently
     * removed.
     */
    setNumChildren(parentId, numChildren) {
        const parentNode = parentId === undefined ? this._rootNode : this._tree.getNode(parentId);
        if (parentNode === undefined) {
            return;
        }
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
        parentNode.numChildren = numChildren;
        this._tree.setNumChildren(parentId, numChildren !== null && numChildren !== void 0 ? numChildren : 0);
    }
    /** Removes children specified by id.
     * @param parentId id of the parent node.
     * @param child child node id or index that should be removed.
     */
    removeChild(parentId, child) {
        const parentNode = parentId === undefined ? this._rootNode : this._tree.getNode(parentId);
        this._tree.removeChild(parentId, child);
        // istanbul ignore else
        if (parentNode)
            MutableTreeModel.setNumChildrenForNode(parentNode, this._tree.getChildren(parentNode.id));
    }
    /** Removes all children for parent specified by id.
     * @param parentId id of parent node or undefined to remove root nodes.
     */
    clearChildren(parentId) {
        const parentNode = parentId === undefined ? this._rootNode : this._tree.getNode(parentId);
        if (parentNode !== undefined) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
            parentNode.numChildren = undefined;
        }
        this._tree.deleteSubtree(parentId, false);
    }
    /** Generates flat list of visible nodes in the tree model. */
    computeVisibleNodes() {
        const result = MutableTreeModel.getVisibleDescendants(this._tree, this._rootNode);
        return {
            getNumNodes: () => result.length,
            getAtIndex: (index) => {
                if (typeof (index) === "number") {
                    return result[index];
                }
                return this._tree.getNode(index);
            },
            getModel: () => this,
            getNumRootNodes: () => this._rootNode.numChildren,
            [Symbol.iterator]: () => result[Symbol.iterator](),
            getIndexOfNode: (nodeId) => {
                return result.findIndex((visibleNode) => visibleNode.id === nodeId);
            },
        };
    }
    /** Iterates over all nodes present in the tree model. */
    *iterateTreeModelNodes(parentId) {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const _this = this;
        function* iterateDescendants(subParentId) {
            const children = _this.getChildren(subParentId);
            if (children === undefined) {
                return;
            }
            for (const [nodeId] of children.iterateValues()) {
                const node = _this.getNode(nodeId);
                if (node !== undefined) {
                    yield node;
                    yield* iterateDescendants(nodeId);
                }
            }
        }
        yield* iterateDescendants(parentId);
    }
    static setNumChildrenForNode(node, children) {
        const numChildren = children ? children.getLength() : undefined;
        if (node.numChildren === numChildren)
            return;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
        node.numChildren = numChildren;
    }
    static createTreeModelNode(parentNode, input) {
        return {
            id: input.id,
            parentId: parentNode.id,
            depth: parentNode.depth + 1,
            isLoading: input.isLoading,
            numChildren: input.numChildren,
            description: input.description || "",
            isExpanded: input.isExpanded,
            label: input.label,
            isSelected: input.isSelected,
            checkbox: {
                state: input.item.checkBoxState || ui_core_1.CheckBoxState.Off,
                isDisabled: !!input.item.isCheckboxDisabled,
                isVisible: !!input.item.isCheckboxVisible,
            },
            item: lodash_1.default.cloneDeep(input.item),
        };
    }
    // Traverses the tree and collects visible descendants.
    static getVisibleDescendants(tree, rootNode, result = []) {
        const children = tree.getChildren(rootNode.id);
        if (!children) {
            return result;
        }
        let index = 0;
        for (const childId of children) {
            if (childId === undefined) {
                result.push({ parentId: rootNode.id, depth: rootNode.depth + 1, childIndex: index });
            }
            else {
                const childNode = tree.getNode(childId);
                if (childNode === undefined) {
                    // node was disposed
                    result.push({ parentId: rootNode.id, depth: rootNode.depth + 1, childIndex: index });
                }
                else {
                    result.push(childNode);
                    if (childNode.isExpanded && childNode.numChildren !== undefined) {
                        MutableTreeModel.getVisibleDescendants(tree, childNode, result);
                    }
                }
            }
            ++index;
        }
        return result;
    }
}
exports.MutableTreeModel = MutableTreeModel;
_a = immer_1.immerable;
//# sourceMappingURL=TreeModel.js.map