"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeModelSource = void 0;
const immer_1 = require("immer");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const TreeModel_1 = require("./TreeModel");
/**
 * Controls tree model and visible tree nodes.
 * It is used to modify model and inform when tree model changes.
 * @public
 */
class TreeModelSource {
    constructor(_model = new TreeModel_1.MutableTreeModel()) {
        this._model = _model;
        /** Event that is emitted every time tree model is changed. */
        this.onModelChanged = new bentleyjs_core_1.BeUiEvent();
        this.onModelChanged.addListener(() => this._visibleNodes = undefined);
    }
    /**
     * Modifies tree model using provided callback.
     * If changes to tree model is detected then onModelChanged event is emitted.
     */
    modifyModel(callback) {
        let changes = { addedNodeIds: [], modifiedNodeIds: [], removedNodeIds: [] };
        const newModel = immer_1.produce(this._model, (draft) => callback(draft), (patches) => { changes = this.collectModelChanges(patches); });
        if (newModel !== this._model) {
            this._model = newModel;
            this.onModelChanged.emit([this._model, changes]);
        }
    }
    /** Returns tree model. */
    getModel() { return this._model; }
    /** Computes and returns flat list of visible tree nodes. */
    getVisibleNodes() {
        if (!this._visibleNodes) {
            this._visibleNodes = this._model.computeVisibleNodes();
        }
        return this._visibleNodes;
    }
    collectModelChanges(modelPatches) {
        const addedNodeIds = [];
        const modifiedNodeIds = new Set();
        const removedNodeIds = [];
        for (const patch of modelPatches) {
            if (patch.path.length >= 3 && patch.path[0] === "_tree" && patch.path[1] === "_idToNode") {
                const nodeId = patch.path[2];
                if (patch.path.length > 3) {
                    // Modification occured somewhere inside a node
                    modifiedNodeIds.add(nodeId);
                    continue;
                }
                // Modification occured directly on _idToNode object
                switch (patch.op) {
                    case "add":
                        addedNodeIds.push(nodeId);
                        break;
                    case "remove":
                        removedNodeIds.push(nodeId);
                        break;
                    case "replace":
                        modifiedNodeIds.add(nodeId);
                        break;
                }
            }
        }
        return { addedNodeIds, modifiedNodeIds: [...modifiedNodeIds], removedNodeIds };
    }
}
exports.TreeModelSource = TreeModelSource;
//# sourceMappingURL=TreeModelSource.js.map