"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControlledTree = void 0;
const React = __importStar(require("react"));
const ui_core_1 = require("@bentley/ui-core");
const DelayedSpinner_1 = require("../../../common/DelayedSpinner");
const UiComponents_1 = require("../../../UiComponents");
const ImageLoader_1 = require("../../ImageLoader");
const Observable_1 = require("../Observable");
const TreeEventDispatcher_1 = require("../TreeEventDispatcher");
const TreeModel_1 = require("../TreeModel");
const TreeNodeRenderer_1 = require("./TreeNodeRenderer");
const TreeRenderer_1 = require("./TreeRenderer");
/**
 * React tree component which rendering is fully controlled from outside.
 * @public
 */
function ControlledTree(props) {
    const nodeHeight = useNodeHeight(!!props.descriptionsEnabled);
    const imageLoader = React.useMemo(() => new ImageLoader_1.TreeImageLoader(), []);
    const nodeRenderer = React.useCallback((nodeProps) => (React.createElement(TreeNodeRenderer_1.TreeNodeRenderer, Object.assign({}, nodeProps, { descriptionEnabled: props.descriptionsEnabled, imageLoader: props.iconsEnabled ? imageLoader : undefined }))), [props.descriptionsEnabled, props.iconsEnabled, imageLoader]);
    const eventDispatcher = useEventDispatcher(props.nodeLoader, props.treeEvents, props.selectionMode, props.visibleNodes);
    const treeProps = {
        nodeRenderer,
        nodeHeight,
        treeActions: eventDispatcher,
        nodeLoader: props.nodeLoader,
        visibleNodes: props.visibleNodes,
        nodeHighlightingProps: props.nodeHighlightingProps,
        onItemsRendered: props.onItemsRendered,
        width: props.width,
        height: props.height,
    };
    const loading = useRootNodeLoader(props.visibleNodes, props.nodeLoader);
    const noData = props.visibleNodes.getNumRootNodes() === 0;
    return (React.createElement(Loader, { loading: loading, noData: noData, spinnerRenderer: props.spinnerRenderer, noDataRenderer: props.noDataRenderer }, props.treeRenderer ? props.treeRenderer(treeProps) : React.createElement(TreeRenderer_1.TreeRenderer, Object.assign({}, treeProps))));
}
exports.ControlledTree = ControlledTree;
function useRootNodeLoader(visibleNodes, nodeLoader) {
    React.useEffect(() => {
        if (visibleNodes.getNumRootNodes() === undefined) {
            const subscription = Observable_1.toRxjsObservable(nodeLoader.loadNode(visibleNodes.getModel().getRootNode(), 0)).subscribe();
            return () => subscription.unsubscribe();
        }
        return () => { };
    }, [visibleNodes, nodeLoader]);
    return visibleNodes.getNumRootNodes() === undefined;
}
function useEventDispatcher(nodeLoader, treeEvents, selectionMode, visibleNodes) {
    /* istanbul ignore next */
    const getVisibleNodes = React.useCallback(() => visibleNodes, [visibleNodes]);
    const eventDispatcher = React.useMemo(() => new TreeEventDispatcher_1.TreeEventDispatcher(treeEvents, nodeLoader, selectionMode), [treeEvents, nodeLoader, selectionMode]);
    React.useEffect(() => {
        eventDispatcher.setVisibleNodes(getVisibleNodes);
    }, [eventDispatcher, getVisibleNodes]);
    return eventDispatcher;
}
function Loader(props) {
    if (props.loading) {
        return props.spinnerRenderer
            ? props.spinnerRenderer()
            : (React.createElement("div", { className: "components-controlledTree-loader" },
                React.createElement(DelayedSpinner_1.DelayedSpinner, { size: ui_core_1.SpinnerSize.Large })));
    }
    if (props.noData) {
        return props.noDataRenderer
            ? props.noDataRenderer()
            : (React.createElement(ui_core_1.FillCentered, null,
                React.createElement("p", { className: "components-controlledTree-errorMessage" }, UiComponents_1.UiComponents.translate("general.noData"))));
    }
    return props.children;
}
function useNodeHeight(descriptionsEnabled) {
    return React.useCallback((node) => {
        const contentHeight = (TreeModel_1.isTreeModelNode(node) && descriptionsEnabled && node && node.description) ? 43 : 24;
        const borderSize = 1;
        // Not counting node's border size twice because we want neighboring borders to overlap
        return contentHeight + borderSize;
    }, [descriptionsEnabled]);
}
//# sourceMappingURL=ControlledTree.js.map