"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeModelMutator = void 0;
const empty_1 = require("rxjs/internal/observable/empty");
/**
 * Provides basic tree manipulation implementation for various cases like
 * expand/collapse node, modify/replace/clear selection. It is used by default
 * tree event handler to modify model when events occur.
 * @internal
 */
class TreeModelMutator {
    constructor(modelSource, nodeLoader, collapsedChildrenDisposalEnabled) {
        this._modelSource = modelSource;
        this._nodeLoader = nodeLoader;
        this._collapsedChildrenDisposalEnabled = collapsedChildrenDisposalEnabled;
    }
    get modelSource() { return this._modelSource; }
    expandNode(nodeId) {
        let needToLoadChildren = false;
        this._modelSource.modifyModel((model) => {
            const node = model.getNode(nodeId);
            if (node === undefined || node.isExpanded) {
                return;
            }
            needToLoadChildren = node.numChildren === undefined;
            node.isExpanded = true;
            if (needToLoadChildren) {
                node.isLoading = true;
            }
        });
        const expandedNode = this._modelSource.getModel().getNode(nodeId);
        return needToLoadChildren && expandedNode ? this._nodeLoader.loadNode(expandedNode, 0) : empty_1.EMPTY;
    }
    collapseNode(nodeId) {
        this._modelSource.modifyModel((model) => {
            const node = model.getNode(nodeId);
            if (node === undefined || !node.isExpanded) {
                return;
            }
            node.isExpanded = false;
            if (this._collapsedChildrenDisposalEnabled) {
                model.clearChildren(node.id);
            }
        });
    }
    modifySelection(nodesToSelect, nodesToDeselect) {
        this._modelSource.modifyModel((model) => {
            for (const nodeItem of nodesToSelect) {
                const node = model.getNode(nodeItem.id);
                if (node !== undefined) {
                    node.isSelected = true;
                }
            }
            for (const nodeItem of nodesToDeselect) {
                const node = model.getNode(nodeItem.id);
                if (node !== undefined) {
                    node.isSelected = false;
                }
            }
        });
    }
    replaceSelection(nodesToSelect) {
        this._modelSource.modifyModel((model) => {
            for (const node of model.iterateTreeModelNodes()) {
                node.isSelected = false;
            }
            for (const nodeItem of nodesToSelect) {
                const node = model.getNode(nodeItem.id);
                if (node !== undefined) {
                    node.isSelected = true;
                }
            }
        });
    }
    clearNodeSelection() {
        this._modelSource.modifyModel((model) => {
            for (const node of model.iterateTreeModelNodes()) {
                node.isSelected = false;
            }
        });
    }
    setCheckboxStates(stateChanges) {
        this._modelSource.modifyModel((model) => {
            for (const { nodeItem, newState } of stateChanges) {
                const node = model.getNode(nodeItem.id);
                if (node !== undefined) {
                    node.checkbox.state = newState;
                }
            }
        });
    }
    activateEditing(nodeId, onCommit) {
        this._modelSource.modifyModel((model) => {
            const node = model.getNode(nodeId);
            if (!node)
                return;
            if (node.item.isEditable) {
                node.editingInfo = this.createNodeEditingInfo(nodeId, onCommit);
            }
        });
    }
    createNodeEditingInfo(nodeId, onCommit) {
        const closeEditing = () => {
            this._modelSource.modifyModel((model) => {
                const node = model.getNode(nodeId);
                // istanbul ignore if
                if (!node)
                    return;
                node.editingInfo = undefined;
            });
        };
        const onEditCommitted = (node, newValue) => {
            onCommit(node, newValue);
            closeEditing();
        };
        return { onCommit: onEditCommitted, onCancel: closeEditing };
    }
}
exports.TreeModelMutator = TreeModelMutator;
//# sourceMappingURL=TreeModelMutator.js.map