"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeSelectionManager = exports.isRangeSelection = void 0;
const take_1 = require("rxjs/internal/operators/take");
const Subject_1 = require("rxjs/internal/Subject");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const SelectionHandler_1 = require("../../../common/selection/SelectionHandler");
const TreeModel_1 = require("../TreeModel");
const ui_core_1 = require("@bentley/ui-core");
/** @internal */
function isRangeSelection(selection) {
    return selection && typeof (selection.from) === "string" && typeof (selection.to) === "string";
}
exports.isRangeSelection = isRangeSelection;
/** @internal */
class TreeSelectionManager {
    constructor(selectionMode, getVisibleNodes) {
        this.onSelectionChanged = new bentleyjs_core_1.BeUiEvent();
        this.onSelectionReplaced = new bentleyjs_core_1.BeUiEvent();
        this.onDragSelection = new bentleyjs_core_1.BeUiEvent();
        this._onKeyboardEvent = (e, actions, isKeyDown) => {
            if (ui_core_1.isNavigationKey(e.key)) {
                // istanbul ignore next
                if (!this._getVisibleNodes)
                    return;
                const processedNodeId = this._selectionHandler.processedItem;
                const isExpandable = (node) => !node.isLoading && node.numChildren !== 0;
                const isEditing = (node) => node.editingInfo !== undefined;
                // istanbul ignore else
                if (this._getVisibleNodes && isIndividualSelection(processedNodeId)) {
                    const processedNode = this._getVisibleNodes().getModel().getNode(processedNodeId);
                    // istanbul ignore next
                    if (processedNode && isEditing(processedNode))
                        return;
                }
                const handleKeyboardSelectItem = (index) => {
                    // istanbul ignore else
                    if (this._getVisibleNodes) {
                        const node = this.getVisibleNodeAtIndex(this._getVisibleNodes(), index);
                        // istanbul ignore else
                        if (node) {
                            // istanbul ignore next
                            if (isEditing(node))
                                return;
                            const selectionFunc = this._selectionHandler.createSelectionFunction(...this.createSelectionHandlers(node.id));
                            selectionFunc(e.shiftKey, e.ctrlKey);
                        }
                    }
                };
                const handleKeyboardActivateItem = (_index) => {
                    // istanbul ignore else
                    if (this._getVisibleNodes && isIndividualSelection(processedNodeId)) {
                        const node = this._getVisibleNodes().getModel().getNode(processedNodeId);
                        // istanbul ignore else
                        if (node) {
                            if (isExpandable(node)) {
                                if (!node.isExpanded)
                                    actions.onNodeExpanded(node.id);
                                else
                                    actions.onNodeCollapsed(node.id);
                            }
                            else {
                                actions.onNodeEditorActivated(node.id);
                            }
                        }
                    }
                };
                const handleCrossAxisArrowKey = (forward) => {
                    // istanbul ignore else
                    if (this._getVisibleNodes && isIndividualSelection(processedNodeId)) {
                        const node = this._getVisibleNodes().getModel().getNode(processedNodeId);
                        // istanbul ignore else
                        if (node) {
                            if (isExpandable(node)) {
                                if (forward && !node.isExpanded)
                                    actions.onNodeExpanded(node.id);
                                else if (!forward && node.isExpanded)
                                    actions.onNodeCollapsed(node.id);
                            }
                        }
                    }
                };
                // istanbul ignore else
                if (isIndividualSelection(processedNodeId)) {
                    const itemKeyboardNavigator = new ui_core_1.ItemKeyboardNavigator(handleKeyboardSelectItem, handleKeyboardActivateItem);
                    itemKeyboardNavigator.orientation = ui_core_1.Orientation.Vertical;
                    itemKeyboardNavigator.crossAxisArrowKeyHandler = handleCrossAxisArrowKey;
                    itemKeyboardNavigator.allowWrap = false;
                    itemKeyboardNavigator.itemCount = this._getVisibleNodes().getNumNodes();
                    const index = this._getVisibleNodes().getIndexOfNode(processedNodeId);
                    isKeyDown ?
                        itemKeyboardNavigator.handleKeyDownEvent(e, index) :
                        itemKeyboardNavigator.handleKeyUpEvent(e, index);
                }
            }
        };
        this._getVisibleNodes = getVisibleNodes;
        const onItemsSelected = (selections, replacement) => {
            // Assumes that `selections` will only contain either a list of
            // `IndividualSelection` items or a single `RangeSelection` item.
            if (isIndividualSelection(selections[0])) {
                if (replacement) {
                    this.onSelectionReplaced.emit({
                        selectedNodeIds: selections,
                    });
                }
                else {
                    this.onSelectionChanged.emit({
                        selectedNodes: selections,
                        deselectedNodes: [],
                    });
                }
            }
            else {
                if (replacement) {
                    this.onSelectionReplaced.emit({
                        selectedNodeIds: selections[0],
                    });
                }
                else {
                    this.onSelectionChanged.emit({
                        selectedNodes: selections[0],
                        deselectedNodes: [],
                    });
                }
            }
        };
        const onItemsDeselected = (deselections) => {
            // Assumes that this will be always called with `IndividualSelection` items
            this.onSelectionChanged.emit({
                selectedNodes: [],
                deselectedNodes: deselections,
            });
        };
        this._selectionHandler = new SelectionHandler_1.SelectionHandler(selectionMode, onItemsSelected, onItemsDeselected);
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const _this = this;
        const itemHandlers = new Proxy({}, {
            get(_target, prop) {
                if (prop === "length") {
                    return _this._getVisibleNodes === undefined ? 0 : _this._getVisibleNodes().getNumNodes();
                }
                // istanbul ignore next
                if (_this._getVisibleNodes === undefined)
                    return undefined;
                const index = +prop;
                const node = _this.getVisibleNodeAtIndex(_this._getVisibleNodes(), index);
                return new ItemHandler(node);
            },
        });
        this._itemHandlers = [itemHandlers];
    }
    getVisibleNodeAtIndex(nodes, index) {
        const foundNode = nodes !== undefined ? nodes.getAtIndex(index) : /* istanbul ignore next */ undefined;
        return TreeModel_1.isTreeModelNode(foundNode) ? foundNode : undefined;
    }
    setVisibleNodes(visibleNodes) {
        this._getVisibleNodes = visibleNodes;
    }
    onNodeClicked(nodeId, event) {
        const selectionFunc = this._selectionHandler.createSelectionFunction(...this.createSelectionHandlers(nodeId));
        selectionFunc(event.shiftKey, event.ctrlKey);
    }
    onNodeMouseDown(nodeId) {
        this._selectionHandler.createDragAction(this.createSelectionHandlers(nodeId)[0], this._itemHandlers, nodeId);
        window.addEventListener("mouseup", () => {
            /* istanbul ignore else */
            if (this._dragSelectionOperation) {
                this._selectionHandler.completeDragAction();
                this._dragSelectionOperation.complete();
                this._dragSelectionOperation = undefined;
            }
        }, { once: true });
        this._dragSelectionOperation = new Subject_1.Subject();
        this._dragSelectionOperation
            .pipe(take_1.take(1))
            .subscribe((value) => {
            this.onDragSelection.emit({ selectionChanges: this._dragSelectionOperation });
            this._dragSelectionOperation.next(value);
        });
    }
    onNodeMouseMove(nodeId) {
        this._selectionHandler.updateDragAction(nodeId);
    }
    createSelectionHandlers(nodeId) {
        let deselectedAll = false;
        const multiSelectionHandler = {
            selectBetween: (item1, item2) => [{ from: item1, to: item2 }],
            updateSelection: (selections, deselections) => {
                // Assumes `updateSelection` will never be called with selection ranges
                const selectedNodeIds = selections;
                const deselectedNodeIds = deselections;
                this._dragSelectionOperation.next({ selectedNodes: selectedNodeIds, deselectedNodes: deselectedNodeIds });
            },
            deselectAll: () => {
                deselectedAll = true;
            },
            areEqual: (item1, item2) => item1 === item2,
        };
        const singleSelectionHandler = {
            preselect: () => { },
            select: () => { },
            deselect: () => { },
            isSelected: () => {
                if (deselectedAll || this._getVisibleNodes === undefined) {
                    return false;
                }
                const node = this._getVisibleNodes().getModel().getNode(nodeId);
                return node !== undefined && node.isSelected;
            },
            item: () => nodeId,
        };
        return [multiSelectionHandler, singleSelectionHandler];
    }
    onTreeKeyDown(event, actions) {
        this._onKeyboardEvent(event, actions, true);
    }
    onTreeKeyUp(event, actions) {
        this._onKeyboardEvent(event, actions, false);
    }
}
exports.TreeSelectionManager = TreeSelectionManager;
function isIndividualSelection(selection) {
    return typeof (selection) === "string";
}
class ItemHandler {
    constructor(node) {
        this._node = node;
    }
    /* istanbul ignore next: noop */
    preselect() { }
    /* istanbul ignore next: noop */
    select() { }
    /* istanbul ignore next: noop */
    deselect() { }
    // eslint-disable-next-line @bentley/prefer-get
    isSelected() {
        var _a;
        // istanbul ignore next
        return !!((_a = this._node) === null || _a === void 0 ? void 0 : _a.isSelected);
    }
    item() {
        var _a, _b;
        return (_b = (_a = this._node) === null || _a === void 0 ? void 0 : _a.id) !== null && _b !== void 0 ? _b : "";
    }
}
//# sourceMappingURL=TreeSelectionManager.js.map