"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.onCancelation = exports.makeObservableCallback = exports.NodeSet = exports.NodeKey = exports.PendingNodeTracker = exports.NodeLoadingOrchestrator = void 0;
const Observable_1 = require("rxjs/internal/Observable");
const concat_1 = require("rxjs/internal/observable/concat");
const merge_1 = require("rxjs/internal/observable/merge");
const of_1 = require("rxjs/internal/observable/of");
const filter_1 = require("rxjs/internal/operators/filter");
const finalize_1 = require("rxjs/internal/operators/finalize");
const map_1 = require("rxjs/internal/operators/map");
const mergeAll_1 = require("rxjs/internal/operators/mergeAll");
const publish_1 = require("rxjs/internal/operators/publish");
const share_1 = require("rxjs/internal/operators/share");
const takeUntil_1 = require("rxjs/internal/operators/takeUntil");
const takeWhile_1 = require("rxjs/internal/operators/takeWhile");
const tap_1 = require("rxjs/internal/operators/tap");
const Subject_1 = require("rxjs/internal/Subject");
const BeInspireTree_1 = require("./component/BeInspireTree");
/**
 * @internal @deprecated
 * Loads tree nodes for event handling. Makes sure that the order of
 * subscribers that receive loaded nodes is the same as their subscription order.
 */
class NodeLoadingOrchestrator {
    constructor(model, callbacks) {
        this._nodeLoadEvents = new Subject_1.Subject();
        this._pendingNodeTracker = new PendingNodeTracker();
        this._activeNodeLoads = new Set();
        this._reportProgress = () => {
            this._callbacks.onLoadProgress(this._pendingNodeTracker.getTotalAddedNodes() - this._pendingNodeTracker.getNumPendingNodes(), this._pendingNodeTracker.getTotalAddedNodes(), () => this.cancelLoading());
        };
        this._model = model;
        this._callbacks = callbacks;
        const [cancelLoading, onLoadingCanceled] = makeObservableCallback();
        this._cancelLoading = cancelLoading;
        this._onLoadingCanceled = onLoadingCanceled;
        // Central node load event transmitter, handles global progress notification and cancelation.
        this._onNodesLoaded = new Observable_1.Observable(
        // We get here when the `share` operator subscribes
        (subscriber) => {
            // Relay node load events to the subscriber
            const subscription = this._nodeLoadEvents
                .pipe(onCancelation(() => {
                if (this._pendingNodeTracker.getNumPendingNodes() === 0) {
                    // Progress is normally reported before nodes are loaded. Final tally needs to be reported here.
                    this._reportProgress();
                    this._callbacks.onLoadFinished();
                }
                else {
                    this._pendingNodeTracker.reset();
                    this._callbacks.onLoadCanceled();
                }
            }), takeUntil_1.takeUntil(this._onLoadingCanceled))
                .subscribe(subscriber);
            // Begin loading nodes
            this.requestPendingNodeLoad();
            return () => {
                // We get here when the `share` operator unsubscribes
                this._pendingNodeTracker.reset();
                this._activeNodeLoads.clear();
                subscription.unsubscribe();
            };
        })
            .pipe(
        // Multicasts node load events. Subscribes to the source observable when the
        // first observer subscribes, and unsubscribes after the last observer unsubscribes.
        share_1.share());
    }
    /** Stops all ongoing node preparation operations. */
    cancelLoading() {
        this._cancelLoading();
    }
    /**
     * Makes sure that all input nodes are loaded.
     * @returns Observable that first emits already loaded input node
     * synchronously (empty array if none) and then the rest as nodes get loaded.
     */
    prepareNodes(nodes) {
        const [preloadedNodes, nodesToLoad] = NodeLoadingOrchestrator.sortNodesByLoadingState(nodes);
        // If no new nodes need to be loaded, then return immediately
        if (nodesToLoad.empty) {
            return of_1.of(preloadedNodes).pipe(makeHot());
        }
        this._pendingNodeTracker.addNodes(nodesToLoad);
        // Observable that emits loaded nodes immediately, then pending nodes once they are loaded
        const observable = concat_1.concat(of_1.of(preloadedNodes)
            .pipe(tap_1.tap(this._reportProgress), onCancelation(() => {
            // Handle cancelation only if it has not been handled yet
            if (this._pendingNodeTracker.getNumPendingNodes() > 0) {
                this._pendingNodeTracker.reset();
                this._callbacks.onLoadCanceled();
            }
        })), this._onNodesLoaded
            .pipe(map_1.map((loadedNodes) => loadedNodes.filter((node) => nodesToLoad.delete(NodeKey.for(node)))), takeWhile_1.takeWhile(() => !nodesToLoad.empty, true), filter_1.filter(({ length }) => length > 0))).pipe(
        // If cancelation occurs before we begin loading nodes, this prevents node loading from starting.
        takeUntil_1.takeUntil(this._onLoadingCanceled), finalize_1.finalize(() => {
            this._activeNodeLoads.delete(observable);
        }), makeHot());
        this._activeNodeLoads.add(observable);
        return observable;
    }
    /**
     * Makes sure that all nodes between and including the input nodes are loaded.
     * Input nodes can be supplied in any order and are assumed to be loaded.
     * @returns Observable that first emits already loaded nodes synchronously
     * (empty array if none) and then the rest as nodes and their children get loaded.
     */
    prepareNodesBetween(node1, node2) {
        const visibleNodesBetween = this._model.getVisibleNodesBetween(node1, node2);
        const [preloadedNodes, nodesToLoadRecursively] = NodeLoadingOrchestrator.sortNodesByLoadingState(visibleNodesBetween);
        // If no new nodes need to be loaded, then return immediately
        if (nodesToLoadRecursively.empty) {
            return of_1.of(preloadedNodes).pipe(makeHot());
        }
        this._pendingNodeTracker.addNodesRecursively(nodesToLoadRecursively);
        let nodesBetween = [];
        const emittedNodes = new NodeSet(preloadedNodes.map((node) => NodeKey.for(node)));
        // Observable that emits loaded nodes immediately, then pending nodes once they are loaded
        const observable = concat_1.concat(of_1.of(preloadedNodes)
            .pipe(tap_1.tap(this._reportProgress), onCancelation(() => {
            // Handle cancelation only if it has not been handled yet
            if (this._pendingNodeTracker.getNumPendingNodes() > 0) {
                this._pendingNodeTracker.reset();
                this._callbacks.onLoadCanceled();
            }
        })), this._onNodesLoaded
            .pipe(tap_1.tap(() => nodesBetween = this._model.getVisibleNodesBetween(node1, node2)), map_1.map((loadedNodes) => {
            const nodeSet = new NodeSet(nodesBetween.map((node) => NodeKey.for(node)));
            return loadedNodes.filter((node) => {
                const key = NodeKey.for(node);
                return node.payload && nodeSet.has(key) && !emittedNodes.has(key);
            });
        }), tap_1.tap((loadedNodes) => loadedNodes.forEach((node) => emittedNodes.add(NodeKey.for(node)))), takeWhile_1.takeWhile(() => {
            // Take while there are still nodes in between that we have not emitted
            if (emittedNodes.size !== nodesBetween.length) {
                return true;
            }
            // Or while there are still nodes waiting for their child list to load
            return nodesBetween.some((node) => node.expanded() && typeof node.children === "boolean");
        }, true), filter_1.filter(({ length }) => length > 0)))
            .pipe(
        // If cancelation occurs before we begin loading nodes, this prevents node loading from starting.
        takeUntil_1.takeUntil(this._onLoadingCanceled), finalize_1.finalize(() => {
            this._activeNodeLoads.delete(observable);
        }), makeHot());
        this._activeNodeLoads.add(observable);
        return observable;
    }
    /**
     * Makes sure that all currently pending nodes get loaded.
     * @returns Observable that emits nodes once they are loaded.
     */
    preparePendingNodes() {
        const emittedNodes = new NodeSet();
        const activeNodeLoads = Array.from(this._activeNodeLoads);
        return merge_1.merge(activeNodeLoads)
            .pipe(mergeAll_1.mergeAll(), map_1.map((loadedNodes) => loadedNodes.filter((node) => !emittedNodes.has(NodeKey.for(node)))), tap_1.tap((loadedNodes) => loadedNodes.forEach((node) => emittedNodes.add(NodeKey.for(node)))), filter_1.filter(({ length }) => length > 0));
    }
    /** Returns observable that emits loaded visible nodes and then completes synchronously. */
    prepareLoadedNodes() {
        const loadedNodes = NodeLoadingOrchestrator.sortNodesByLoadingState(this._model.visible())[0];
        return of_1.of(loadedNodes);
    }
    /** Notifies the consumer about load progress and makes a request to load some pending nodes. */
    requestPendingNodeLoad() {
        this._reportProgress();
        /** Takes first n values from the input iterable */
        function* take(iterable, n) {
            const iterator = iterable[Symbol.iterator]();
            for (let i = 0; i < n; ++i) {
                const { done, value } = iterator.next();
                if (value !== undefined || !done) {
                    yield value;
                }
                if (done) {
                    return;
                }
            }
        }
        const promises = Array.from(take(this._pendingNodeTracker, this._model.props.pageSize))
            .map(async (node) => this.requestNodeLoad(node));
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        Promise.all(promises).then((loadedNodes) => {
            const collectedLoadedNodes = [];
            for (const loadedNode of loadedNodes) {
                this._pendingNodeTracker.onNodeLoaded(loadedNode).forEach((node) => collectedLoadedNodes.push(node));
            }
            this._nodeLoadEvents.next(collectedLoadedNodes);
            if (!this._pendingNodeTracker.empty) {
                this.requestPendingNodeLoad();
            }
        });
    }
    async requestNodeLoad(nodeKey) {
        const parent = nodeKey.parentId ? this._model.node(nodeKey.parentId) : undefined;
        await this._model.requestNodeLoad(parent, nodeKey.childIndex);
        return nodeKey.toNode(this._model);
    }
    /**
     * Sorts input nodes into loaded and pending node groups.
     * @returns A [loadedNodes, pendingNodes] tuple.
     */
    static sortNodesByLoadingState(nodesToSort) {
        const loadedNodes = [];
        const pendingNodes = new NodeSet();
        for (const node of nodesToSort) {
            if (node.payload) {
                loadedNodes.push(node);
            }
            else {
                pendingNodes.add(NodeKey.for(node));
            }
        }
        return [loadedNodes, pendingNodes];
    }
}
exports.NodeLoadingOrchestrator = NodeLoadingOrchestrator;
/**
 * @internal @deprecated
 * Keeps track of which nodes are queued for loading.
 */
class PendingNodeTracker {
    constructor() {
        /** Nodes that do not yet have payload */
        this._pendingNodes = new NodeSet();
        /** Nodes that do not yet have payload and need their children to be queued for loading too */
        this._pendingNodesRecursive = new NodeSet();
        /** Amount of total unique nodes that were queued for loading */
        this._totalAddedNodes = 0;
    }
    /** Queues input nodes for loading. When node's payload gets loaded, the node is removed from this collection. */
    addNodes(nodes) {
        const sizeBefore = this._pendingNodes.size;
        for (const node of nodes) {
            if (!this._pendingNodesRecursive.has(node)) {
                this._pendingNodes.add(node);
            }
        }
        this._totalAddedNodes += this._pendingNodes.size - sizeBefore;
    }
    /**
     * Queues input nodes for loading. When node's payload gets loaded, the node
     * is removed and its children are queued recursively.
     */
    addNodesRecursively(nodes) {
        const sizeBefore = this._pendingNodes.size + this._pendingNodesRecursive.size;
        for (const node of nodes) {
            this._pendingNodes.delete(node);
            this._pendingNodesRecursive.add(node);
        }
        this._totalAddedNodes += this._pendingNodes.size + this._pendingNodesRecursive.size - sizeBefore;
    }
    /**
     * Notifies the tracker that a node got its payload loaded. Queues node's
     * children for loading if the parent is set to load recursively.
     * @returns A list of nodes that have been awaited and got loaded.
     */
    onNodeLoaded(loadedNode) {
        if (!loadedNode.payload) {
            // Node is not loaded
            return [];
        }
        const nodeKey = NodeKey.for(loadedNode);
        const inPendingNodesRecursive = this._pendingNodesRecursive.delete(nodeKey);
        if (inPendingNodesRecursive) {
            const loadedNodes = [loadedNode];
            this.addChildNodesRecursively(loadedNode).forEach((node) => loadedNodes.push(node));
            return loadedNodes;
        }
        const inPendingNodes = this._pendingNodes.delete(nodeKey);
        if (inPendingNodes) {
            return [loadedNode];
        }
        return [];
    }
    /** Resets the traker's state. */
    reset() {
        this._pendingNodes.clear();
        this._pendingNodesRecursive.clear();
        this._totalAddedNodes = 0;
    }
    /** Returns the total amount of nodes that were queued for loading. */
    getTotalAddedNodes() {
        return this._totalAddedNodes;
    }
    /**
     * Returns the number of nodes that are currently being awaited, including
     * loaded nodes with pending child lists.
     */
    getNumPendingNodes() {
        return this._pendingNodes.size + this._pendingNodesRecursive.size;
    }
    /** Returns `true` if the tracker has some pending nodes; otherwise `false`. */
    get empty() {
        return this._pendingNodes.empty && this._pendingNodesRecursive.empty;
    }
    /**
     * Iterates over nodes that are pending payload load. Does not include loaded
     * nodes with pending child lists.
     */
    [Symbol.iterator]() {
        /** Concatenates input iterables */
        // eslint-disable-next-line @typescript-eslint/no-shadow
        function* concat(...iterables) {
            for (const iterable of iterables) {
                for (const value of iterable) {
                    yield value;
                }
            }
        }
        return concat(this._pendingNodesRecursive, this._pendingNodes);
    }
    /**
     * Recursively adds children to pending nodes collection.
     * @returns Array of nodes that have their payload loaded and (including `parent`).
     */
    addChildNodesRecursively(parent) {
        const loadedNodes = [];
        const parentStack = [parent];
        while (parentStack.length > 0) {
            const currentParent = parentStack.pop();
            if (!currentParent.payload) {
                continue;
            }
            if (!currentParent.expanded()) {
                continue;
            }
            const childrenToLoad = [];
            for (const child of BeInspireTree_1.toNodes(currentParent.getChildren())) {
                parentStack.push(child);
                if (child.payload) {
                    loadedNodes.push(child);
                }
                else {
                    childrenToLoad.push(NodeKey.for(child));
                }
            }
            this.addNodesRecursively(childrenToLoad);
        }
        return loadedNodes;
    }
}
exports.PendingNodeTracker = PendingNodeTracker;
/**
 * @internal @deprecated
 * Node identifier used by [[NodeSet]]
 */
class NodeKey {
    constructor(parentId, childIndex) {
        this.parentId = parentId;
        this.childIndex = childIndex;
    }
    /** Retrieves a node from the input tree that is represented by this key. */
    toNode(tree) {
        if (this.parentId) {
            return BeInspireTree_1.toNode(tree.node(this.parentId).getChildren()[this.childIndex]);
        }
        return tree.nodes()[this.childIndex];
    }
    /** Creates a [[NodeKey]] which represents the input node. */
    static for(node) {
        return new NodeKey(NodeKey.getParentId(node), NodeKey.getChildIndex(node));
    }
    /** Identifies the input node's parent. */
    static getParentId(node) {
        const parentNode = BeInspireTree_1.toNode(node.getParent());
        return parentNode ? parentNode.id : undefined;
    }
    /** Returns input node index in its parent's child list */
    static getChildIndex(node) {
        if (!node.payload) {
            return node.placeholderIndex;
        }
        return node.context().indexOf(node);
    }
}
exports.NodeKey = NodeKey;
/**
 * @internal
 * A set that stores unique nodes of a tree. Nodes are identified by their position in the hierarchy.
 */
class NodeSet {
    /** Constructs the set and inserts input nodes */
    constructor(nodes = []) {
        this._parentToChildToValue = new Map();
        nodes.forEach((node) => this.add(node));
    }
    /** Calculates the number of nodes in the set. */
    get size() {
        let size = 0;
        for (const childToValue of this._parentToChildToValue.values()) {
            size += childToValue.size;
        }
        return size;
    }
    /** Checks whether the set is empty */
    get empty() {
        return this._parentToChildToValue.size === 0;
    }
    /**
     * Adds a node to the set.
     * @returns `this`.
     */
    add(node) {
        const childToValue = this._parentToChildToValue.get(node.parentId) || new Set();
        childToValue.add(node.childIndex);
        this._parentToChildToValue.set(node.parentId, childToValue);
        return this;
    }
    /**
     * Removes node from the set.
     * @returns `true` if an element in the map existed and has been removed, or `false` if the element does not exist.
     */
    delete(node) {
        const childToValue = this._parentToChildToValue.get(node.parentId);
        if (!childToValue) {
            return false;
        }
        const deleted = childToValue.delete(node.childIndex);
        if (childToValue.size === 0) {
            this._parentToChildToValue.delete(node.parentId);
        }
        return deleted;
    }
    /** Removes all nodes from the set. */
    clear() {
        this._parentToChildToValue.clear();
    }
    /** Checks whether the input node is present in the set. */
    has(node) {
        const childToValue = this._parentToChildToValue.get(node.parentId);
        if (!childToValue) {
            return false;
        }
        return childToValue.has(node.childIndex);
    }
    /**
     * Iterates over each value in the set. Unlike regular `Set`, this class does not preserve element insertion order.
     */
    [Symbol.iterator]() {
        const parentToChildToValue = this._parentToChildToValue;
        return (function* () {
            for (const [parentId, children] of parentToChildToValue) {
                for (const childIndex of children) {
                    yield new NodeKey(parentId, childIndex);
                }
            }
        })();
    }
}
exports.NodeSet = NodeSet;
/**
 * @internal
 * Creates a hot `Observable` that receives values from function invocations.
 * @returns A tuple of `[nextFunc, observable]`.
 */
function makeObservableCallback() {
    const subject = new Subject_1.Subject();
    return [subject.next.bind(subject), subject.asObservable()];
}
exports.makeObservableCallback = makeObservableCallback;
/**
 * @internal
 * Invokes `onCanceled` callback if error occurs or if the observable is unsubscribed from before having been completed.
 * @returns An `Observable` that emits each value from the input `Observable` and then completes.
 */
function onCancelation(onCanceled) {
    return (observable) => {
        return new Observable_1.Observable((subscriber) => {
            let completed = false;
            const subscription = observable
                .pipe(tap_1.tap({
                complete: () => completed = true,
            }))
                .subscribe(subscriber);
            return () => {
                subscription.unsubscribe();
                if (!completed) {
                    onCanceled();
                }
            };
        });
    };
}
exports.onCancelation = onCancelation;
function makeHot() {
    return (observable) => {
        const connectableObservable = observable.pipe(publish_1.publish());
        return new Observable_1.Observable((subscriber) => {
            const subscription = connectableObservable.subscribe(subscriber);
            connectableObservable.connect();
            return subscription;
        });
    };
}
//# sourceMappingURL=NodeLoadingOrchestrator.js.map