"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Viewport
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewportComponent = void 0;
const React = __importStar(require("react"));
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const geometry_core_1 = require("@bentley/geometry-core");
const imodeljs_common_1 = require("@bentley/imodeljs-common");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const UiComponents_1 = require("../UiComponents");
const ViewportComponentEvents_1 = require("./ViewportComponentEvents");
/**
 * A viewport React component that creates a ScreenViewport.
 * @public
 */
class ViewportComponent extends React.Component {
    constructor(props) {
        super(props);
        this._viewClassFullName = "";
        this._mounted = false;
        this._handleDisconnectFromViewManager = () => {
            const screenViewport = this._vp;
            const parentDiv = this._viewportDiv.current;
            if (screenViewport) {
                const viewManager = imodeljs_frontend_1.IModelApp.viewManager;
                viewManager.dropViewport(screenViewport, true);
                screenViewport.onViewChanged.removeListener(this._handleViewChanged);
                this._vp = undefined;
            }
            // istanbul ignore else
            if (parentDiv) {
                const parentWindow = parentDiv.ownerDocument.defaultView;
                parentWindow.removeEventListener("beforeunload", this._handleDisconnectFromViewManager, false);
            }
        };
        this._handleDrawingViewportChangeEvent = (args) => {
            const viewManager = this.props.viewManagerOverride ? this.props.viewManagerOverride : /* istanbul ignore next */ imodeljs_frontend_1.IModelApp.viewManager;
            /* istanbul ignore else */
            if (this._vp && viewManager.selectedView === this._vp) {
                this._vp.view.setOrigin(args.origin);
                this._vp.view.setRotation(args.rotation);
                this._vp.synchWithView({ noSaveInUndo: args.complete !== true });
            }
        };
        this._handleCubeRotationChangeEvent = (args) => {
            const viewManager = this.props.viewManagerOverride ? this.props.viewManagerOverride : /* istanbul ignore next */ imodeljs_frontend_1.IModelApp.viewManager;
            /* istanbul ignore else */
            if (this._vp && viewManager.selectedView === this._vp) {
                const rotMatrix = args.rotMatrix;
                /* istanbul ignore else */
                if (this._vp.rotation !== rotMatrix) {
                    const inverse = rotMatrix.transpose(); // rotation is from current nav cube state...
                    const center = this._getRotatePoint(this._vp);
                    const targetMatrix = inverse.multiplyMatrixMatrix(this._vp.view.getRotation());
                    const worldTransform = geometry_core_1.Transform.createFixedPointAndMatrix(center, targetMatrix);
                    const frustum = this._vp.getWorldFrustum();
                    frustum.multiply(worldTransform);
                    this._vp.view.setupFromFrustum(frustum);
                    this._vp.synchWithView({ noSaveInUndo: !args.complete });
                }
            }
        };
        this._handleStandardRotationChangeEvent = (args) => {
            const viewManager = this.props.viewManagerOverride ? this.props.viewManagerOverride : /* istanbul ignore next */ imodeljs_frontend_1.IModelApp.viewManager;
            /* istanbul ignore else */
            if (this._vp && viewManager.selectedView === this._vp) {
                // this._vp.view.setStandardRotation(args.standardRotation);
                this._vp.view.setRotationAboutPoint(imodeljs_frontend_1.ViewState.getStandardViewMatrix(args.standardRotation));
                this._vp.synchWithView();
            }
        };
        this._handleViewChanged = (vp) => {
            ViewportComponentEvents_1.ViewportComponentEvents.setViewMatrix(vp);
            /* istanbul ignore else */
            if (this._viewClassFullName !== vp.view.classFullName) {
                setTimeout(() => {
                    ViewportComponentEvents_1.ViewportComponentEvents.onViewClassFullNameChangedEvent.emit({ viewport: vp, oldName: this._viewClassFullName, newName: vp.view.classFullName });
                    this._viewClassFullName = vp.view.classFullName;
                });
            }
            /* istanbul ignore else */
            if (this.state.viewId !== vp.view.id) {
                setTimeout(() => {
                    ViewportComponentEvents_1.ViewportComponentEvents.onViewIdChangedEvent.emit({ viewport: vp, oldId: this.state.viewId, newId: vp.view.id });
                    /* istanbul ignore next - flaky mounted condition in the setTimeout */
                    if (this._mounted)
                        this.setState({ viewId: vp.view.id });
                });
            }
        };
        this._handleContextMenu = (e) => {
            e.preventDefault();
            if (this.props.onContextMenu)
                this.props.onContextMenu(e);
            return false;
        };
        this._viewportDiv = React.createRef();
        this.state = {
            viewId: "",
        };
    }
    async componentDidMount() {
        this._mounted = true;
        // istanbul ignore next
        if (!this._viewportDiv.current) {
            bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `Parent <div> failed to load`);
            return;
        }
        const viewState = await this.getViewState();
        if (viewState === undefined) {
            bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `Failed to obtain ViewState`);
            return;
        }
        /* istanbul ignore next */
        if (!this._mounted)
            return;
        const viewManager = this.props.viewManagerOverride ? this.props.viewManagerOverride : /* istanbul ignore next */ imodeljs_frontend_1.IModelApp.viewManager;
        const screenViewportFactory = this.props.screenViewportOverride ? this.props.screenViewportOverride : /* istanbul ignore next */ imodeljs_frontend_1.ScreenViewport;
        const parentDiv = this._viewportDiv.current;
        const screenViewport = screenViewportFactory.create(parentDiv, viewState);
        this._vp = screenViewport;
        viewManager.addViewport(this._vp);
        const parentWindow = parentDiv.ownerDocument.defaultView;
        parentWindow.addEventListener("beforeunload", this._handleDisconnectFromViewManager, false);
        ViewportComponentEvents_1.ViewportComponentEvents.initialize();
        ViewportComponentEvents_1.ViewportComponentEvents.onDrawingViewportChangeEvent.addListener(this._handleDrawingViewportChangeEvent);
        ViewportComponentEvents_1.ViewportComponentEvents.onCubeRotationChangeEvent.addListener(this._handleCubeRotationChangeEvent);
        ViewportComponentEvents_1.ViewportComponentEvents.onStandardRotationChangeEvent.addListener(this._handleStandardRotationChangeEvent);
        this._vp.onViewChanged.addListener(this._handleViewChanged);
        this._viewClassFullName = this._vp.view.classFullName;
        this.setState({ viewId: this._vp.view.id });
        /* istanbul ignore else */
        if (this.props.viewportRef)
            this.props.viewportRef(this._vp);
    }
    componentWillUnmount() {
        this._mounted = false;
        this._handleDisconnectFromViewManager();
        ViewportComponentEvents_1.ViewportComponentEvents.onDrawingViewportChangeEvent.removeListener(this._handleDrawingViewportChangeEvent);
        ViewportComponentEvents_1.ViewportComponentEvents.onCubeRotationChangeEvent.removeListener(this._handleCubeRotationChangeEvent);
        ViewportComponentEvents_1.ViewportComponentEvents.onStandardRotationChangeEvent.removeListener(this._handleStandardRotationChangeEvent);
    }
    async componentDidUpdate(prevProps) {
        if (this.props.imodel === prevProps.imodel &&
            this.props.viewState === prevProps.viewState &&
            this.props.viewDefinitionId === prevProps.viewDefinitionId)
            return;
        /* istanbul ignore else */
        if (this._vp) {
            const viewState = await this.getViewState();
            if (viewState === undefined) {
                bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `Failed to obtain ViewState`);
                return;
            }
            this._vp.changeView(viewState);
            /* istanbul ignore else */
            if (this._mounted)
                this.setState({ viewId: this._vp.view.id });
        }
    }
    async getViewState() {
        let viewState;
        if (this.props.viewState) {
            if (typeof this.props.viewState === "function")
                viewState = this.props.viewState();
            else
                viewState = this.props.viewState;
        }
        else if (this.props.viewDefinitionId) {
            viewState = await this.props.imodel.views.load(this.props.viewDefinitionId);
            if (!viewState) {
                bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `View state failed to load`);
                return undefined;
            }
            else {
                return viewState;
            }
        }
        else {
            bentleyjs_core_1.Logger.logError(UiComponents_1.UiComponents.loggerCategory(this), `Either viewDefinitionId or viewState must be provided as a ViewportComponent Prop`);
            return undefined;
        }
        return viewState.clone();
    }
    _getRotatePoint(vp) {
        const tentativePoint = this.props.tentativePointOverride ? this.props.tentativePointOverride : /* istanbul ignore next */ imodeljs_frontend_1.IModelApp.tentativePoint;
        if (tentativePoint.isActive)
            return tentativePoint.getPoint();
        /* istanbul ignore else */
        if (undefined !== vp.viewCmdTargetCenter) {
            const testPt = vp.worldToView(vp.viewCmdTargetCenter);
            const viewRect = vp.viewRect;
            if (viewRect.containsPoint(testPt))
                return vp.viewCmdTargetCenter;
            vp.viewCmdTargetCenter = undefined;
        }
        if (undefined !== this._lastTargetPoint) {
            const testPt = vp.worldToView(this._lastTargetPoint);
            const viewRect = vp.viewRect.clone();
            viewRect.scaleAboutCenter(0.25, 0.25);
            // istanbul ignore next hard to reach because of mocks
            if (viewRect.containsPoint(testPt))
                return this._lastTargetPoint;
            this._lastTargetPoint = undefined;
        }
        const visiblePoint = vp.pickNearestVisibleGeometry(vp.npcToWorld(imodeljs_common_1.NpcCenter), vp.pixelsFromInches(imodeljs_frontend_1.ToolSettings.viewToolPickRadiusInches));
        this._lastTargetPoint = (undefined !== visiblePoint ? visiblePoint : vp.view.getTargetPoint());
        return this._lastTargetPoint;
    }
    render() {
        const viewOverlay = this._vp && this.props.getViewOverlay ? this.props.getViewOverlay(this._vp) : null;
        const parentDivStyle = {
            height: "100%", width: "100%", position: "relative",
        };
        const viewportDivStyle = Object.assign({ height: "100%", width: "100%" }, this.props.style);
        return (React.createElement("div", { style: parentDivStyle },
            React.createElement("div", { ref: this._viewportDiv, "data-testid": "viewport-component", className: this.props.className, style: viewportDivStyle, onContextMenu: this._handleContextMenu }),
            viewOverlay));
    }
}
exports.ViewportComponent = ViewportComponent;
//# sourceMappingURL=ViewportComponent.js.map