"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Viewport
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewportComponentEvents = exports.ViewIdChangedEvent = exports.ViewClassFullNameChangedEvent = exports.ViewRotationChangeEvent = exports.StandardRotationChangeEvent = exports.CubeRotationChangeEvent = exports.DrawingViewportChangeEvent = void 0;
const geometry_core_1 = require("@bentley/geometry-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_core_1 = require("@bentley/ui-core");
/** Drawing View Change event
 * @public
 */
class DrawingViewportChangeEvent extends ui_core_1.UiEvent {
}
exports.DrawingViewportChangeEvent = DrawingViewportChangeEvent;
/** 3d Cube Rotation Change event
 * @public
 */
class CubeRotationChangeEvent extends ui_core_1.UiEvent {
}
exports.CubeRotationChangeEvent = CubeRotationChangeEvent;
/** Standard Rotation Change event
 * @public
 */
class StandardRotationChangeEvent extends ui_core_1.UiEvent {
}
exports.StandardRotationChangeEvent = StandardRotationChangeEvent;
/** View Rotation Change event
 * @public
 */
class ViewRotationChangeEvent extends ui_core_1.UiEvent {
}
exports.ViewRotationChangeEvent = ViewRotationChangeEvent;
/** View Class Full Name Change event
 * @public
 */
class ViewClassFullNameChangedEvent extends ui_core_1.UiEvent {
}
exports.ViewClassFullNameChangedEvent = ViewClassFullNameChangedEvent;
/** View Id Change event
 * @public
 */
class ViewIdChangedEvent extends ui_core_1.UiEvent {
}
exports.ViewIdChangedEvent = ViewIdChangedEvent;
/** Viewport Rotation events and methods
 * @public
 */
class ViewportComponentEvents {
    static initialize() {
        if (undefined !== this._removeListener)
            return;
        this._removeListener = imodeljs_frontend_1.IModelApp.viewManager.onSelectedViewportChanged.addListener(ViewportComponentEvents.handleSelectedViewportChanged);
    }
    /** @internal - for unit testing */
    static terminate() {
        if (this._removeListener) {
            this._removeListener();
            this._removeListener = undefined;
        }
    }
    static handleSelectedViewportChanged(args) {
        if (args.current)
            ViewportComponentEvents.setViewMatrix(args.current);
    }
    static setCubeMatrix(rotMatrix, face = ui_core_1.Face.None, complete = false) {
        this.rotationMatrix.setFrom(rotMatrix);
        this.face = face;
        this.onCubeRotationChangeEvent.emit({ rotMatrix, complete, face });
    }
    static setDrawingViewportState(origin, rotation, complete = false) {
        this.onDrawingViewportChangeEvent.emit({ origin, rotation, complete });
    }
    static setStandardRotation(standardRotation) {
        this.onStandardRotationChangeEvent.emit({ standardRotation });
    }
    static setViewMatrix(viewport, animationTime) {
        // When handling onViewChanged, use setTimeout
        setTimeout(() => {
            if (viewport.view) {
                this.origin.setFrom(viewport.view.getOrigin());
                this.extents.setFrom(viewport.view.getExtents());
            }
            this.rotationMatrix.setFrom(viewport.rotation);
            this.onViewRotationChangeEvent.emit({ viewport, animationTime });
        });
    }
}
exports.ViewportComponentEvents = ViewportComponentEvents;
ViewportComponentEvents.face = ui_core_1.Face.None;
ViewportComponentEvents.origin = geometry_core_1.Point3d.createZero();
ViewportComponentEvents.extents = geometry_core_1.Vector3d.createZero();
ViewportComponentEvents.rotationMatrix = geometry_core_1.Matrix3d.createIdentity();
ViewportComponentEvents.onDrawingViewportChangeEvent = new DrawingViewportChangeEvent();
ViewportComponentEvents.onCubeRotationChangeEvent = new CubeRotationChangeEvent();
ViewportComponentEvents.onStandardRotationChangeEvent = new StandardRotationChangeEvent();
ViewportComponentEvents.onViewRotationChangeEvent = new ViewRotationChangeEvent();
ViewportComponentEvents.onViewClassFullNameChangedEvent = new ViewClassFullNameChangedEvent();
ViewportComponentEvents.onViewIdChangedEvent = new ViewIdChangedEvent();
//# sourceMappingURL=ViewportComponentEvents.js.map